@@ -67,7 +67,7 @@ int main(int argc, char *argv[]) {
   if (!do_skip) do_ave = false;
 
   file = KstDataSource::loadSource(filename);
-  if (!file || !file->isValid() || file->frameCount() < 1) {
+  if (!file || !file->isValid() || file->isEmpty()) {
     fprintf(stderr, "d2asc error: file %s has no data\n", filename);
     exit(0);
   }
