@@ -67,7 +67,7 @@ bool AsciiSource::initFile() {
 KstObject::UpdateType AsciiSource::update(int u) {
   Q_UNUSED(u)
   bool forceUpdate = false;
-  static char *tmpbuf = 0L;
+  char tmpbuf[MAXBUFREADLEN];
   QFile file(_filename);
 
   if (!_valid) {
@@ -94,10 +94,6 @@ KstObject::UpdateType AsciiSource::update(int u) {
 
   _valid = true;
 
-  if (!tmpbuf) {
-    tmpbuf = new char[MAXBUFREADLEN];
-  }
-
   int bufstart, bufread;
   bool new_data = false;
   do {
@@ -143,13 +139,12 @@ KstObject::UpdateType AsciiSource::update(int u) {
 
   file.close();
 
+  updateNumFramesScalar();
   return forceUpdate ? KstObject::UPDATE : (new_data ? KstObject::UPDATE : KstObject::NO_CHANGE);
 }
 
 
 int AsciiSource::readField(double *v, const QString& field, int s, int n) {
-  bool ok;
-
   if (n < 0) {
     n = 1; /* n < 0 means read one sample, not frame - irrelavent here */
   }
@@ -161,6 +156,7 @@ int AsciiSource::readField(double *v, const QString& field, int s, int n) {
     return n;
   }
 
+  bool ok;
   int col = (int)field.toUInt(&ok);
   if (!ok) {
     return 0;
@@ -240,7 +236,8 @@ int AsciiSource::samplesPerFrame(const QString &field) {
 }
 
 
-int AsciiSource::frameCount() const {
+int AsciiSource::frameCount(const QString& field) const {
+  Q_UNUSED(field)
   return _numFrames;
 }
 
@@ -250,6 +247,11 @@ QString AsciiSource::fileType() const {
 }
 
 
+bool AsciiSource::isEmpty() const {
+  return _numFrames < 1;
+}
+
+
 // FIXME: refactor to cache the number of columns and just update it in
 //        ::update() as needed
 QStringList AsciiSource::fieldList() const {
@@ -304,7 +306,7 @@ QStringList provides_ascii() {
     is a number.  **/
 // FIXME: might it be better to be more lenient and ignore lines with
 //        ascii free text?
-bool understands_ascii(const QString& filename) {
+int understands_ascii(const QString& filename) {
   QFile f(filename);
 
   if (f.open(IO_ReadOnly)) {
@@ -322,13 +324,13 @@ bool understands_ascii(const QString& filename) {
         // comment; do nothing
       } else if (QRegExp("^[\\s]*(([Nn][Aa][Nn]|(\\-\\+)?[Ii][Nn][Ff]|[0-9\\+\\-\\.eE]+)[\\s]*)+").exactMatch(s)) {
         // a number - this may be an ascii file - assume that it is
-        return true;
+        return 75;
       } else {
-        return false;
+        return 20;
       }
     }
   }
-  return false;
+  return 0;
 }
 }
 
