@@ -37,7 +37,7 @@ public:
 
   virtual int samplesPerFrame(const QString &field);
 
-  virtual int frameCount() const;
+  virtual int frameCount(const QString& field = QString::null) const;
 
   virtual QString fileType() const;
 
@@ -45,6 +45,8 @@ public:
 
   virtual void save(QTextStream &ts);
 
+  virtual bool isEmpty() const;
+
 private:
   int *_rowIndex;
   int _numLinesAlloc;
