@@ -44,7 +44,7 @@ static struct {
 int recurse_level = 0;
 int first_time = 1;
 
-int DoField(struct FormatType *F, char *field_code,
+int DoField(struct FormatType *F, const char *field_code,
 	    int first_frame, int first_samp,
 	    int num_frames, int num_samp, 
 	    char return_type, void *data_out,
@@ -95,7 +95,7 @@ static void FreeF(struct FormatType *F) {
 /*                                                                         */
 /***************************************************************************/
 static void ParseRaw(char in_cols[15][MAX_LINE_LENGTH],
-	      int n_cols, struct RawEntryType *R, int *error_code) {
+		     struct RawEntryType *R, int *error_code) {
   strcpy(R->field, in_cols[0]); /* field */
   R->fp = -1; /* file not opened yet */
   switch (in_cols[2][0]) {
@@ -150,8 +150,7 @@ static void ParseLincom(char in_cols[15][MAX_LINE_LENGTH],
 /*                                                                         */
 /***************************************************************************/
 static void ParseLinterp(char in_cols[15][MAX_LINE_LENGTH],
-		  int n_cols, struct LinterpEntryType *L,
-		  int *error_code) {
+			 struct LinterpEntryType *L) {
   strcpy(L->field, in_cols[0]); /* field */
   strncpy(L->raw_field, in_cols[2], FIELD_LENGTH);
   strcpy(L->linterp_file, in_cols[3]);
@@ -190,8 +189,8 @@ static void ParseMplex(char in_cols[15][MAX_LINE_LENGTH],
 /*                                                                         */
 /***************************************************************************/
 static void ParseBit(char in_cols[15][MAX_LINE_LENGTH],
-		int n_cols, struct BitEntryType *B,
-		int *error_code) {
+		     struct BitEntryType *B,
+		     int *error_code) {
   strcpy(B->field, in_cols[0]); /* field */
   strncpy(B->raw_field, in_cols[2], FIELD_LENGTH); /* field */
 
@@ -303,7 +302,7 @@ struct FormatType *GetFormat(const char *filedir, int *error_code) {
       F->n_raw++;
       F->rawEntries =
 	realloc(F->rawEntries, F->n_raw*sizeof(struct RawEntryType));
-      ParseRaw(in_cols, n_cols, F->rawEntries+F->n_raw - 1, error_code);
+      ParseRaw(in_cols, F->rawEntries+F->n_raw - 1, error_code);
     } else if (strcmp(in_cols[1], "LINCOM")==0) {
       F->n_lincom++;
       F->lincomEntries =
@@ -316,8 +315,7 @@ struct FormatType *GetFormat(const char *filedir, int *error_code) {
       F->linterpEntries =
 	realloc(F->linterpEntries,
 		F->n_linterp*sizeof(struct LinterpEntryType));
-      ParseLinterp(in_cols, n_cols, F->linterpEntries+F->n_linterp - 1,
-		  error_code); 
+      ParseLinterp(in_cols, F->linterpEntries+F->n_linterp - 1); 
     } else if (strcmp(in_cols[1], "MPLEX")==0) {
       F->n_mplex++;
       F->mplexEntries =
@@ -330,7 +328,7 @@ struct FormatType *GetFormat(const char *filedir, int *error_code) {
       F->bitEntries =
 	realloc(F->bitEntries,
 		F->n_bit*sizeof(struct BitEntryType));
-      ParseBit(in_cols, n_cols, F->bitEntries+F->n_bit - 1,
+      ParseBit(in_cols, F->bitEntries+F->n_bit - 1,
 		  error_code); 
     } else {
       FreeF(F);
@@ -647,7 +645,7 @@ static int ConvertType(unsigned char *data_in, char in_type,
 /*   Get samples per frame for field, given FormatType *F                  */
 /*                                                                         */
 /***************************************************************************/
-static int GetSPF(char *field_code, struct FormatType *F, int *error_code) {
+static int GetSPF(const char *field_code, struct FormatType *F, int *error_code) {
   struct RawEntryType tR;
   struct RawEntryType *R;
   struct LincomEntryType tL;
@@ -739,7 +737,7 @@ static int GetSPF(char *field_code, struct FormatType *F, int *error_code) {
 /*   Look to see if the field code belongs to a raw.  If so, parse it.     */
 /*                                                                         */
 /***************************************************************************/
-static int DoIfRaw(struct FormatType *F, char *field_code,
+static int DoIfRaw(struct FormatType *F, const char *field_code,
 	    int first_frame, int first_samp,
 	    int num_frames, int num_samp, 
 	    char return_type, void *data_out,
@@ -956,7 +954,7 @@ static void AddData(void *A, void *B, char type, int n) {
 /*   Look to see if the field code belongs to a lincom.  If so, parse it.  */
 /*                                                                         */
 /***************************************************************************/
-static int DoIfLincom(struct FormatType *F, char *field_code,
+static int DoIfLincom(struct FormatType *F, const char *field_code,
 	    int first_frame, int first_samp,
 	    int num_frames, int num_samp, 
 	       char return_type, void *data_out,
@@ -1018,7 +1016,7 @@ static int DoIfLincom(struct FormatType *F, char *field_code,
 /*   Look to see if the field code belongs to a bitfield.  If so, parse it.*/
 /*                                                                         */
 /***************************************************************************/
-static int DoIfBit(struct FormatType *F, char *field_code,
+static int DoIfBit(struct FormatType *F, const char *field_code,
 	    int first_frame, int first_samp,
 	    int num_frames, int num_samp, 
 	    char return_type, void *data_out,
@@ -1076,6 +1074,15 @@ static int DoIfBit(struct FormatType *F, char *field_code,
 /*   ReadLinterpFile: Read in the linterp data for this field              */
 /*                                                                         */
 /***************************************************************************/
+void MakeDummyLinterp(struct LinterpEntryType *E); /* prototype => no warning... */
+void MakeDummyLinterp(struct LinterpEntryType *E) {
+  E->n_interp = 2;
+  E->x = (double *)malloc(2*sizeof(double));
+  E->y = (double *)malloc(2*sizeof(double));
+  E->x[0] = 0; E->y[0] = 0;
+  E->x[1] = 1; E->y[1] = 0;
+}
+
 static int ReadLinterpFile(struct LinterpEntryType *E) {
   FILE *fp;
   int i;
@@ -1083,6 +1090,7 @@ static int ReadLinterpFile(struct LinterpEntryType *E) {
 
   fp = fopen(E->linterp_file, "r");
   if (fp==NULL) {
+    MakeDummyLinterp(E);
     return (GD_E_OPEN_LINFILE);
   }
   
@@ -1092,6 +1100,7 @@ static int ReadLinterpFile(struct LinterpEntryType *E) {
     i++;
   }
   if (i<2) {
+    MakeDummyLinterp(E);
     return (GD_E_OPEN_LINFILE);
   }
   E->n_interp = i;
@@ -1195,7 +1204,7 @@ static void LinterpData(void *data, char type, int npts,
 /*   Look to see if the field code belongs to a bitfield.  If so, parse it.*/
 /*                                                                         */
 /***************************************************************************/
-static int DoIfLinterp(struct FormatType *F, char *field_code,
+static int DoIfLinterp(struct FormatType *F, const char *field_code,
 		int first_frame, int first_samp,
 		int num_frames, int num_samp, 
 		char return_type, void *data_out,
@@ -1235,7 +1244,7 @@ static int DoIfLinterp(struct FormatType *F, char *field_code,
 /*  DoField: Doing one field once F has been identified                    */
 /*                                                                         */
 /***************************************************************************/
-int DoField(struct FormatType *F, char *field_code,
+int DoField(struct FormatType *F, const char *field_code,
 	    int first_frame, int first_samp,
 	    int num_frames, int num_samp, 
 	    char return_type, void *data_out,
@@ -1354,8 +1363,10 @@ int GetNFrames(const char *filename_in, int *error_code, const char *in_field) {
   char raw_data_filename[MAX_FILENAME_LENGTH+FIELD_LENGTH+2];
   struct stat statbuf;
   int nf;
-  char *field;
-
+  char field_buf[80];
+  const char *field;
+  int i;
+  
   *error_code = GD_E_OK;
   
   if (first_time) {
@@ -1376,16 +1387,27 @@ int GetNFrames(const char *filename_in, int *error_code, const char *in_field) {
     return(0);
   }
 
-  if (in_field == NULL) field = F->rawEntries[0].field;
-  else field = in_field;
-    
+  if (in_field == NULL) {
+    /* check for first valid raw field */
+    for (i=0; i<F->n_raw; i++) {
+      sprintf(raw_data_filename,"%s/%s", filename, F->rawEntries[i].field);
+      if (stat(raw_data_filename, &statbuf) >=0) {
+        strncpy(field_buf, F->rawEntries[i].field, 79);
+        i = F->n_raw;
+      }
+    }
+    field = field_buf;
+  } else {
+    field = in_field;
+  }
+
   sprintf(raw_data_filename,"%s/%s", filename, field);
   if (stat(raw_data_filename, &statbuf) < 0) {
     return(0);
   }
 
   nf = statbuf.st_size/
-	 (F->rawEntries[0].size*F->rawEntries[0].samples_per_frame);
+    (F->rawEntries[0].size*F->rawEntries[0].samples_per_frame);
 
   nf -= 2;
   if (nf < 0)
@@ -1424,3 +1446,4 @@ int GetSamplesPerFrame(const char *filename_in, const char *field_name, int *err
 
   return (GetSPF(field_name, F, error_code));
 }
+/* vim: ts=2 sw=2 et */
