@@ -127,6 +127,7 @@ KstObject::UpdateType FrameSource::update(int u) {
   bool isnew = _frameCount != newN;
   _frameCount = newN;
 
+  updateNumFramesScalar();
   return isnew ? KstObject::UPDATE : KstObject::NO_CHANGE;
 }
 
@@ -174,7 +175,8 @@ int FrameSource::samplesPerFrame(const QString &field) {
 }
 
 
-int FrameSource::frameCount() const {
+int FrameSource::frameCount(const QString& field) const {
+  Q_UNUSED(field)
   return _frameCount;
 }
 
@@ -184,11 +186,21 @@ QString FrameSource::fileType() const {
 }
 
 
+bool FrameSource::isEmpty() const {
+  return _frameCount < 1;
+}
+
+
 void FrameSource::save(QTextStream &ts) {
   KstDataSource::save(ts);
 }
 
 
+bool FrameSource::fieldListIsComplete() const {
+  return false;
+}
+
+
 extern "C" {
 KstDataSource *create_frame(const QString& filename, const QString& type) {
   return new FrameSource(filename, type);
@@ -200,7 +212,7 @@ QStringList provides_frame() {
   return rc;
 }
 
-bool understands_frame(const QString& filename) {
+int understands_frame(const QString& filename) {
   int err = 0;
   CReadData(filename.latin1(), "INDEX",
             0, 0, /* 1st sframe, 1st samp */
@@ -208,10 +220,10 @@ bool understands_frame(const QString& filename) {
             'n', (void*)0L,
             &err);
   if (err == E_OK) {
-    return true;
+    return 98;
   }
 
-  return false;
+  return 0;
 }
 
 }
