@@ -68,8 +68,8 @@ int IndirectSource::samplesPerFrame(const QString &field) {
 }
 
 
-int IndirectSource::frameCount() const {
-  return _child->frameCount();
+int IndirectSource::frameCount(const QString& field) const {
+  return _child->frameCount(field);
 }
 
 
@@ -88,6 +88,11 @@ bool IndirectSource::isValid() const {
 }
 
 
+bool IndirectSource::isEmpty() const {
+  return _child->isEmpty();
+}
+
+
 extern "C" {
 KstDataSource *create_indirect(const QString& filename, const QString& type) {
   if (!type.isEmpty() && type != "Indirect") {
@@ -120,22 +125,22 @@ QStringList provides_indirect() {
   return rc;
 }
 
-bool understands_indirect(const QString& filename) {
+int understands_indirect(const QString& filename) {
   if (!filename.endsWith(".cur")) { // Do we really have to do this?
-    return false;
+    return 0;
   }
 
   QFile f(filename);
   if (!f.open(IO_ReadOnly)) {
-    return false;
+    return 0;
   }
 
   QString ifn;
   if (0 >= f.readLine(ifn, 1000)) {
-    return false;
+    return 0;
   }
 
-  return QFile::exists(ifn.stripWhiteSpace());
+  return QFile::exists(ifn.stripWhiteSpace()) ? 100 : 0;
 }
 
 }
