@@ -31,9 +31,10 @@ public:
   virtual int readField(double *v, const QString &field, int s, int n);
   virtual bool isValidField(const QString &field) const;
   virtual int samplesPerFrame(const QString &field);
-  virtual int frameCount() const;
+  virtual int frameCount(const QString& field = QString::null) const;
   virtual QString fileType() const;
   virtual void save(QTextStream &ts);
+  virtual bool isEmpty() const;
 
 private:
   bool        getColNumber( const QString& field, int* piColNumber ) const;
