@@ -47,6 +47,7 @@ PlanckSource::~PlanckSource() {
 KstObject::UpdateType PlanckSource::update(int u) {
   Q_UNUSED(u)
   if (_valid && _planckObject && _planckObject->updated()) {
+    updateNumFramesScalar();
     return KstObject::UPDATE;
   }
   return KstObject::NO_CHANGE;
@@ -108,13 +109,21 @@ int PlanckSource::samplesPerFrame(const QString &field) {
 }
 
 
-int PlanckSource::frameCount() const {
+int PlanckSource::frameCount(const QString& field) const {
   if (!_valid) {
     return 0;
   }
 
-  // FIXME: can't use the last field!!
-  QSize sz = _planckObject->range(_fieldList.last());
+  if (field.isEmpty() || _fieldList.contains(field)) {
+    int maxLen = 0;
+    for (QStringList::ConstIterator i = _fieldList.begin(); i != _fieldList.end(); ++i) {
+      QSize sz = _planckObject->range(*i);
+      maxLen = QMAX(sz.height() - sz.width() + 1, maxLen);
+    }
+    return maxLen;
+  }
+
+  QSize sz = _planckObject->range(field);
   return sz.height() - sz.width() + 1;
 }
 
@@ -129,6 +138,22 @@ void PlanckSource::save(QTextStream &ts) {
 }
 
 
+bool PlanckSource::isEmpty() const {
+  if (!_valid) {
+    return true;
+  }
+
+  for (QStringList::ConstIterator i = _fieldList.begin(); i != _fieldList.end(); ++i) {
+    QSize sz = _planckObject->range(*i);
+    if (sz.height() - sz.width() + 1 > 0) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
+
 extern "C" {
 KstDataSource *create_planck(const QString& filename, const QString& type) {
   Q_UNUSED(type)
@@ -141,10 +166,10 @@ QStringList provides_planck() {
   return rc;
 }
 
-bool understands_planck(const QString& filename) {
+int understands_planck(const QString& filename) {
   bool rc = Planck::validDatabase(filename);
   kdDebug() << "-> Valid database? " << rc << endl;
-  return rc;
+  return rc ? 100 : 0;
 }
 
 }
