@@ -162,6 +162,27 @@ bool Division::isConst() {
 }
 
 
+/////////////////////////////////////////////////////////////////
+Modulo::Modulo(Node *left, Node *right)
+: BinaryNode(left, right) {
+  //printf("%p: New Modulo: %p - %p\n", (void*)this, (void*)left, (void*)right);
+}
+
+
+Modulo::~Modulo() {
+}
+
+
+double Modulo::value(Context *ctx) {
+  return fmod(_left->value(ctx), _right->value(ctx));
+}
+
+
+bool Modulo::isConst() {
+  return _left->isConst() && _right->isConst();
+}
+
+
 /////////////////////////////////////////////////////////////////
 Power::Power(Node *left, Node *right)
 : BinaryNode(left, right) {
