@@ -17,17 +17,17 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <kmdimainfrm.h>
 
 // include files for Qt
 #include <qevent.h>
+#include <qeventloop.h>
 #include <qdir.h>
+#include <qobjectlist.h>
 #include <qpainter.h>
 #include <qpaintdevicemetrics.h>
-#include <qtimer.h>
+#include <qprogressbar.h>
 #include <qpushbutton.h>
-#include <qobjectlist.h>
-#include <qmessagebox.h>
+#include <qtimer.h>
 
 // include files for KDE
 #include <kaccel.h>
@@ -37,6 +37,7 @@
 #include <kiconloader.h>
 #include <kkeydialog.h>
 #include <klocale.h>
+#include <kmdimainfrm.h>
 #include <kmenubar.h>
 #include <kmessagebox.h>
 #include <kprinter.h>
@@ -48,7 +49,6 @@
 #include "datawizard.h"
 #include "extensiondlg.h"
 #include "extensionmgr.h"
-#include "filterlisteditor.h"
 #include "kst.h"
 #include "kstchangefiledialog_i.h"
 #include "kstchangenptsdialog_i.h"
@@ -62,10 +62,13 @@
 #include "ksteventmonitor_i.h"
 #include "kstextension.h"
 #include "kstfitdialog_i.h"
+#include "kstfilterdialog_i.h"
 #include "kstgraphfiledialog_i.h"
 #include "ksthsdialog_i.h"
 #include "kstiface_impl.h"
+#include "kstimagedialog_i.h"
 #include "kstlabeldialog_i.h"
+#include "kstmatrixdialog_i.h"
 #include "kstobject.h"
 #include "kstplotbase.h"
 #include "kstplotdialog_i.h"
@@ -81,10 +84,13 @@
 #include "kstviewwidget.h"
 #include "kstviewwindow.h"
 #include "pluginmanager.h"
+#include "psversion.h"
+#include "sysinfo.h"
 #include "updatethread.h"
 #include "vectorsavedialog.h"
 
-#define KST_STATUSBAR_DATA 1
+
+#define KST_STATUSBAR_DATA   1
 #define KST_STATUSBAR_STATUS 2
 
 
@@ -102,7 +108,7 @@ KstApp::KstApp(QWidget *parent, const char *name)
 
   clearWFlags(WDestructiveClose);
 
-  stopping = false;
+  _stopping = false;
   config = kapp->config();
   initStatusBar();
 
@@ -113,7 +119,6 @@ KstApp::KstApp(QWidget *parent, const char *name)
   debugDialog = new KstDebugDialogI(this);
   _plotDialog = new KstPlotDialogI(doc, this);
   dataManager = new KstDataManagerI(doc, this);
-  filterListEditor = new FilterListEditor(this);
   viewScalarsDialog = new KstViewScalarsDialogI(this);
   viewVectorsDialog = new KstViewVectorsDialogI(this);
   viewFitsDialog = new KstViewFitsDialogI(this);
@@ -123,6 +128,7 @@ KstApp::KstApp(QWidget *parent, const char *name)
   vectorSaveDialog = new VectorSaveDialog(this);
   _labelDialog = new KstLabelDialogI(this);
 
+  connect(_labelDialog, SIGNAL(applied()), doc, SLOT(wasModified()));
   connect(KstVectorDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstCurveDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstEqDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
@@ -130,8 +136,12 @@ KstApp::KstApp(QWidget *parent, const char *name)
   connect(KstPsdDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstPluginDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstFitDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
+  connect(KstFilterDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstEventMonitorI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
 
+  connect(KstImageDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
+  connect(KstMatrixDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
+
   initActions();
 
   readOptions();
@@ -150,9 +160,6 @@ KstApp::KstApp(QWidget *parent, const char *name)
   /*** plot dialog ***/
   connect(_plotDialog, SIGNAL(docChanged()), this, SLOT(registerDocChange()));
 
-  connect(filterListEditor, SIGNAL(docChanged()),
-          this,             SLOT(registerDocChange()));
-
   /*** ***/
   connect(doc, SIGNAL(updateDialogs()), this, SLOT(updateDialogs()));
   connect(doc, SIGNAL(dataChanged()), this, SLOT(updateDataDialogs()));
@@ -183,45 +190,38 @@ KstApp::KstApp(QWidget *parent, const char *name)
   checkFontPresent("Symbol");
 
   if (!isFakingSDIApplication()) {
-    QString strMenuTitle;
-    bool bAddedWindowMenu = false;
-    int iMenuCount;
-    int iMenuIndex;
-    int iMenuId;
-
-    iMenuCount = menuBar()->count();
-    for (iMenuIndex = 0; iMenuIndex < iMenuCount; iMenuIndex++) {
-      iMenuId = menuBar()->idAt(iMenuIndex);
-      if (iMenuId != -1) {
-        strMenuTitle =  menuBar()->text(iMenuId);
-        if (strMenuTitle.compare(i18n("&Help")) == 0) {
-          menuBar()->insertItem(i18n("&Window"), windowMenu(), 100, iMenuIndex);
-          bAddedWindowMenu = true;
+    bool addedWindowMenu = false;
+    int menuCount = menuBar()->count();
+    for (int menuIndex = 0; menuIndex < menuCount; menuIndex++) {
+      int menuId = menuBar()->idAt(menuIndex);
+      if (menuId != -1) {
+        const QString& menuTitle = menuBar()->text(menuId);
+        if (menuTitle == i18n("&Help")) {
+          menuBar()->insertItem(i18n("&Window"), windowMenu(), 100, menuIndex);
+          addedWindowMenu = true;
           break;
         }
       }
     }
 
-    if (!bAddedWindowMenu) {
-      menuBar()->insertItem( i18n("&Window"), windowMenu());
+    if (!addedWindowMenu) {
+      menuBar()->insertItem(i18n("&Window"), windowMenu());
     }
   }
 }
 
-void KstApp::checkFontPresent( const QString& strFont ) {
-  QFont fontSymbol( strFont );
-  QFontInfo info( fontSymbol );
-  QString strWarning;
-
-  if (info.family().lower().compare(strFont.lower()) != 0) {
-    strWarning = i18n("The %1 font was not found and was replaced by the %2 font; as a result, some labels may not display correctly.").arg(strFont).arg(info.family());
-    KstDebug::self()->log(strWarning, KstDebug::Warning);
+void KstApp::checkFontPresent(const QString& font) {
+  QFont f(font);
+  QFontInfo info(f);
+  if (info.family().lower() != font.lower()) {
+    QString msg = i18n("The %1 font was not found and was replaced by the %2 font; as a result, some labels may not display correctly.").arg(font).arg(info.family());
+    KstDebug::self()->log(msg, KstDebug::Warning);
   }
 }
 
 
-void KstApp::EventELOGSubmitEntry(const QString& strMessage) {
-  emit ELOGSubmitEntry(strMessage);
+void KstApp::EventELOGSubmitEntry(const QString& msg) {
+  emit ELOGSubmitEntry(msg);
 }
 
 
@@ -233,10 +233,7 @@ void KstApp::EventELOGConfigure() {
 void KstApp::customEvent(QCustomEvent *pEvent) {
   if(pEvent->type() == KstELOGDebugInfoEvent) {
     QTextStream *pTextStream = (QTextStream*)pEvent->data();
-    QString str;
-
-    KstDebug::self()->getText(str);
-    (*pTextStream) << str;
+    (*pTextStream) << KstDebug::self()->text();
   } else if (pEvent->type() == KstELOGConfigureEvent) {
     QTextStream *pTextStream = (QTextStream*)pEvent->data();
     document()->saveDocument(*pTextStream);
@@ -280,8 +277,10 @@ void KstApp::initActions() {
                               this, SLOT(slotFileNewWindow()),
                               actionCollection(),"file_new_window");
 
+#if 0
   fileOpenNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
   fileOpenNew->setWhatsThis(i18n("Open Kst plot file."));
+#endif
 
   fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
   fileSave->setWhatsThis(i18n("Save to current Kst plot file."));
@@ -289,8 +288,8 @@ void KstApp::initActions() {
   fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
   fileSaveAs->setWhatsThis(i18n("Save to new Kst plot file."));
 
-  fileClose = KStdAction::close(this, SLOT(closeActiveView()), actionCollection());
-  fileClose->setWhatsThis(i18n("Close Kst."));
+  fileClose = KStdAction::close(this, SLOT(slotFileNew()), actionCollection());
+  fileClose->setWhatsThis(i18n("Close current Kst plot file."));
 
   fileQuit = KStdAction::quit(this, SLOT(slotFileClose()), actionCollection());
   fileQuit->setWhatsThis(i18n("Quit Kst."));
@@ -383,15 +382,6 @@ void KstApp::initActions() {
   LayoutAction->setToolTip(i18n("Layout mode"));
   LayoutAction->setWhatsThis(i18n("Use this mode to move, resize, and group plots."));
 
-  /************/
-#if 0
-  FilterDialogAction = new KAction(i18n("Edit &Filter..."), 0, 0,
-                                 this, SLOT(showFilterListEditor()),
-                                 actionCollection(), "filterdialog_action");
-  FilterDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
-                                        "to edit filters."));
-#endif
-
   /************/
   PlotDialogAction = new KAction(i18n("Edit &Plot..."), "kst_editplots", 0,
                                  this, SLOT(showPlotDialog()),
@@ -452,7 +442,23 @@ void KstApp::initActions() {
                                    SLOT(show_New()), actionCollection(),
                                    "plugindialog_action");
   PluginDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
-                                        "to crate a new plugin instance."));
+                                        "to create a new plugin instance."));
+
+  /************/
+  MatrixDialogAction = new KAction(i18n("New M&atrix..."), 0, 0,
+                                   KstMatrixDialogI::globalInstance(),
+                                   SLOT(show_New()), actionCollection(),
+                                   "matrixdialog_action");
+  MatrixDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
+                                        "to create a new matrix."));
+
+  /************/
+  ImageDialogAction = new KAction(i18n("New &Image..."), 0, 0,
+                                   KstImageDialogI::globalInstance(),
+                                   SLOT(show_New()), actionCollection(),
+                                   "imagedialog_action");
+  ImageDialogAction->setWhatsThis(i18n("Bring up a dialog box\n"
+                                        "to create a new image instance."));
 
   /************/
   ChangeFileDialogAction = new KAction(i18n("Change Data &File..."),
@@ -650,10 +656,28 @@ bool KstApp::getTieZoom() {
 }
 
 void KstApp::initStatusBar() {
-  statusBar()->insertItem(i18n("Almost Ready"), KST_STATUSBAR_STATUS);
-  _dataBar = new QLabel("(00000000, 00000000)", statusBar());
+  _dataBar = new QLabel(QString::null, statusBar());
   statusBar()->addWidget(_dataBar, 0, true);
 
+#ifdef HAVE_LINUX
+  _memoryBar = new QLabel(i18n("0 MB available"), statusBar());
+  statusBar()->addWidget(_memoryBar, 0, true);
+  connect(&_memTimer, SIGNAL(timeout()), this, SLOT(updateMemoryStatus()));
+  _memTimer.start(5000);
+#endif
+
+  _progressMsg = new QLabel(QString::null, statusBar());
+  statusBar()->addWidget(_progressMsg, 0, true);
+
+  _progressBar = new QProgressBar(statusBar());
+  _progressBar->setPercentageVisible(false);
+  _progressBar->setCenterIndicator(true);
+  statusBar()->addWidget(_progressBar, 0, true);
+  _progressBar->hide();
+  _progressMsg->hide();
+
+  statusBar()->insertItem(i18n("Almost Ready"), KST_STATUSBAR_STATUS);
+
   statusBar()->show();
   slotUpdateStatusMsg(i18n("Ready"));
 }
@@ -774,7 +798,14 @@ void KstApp::readProperties(KConfig* config) {
 }
 
 bool KstApp::queryClose() {
-  return doc->saveModified();
+  if (doc->saveModified()) {
+     doc->cancelUpdate();
+    _stopping = true;
+    kapp->processEvents(); // danger
+    doc->deleteContents();
+    return true;
+  }
+  return false;
 }
 
 bool KstApp::queryExit() {
@@ -834,7 +865,7 @@ void KstApp::slotFileSave() {
   }
 }
 
-void KstApp::slotFileSaveAs() {
+bool KstApp::slotFileSaveAs() {
   slotUpdateStatusMsg(i18n("Saving file with a new filename..."));
 
   QString newName=
@@ -851,14 +882,18 @@ void KstApp::slotFileSaveAs() {
 
       setCaption(kapp->caption() + ": " + doc->title());
     }
+  } else {
+    slotUpdateStatusMsg(i18n("Ready"));
+    return false;
   }
   slotUpdateStatusMsg(i18n("Ready"));
+  return true;
 }
 
 void KstApp::slotFileClose() {
   if (doc->saveModified()) {
-    doc->cancelUpdate();
-    stopping = true;
+     doc->cancelUpdate();
+    _stopping = true;
     kapp->processEvents(); // danger
     doc->deleteContents();
     close();
@@ -921,7 +956,6 @@ void KstApp::slotFilePrint() {
 
   if (iNumPages > 0) {
     KPrinter printer;
-    bool bFirstPage = true;
     printer.setPageSize(KPrinter::Letter);
     printer.setOrientation(KPrinter::Landscape);
     printer.setFromTo(0, 0);
@@ -932,10 +966,8 @@ void KstApp::slotFilePrint() {
       QDateTime dateTime = QDateTime::currentDateTime();
       QPainter paint(&printer);
       QPaintDeviceMetrics metrics(&printer);
-      QString strTitle;
       QRect rect;
       QSize size(metrics.width(), metrics.height());
-      QSize sizeOld;
       int iFromPage = printer.fromPage();
       int iToPage = printer.toPage();
 
@@ -949,27 +981,25 @@ void KstApp::slotFilePrint() {
 
       pIterator = createIterator();
       if (pIterator) {
+        bool firstPage = true;
         while (pIterator->currentItem()) {
           pView = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
           if (pView && !pView->view()->children().isEmpty()) {
             iNumPages++;
             if ((iFromPage == 0 && iToPage == 0) ||
                 (iFromPage <= iNumPages && iToPage >= iNumPages)) {
-              if (!bFirstPage) {
-                if (!printer.newPage()) {
-                  break;
-                }
+              if (!firstPage && !printer.newPage()) {
+                break;
               }
 
-              strTitle = i18n("Page: %1  Name: %2  Date: %3").arg(iNumPages).arg(pView->caption()).arg(dateTime.toString(Qt::ISODate));
+              QString title = i18n("Page: %1  Name: %2  Date: %3").arg(iNumPages).arg(pView->caption()).arg(dateTime.toString(Qt::ISODate));
 
-              sizeOld = pView->view()->size();
-              pView->view()->resize(size);
-              paint.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, strTitle);
+              pView->view()->resizeForPrint(size);
+              paint.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
               pView->view()->paint(P_PRINT, paint);
-              pView->view()->resize(sizeOld);
+              pView->view()->revertForPrint();
 
-              bFirstPage = false;
+              firstPage = false;
             }
           }
           pIterator->next();
@@ -985,7 +1015,6 @@ void KstApp::slotFilePrint() {
 }
 
 void KstApp::immediatePrintToFile(const QString& filename) {
-  KstViewWindow *pView;
   KPrinter printer;
 
   printer.setPageSize(KPrinter::Letter);
@@ -996,14 +1025,12 @@ void KstApp::immediatePrintToFile(const QString& filename) {
 
   KMdiIterator<KMdiChildView*> *pIterator = createIterator();
   if (pIterator) {
-    bool bFirstPage = true;
+    bool firstPage = true;
     QDateTime dateTime = QDateTime::currentDateTime();
     QPainter paint(&printer);
     QPaintDeviceMetrics metrics(&printer);
-    QString strTitle;
     QRect rect;
     QSize size(metrics.width(), metrics.height());
-    QSize sizeOld;
 
     rect.setLeft(0);
     rect.setRight(size.height());
@@ -1012,23 +1039,20 @@ void KstApp::immediatePrintToFile(const QString& filename) {
     rect.setTop(size.height());
 
     while (pIterator->currentItem()) {
-      pView = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
-      if (pView && !pView->view()->children().isEmpty()) {
-        if (!bFirstPage) {
-          if (!printer.newPage()) {
-            break;
-          }
+      KstViewWindow *view = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
+      if (view && !view->view()->children().isEmpty()) {
+        if (!firstPage && !printer.newPage()) {
+          break;
         }
 
-        strTitle = i18n("Name: %1  Date: %2").arg(pView->caption()).arg(dateTime.toString(Qt::ISODate));
+        QString title = i18n("Name: %1  Date: %2").arg(view->caption()).arg(dateTime.toString(Qt::ISODate));
 
-        sizeOld = pView->view()->size();
-        pView->view()->resize(size);
-        paint.drawText( rect, Qt::AlignLeft | Qt::AlignVCenter, strTitle );
-        pView->view()->paint(P_PRINT, paint);
-        pView->view()->resize(sizeOld);
+        view->view()->resizeForPrint(size);
+        paint.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
+        view->view()->paint(P_PRINT, paint);
+        view->view()->revertForPrint();
 
-        bFirstPage = false;
+        firstPage = false;
       }
       pIterator->next();
     }
@@ -1103,6 +1127,34 @@ void KstApp::slotUpdateDataMsg(const QString& msg) {
 }
 
 
+void KstApp::slotUpdateProgress(int total, int step, const QString &msg) {
+  if (step > 0) {
+    _progressBar->show();
+    if (!_progressBar->percentageVisible()) {
+      _progressBar->setPercentageVisible(true);
+    }
+    if (total != _progressBar->totalSteps()) {
+      _progressBar->setTotalSteps(total);
+    }
+    _progressBar->setProgress(step);
+  } else {
+    _progressBar->setPercentageVisible(false);
+    _progressBar->reset();
+    _progressBar->hide();
+  }
+
+  if (msg.isEmpty()) {
+    _progressMsg->setText(QString::null);
+    _progressMsg->hide();
+  } else {
+    _progressMsg->setText(msg);
+    _progressMsg->show();
+  }
+
+  while (kapp->eventLoop()->processEvents(QEventLoop::ExcludeUserInput));
+}
+
+
 bool KstApp::dataMode() const {
   return DataMode->isChecked();
 }
@@ -1141,17 +1193,17 @@ void KstApp::tieAll() {
     (*i)->setTied(tied <= 0);
   }
 
-  paintAll();
+  paintAll(P_PAINT);
 }
 
 
-void KstApp::paintAll() {
+void KstApp::paintAll(KstPaintType pt) {
   KMdiIterator<KMdiChildView*> *pIterator = createIterator();
   if (pIterator) {
     while (pIterator->currentItem()) {
       KstViewWindow *pView = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
       if (pView) {
-        pView->view()->paint(P_DATA);
+        pView->view()->paint(pt);
       }
       pIterator->next();
     }
@@ -1201,10 +1253,6 @@ void KstApp::showGraphFileDialog() {
   graphFileDialog->showGraphFileDialog();
 }
 
-void KstApp::showFilterListEditor() {
-  filterListEditor->show();
-}
-
 void KstApp::showDebugDialog() {
   debugDialog->show_I();
 }
@@ -1234,23 +1282,27 @@ void KstApp::updateDataDialogs(bool dm) {
   if (dm) {
     dataManager->updateContents();
   }
+
+  updateMemoryStatus();
 }
 
 
 void KstApp::updateDialogs() {
-  if (!stopping) {
+  if (!_stopping) {
     KstVectorDialogI::globalInstance()->update();
     KstPluginDialogI::globalInstance()->updateForm();
     KstFitDialogI::globalInstance()->update();
+    KstFilterDialogI::globalInstance()->update();
     KstEqDialogI::globalInstance()->update();
     KstHsDialogI::globalInstance()->update();
     KstPsdDialogI::globalInstance()->update();
     KstCurveDialogI::globalInstance()->update();
     KstEventMonitorI::globalInstance()->update();
+    KstImageDialogI::globalInstance()->update();
+    KstMatrixDialogI::globalInstance()->update();
     _plotDialog->update();
     changeFileDialog->updateChangeFileDialog();
     changeNptsDialog->updateChangeNptsDialog();
-    filterListEditor->update();
     updateDataDialogs(false);
     vectorSaveDialog->init();
     dataManager->update();
@@ -1259,11 +1311,13 @@ void KstApp::updateDialogs() {
 
 
 void KstApp::updateDialogsForWindow() {
-  if (!stopping) {
+  if (!_stopping) {
     KstEqDialogI::globalInstance()->updateWindow();
     KstHsDialogI::globalInstance()->updateWindow();
     KstPsdDialogI::globalInstance()->updateWindow();
     KstCurveDialogI::globalInstance()->updateWindow();
+    KstImageDialogI::globalInstance()->updateWindow();
+    dataManager->update();
     _plotDialog->updateWindow();
   }
 }
@@ -1308,7 +1362,7 @@ void KstApp::registerDocChange() {
 void KstApp::reload() {
   KstReadLocker ml(&KST::vectorList.lock());
   for (KstVectorList::Iterator i = KST::vectorList.begin(); i != KST::vectorList.end(); ++i) {
-    KstRVector *r = dynamic_cast<KstRVector*>((*i).data());
+    KstRVectorPtr r = dynamic_cast<KstRVector*>((*i).data());
     if (r) {
       r->reload();
     }
@@ -1362,9 +1416,7 @@ QString KstApp::newWindow(const QString& name_in) {
   w->setTabCaption(nameToUse);
   addWindow(w);
   w->activate();
-
   updateDialogsForWindow();
-
   return nameToUse;
 }
 
@@ -1427,5 +1479,15 @@ void KstApp::fromEnd() {
   setPaused(false);
 }
 
+
+void KstApp::updateMemoryStatus() {
+#ifdef HAVE_LINUX
+  meminfo();
+  unsigned long mi = S(kb_main_free + kb_main_buffers + kb_main_cached);
+  _memoryBar->setText(i18n("%1 MB available").arg(mi / (1024 * 1024)));
+#endif
+}
+
+
 #include "kst.moc"
 // vim: ts=2 sw=2 et
