@@ -24,7 +24,7 @@
 
 class KstBackBuffer {
   public:
-    KstBackBuffer();
+    KstBackBuffer(int depth = -1);
     ~KstBackBuffer();
 
     QSize size() const { return _buffer.size(); }
