@@ -60,7 +60,7 @@ int KstBaseCurve::getIndexNearX(double x) {
     // the common case.  It will be correct - just slightly slower...
     while (i_bot + 1 < i_top) {
       i0 = (i_top + i_bot)/2;
-      getPoint(i0, rX, rY);
+      point(i0, rX, rY);
       if (x < rX) {
         i_top = i0;
       } else {
@@ -68,8 +68,8 @@ int KstBaseCurve::getIndexNearX(double x) {
       }
     }
     double xt,xb;
-    getPoint(i_top, xt, rY);
-    getPoint(i_bot, xb, rY);
+    point(i_top, xt, rY);
+    point(i_bot, xb, rY);
     if (xt - x < x - xb) {
       return(i_top);
     } else {
@@ -82,12 +82,12 @@ int KstBaseCurve::getIndexNearX(double x) {
     double dx, dx0, rX, rY;
     n = sampleCount();
 
-    getPoint(0, rX, rY);
+    point(0, rX, rY);
     dx0 = fabs(x-rX);
     i0 = 0;
 
     for (i=1; i<n; i++) {
-      getPoint(i, rX, rY);
+      point(i, rX, rY);
       dx = fabs(x-rX);
       if (dx < dx0) {
         dx0 = dx;
@@ -100,26 +100,25 @@ int KstBaseCurve::getIndexNearX(double x) {
 
 /** getIndexNearXY: return index of point within (or closest too)
     x +- dx which is closest to y **/
-int KstBaseCurve::getIndexNearXY(double x, double dx, double y) {
-  int i,i0, iN, sc, index;
-  double xi, yi, dy, dyi;
-
-  sc = sampleCount();
+int KstBaseCurve::getIndexNearXY(double x, double dx_per_pix, double y) {
+  double xi, yi, dx, dxi, dy, dyi;
+  bool bFirst = true;
+  int i,i0, iN, index;
+  int sc = sampleCount();
 
   if (xIsRising()) {
-
     iN = i0 = getIndexNearX(x);
-    getPoint(i0, xi, yi);
+    point(i0, xi, yi);
 
     while ((i0>0) && (x-dx<xi)) {
       i0--;
-      getPoint(i0, xi, yi);
+      point(i0, xi, yi);
     }
-    getPoint(iN, xi, yi);
+    point(iN, xi, yi);
 
     while ((iN<sc-1) && (x+dx>xi)) {
       iN++;
-      getPoint(iN, xi, yi);
+      point(iN, xi, yi);
     }
   } else {
     i0 = 0;
@@ -127,17 +126,24 @@ int KstBaseCurve::getIndexNearXY(double x, double dx, double y) {
   }
 
   index = i0;
-  getPoint(index, xi, yi);
+  point(index, xi, yi);
+  dx = fabs(x-xi);
   dy = fabs(y-yi);
 
   for (i=i0+1; i<=iN; i++) {
-    getPoint(i, xi, yi);
-    if (fabs(x-xi)<dx) {
+    point(i, xi, yi);
+    dxi = fabs(x-xi);
+    if (dxi < dx_per_pix) {
+      dx = dxi;
       dyi = fabs(y-yi);
-      if (dyi<dy) {
-        dy = dyi;
+      if (bFirst || dyi < dy) {
+        bFirst = false;
         index = i;
+        dy = dyi;
       }
+    } else if(dxi < dx) {
+      dx = dxi;
+      index = i;
     }
   }
   return(index);
