@@ -14,29 +14,29 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-#include <qwidget.h>
+#include <qcombobox.h>
 #include <qfontdatabase.h>
-#include <qstring.h>
 #include <qlistbox.h>
 #include <qpushbutton.h>
-#include <qcombobox.h>
+#include <qstring.h>
+#include <qwidget.h>
 
-#include <kurl.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kurl.h>
 #include <kurlrequester.h>
 
+#include "kst.h"
 #include "kstchangefiledialog_i.h"
-#include "kstrvector.h"
 #include "kstdatacollection.h"
+#include "kstrvector.h"
 #include "kstvectordefaults.h"
 
 KstChangeFileDialogI::KstChangeFileDialogI(QWidget* parent,
                                            const char* name,
                                            bool modal,
-                                           WFlags fl) :
-  KstChangeFileDialog(parent, name, modal, fl) {
-
+                                           WFlags fl)
+: KstChangeFileDialog(parent, name, modal, fl) {
 
     connect(ChangeFileClear,     SIGNAL(clicked()),
             ChangeFileCurveList, SLOT(clearSelection()));
@@ -54,22 +54,20 @@ void KstChangeFileDialogI::selectAll() {
 }
 
 void KstChangeFileDialogI::updateChangeFileDialog() {
-  int i;
-
   ChangeFileCurveList->clear();
 
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   /* insert vectors into ChangeFileCurveList */
-  for (i = 0; i < (int)rvl.count(); i++) {
+  for (int i = 0; i < (int)rvl.count(); i++) {
+    rvl[i]->readLock();
     ChangeFileCurveList->insertItem(rvl[i]->tagName(), -1);
+    rvl[i]->readUnlock();
   }
 
   _dataFile->setURL(KST::vectorDefaults.dataSource());
-
 }
 
 void KstChangeFileDialogI::showChangeFileDialog() {
-
   updateChangeFileDialog();
 
   show();
@@ -78,7 +76,6 @@ void KstChangeFileDialogI::showChangeFileDialog() {
 
 void KstChangeFileDialogI::applyFileChange() {
   KstDataSourcePtr file;
-  KstRVectorPtr vector;
   KstWriteLocker ml(&KST::dataSourceList.lock());
 
   KstDataSourceList::Iterator it = KST::dataSourceList.findFileName(_dataFile->url());
@@ -89,7 +86,7 @@ void KstChangeFileDialogI::applyFileChange() {
       KMessageBox::sorry(0L, i18n("The file could not be loaded."));
       return;
     }
-    if (file->frameCount() < 1) {
+    if (file->isEmpty()) {
       KMessageBox::sorry(0L, i18n("The file does not contain data."));
       return;
     }
@@ -98,20 +95,28 @@ void KstChangeFileDialogI::applyFileChange() {
     file = *it;
   }
 
+  file->writeLock();
+
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (int i = 0; i < (int)ChangeFileCurveList->count(); i++) {
     if (ChangeFileCurveList->isSelected(i)) {
-      vector = rvl[i];
-      if (!file->isValidField(vector->getField())) {
-        KMessageBox::sorry(0L, i18n("%1: Field is not defined for the requested file.").arg(vector->getField()));
+      KstRVectorPtr vector = rvl[i];
+      vector->writeLock();
+      if (!file->isValidField(vector->field())) {
+        KMessageBox::sorry(0L, i18n("%1: Field is not defined for the requested file.").arg(vector->field()));
       } else {
         vector->changeFile(file);
       }
+      vector->writeUnlock();
     }
   }
 
+  file->writeUnlock();
+  file = 0L;
   emit docChanged();
+  // Force an update in case we're in paused mode
+  KstApp::inst()->forceUpdate();
 }
 
 #include "kstchangefiledialog_i.moc"
-
+// vim: ts=2 sw=2 et
