@@ -34,7 +34,7 @@ static const char *const colors[colorcnt] = { "red",
 static KStaticDeleter<KstColorSequence> sdColorSequence;
 
 QColor KstColorSequence::next() {
-  bool shift = false;
+  int dark_factor;
 
   if (!_self) {
     _self = sdColorSequence.setObject(_self, new KstColorSequence);
@@ -42,30 +42,47 @@ QColor KstColorSequence::next() {
 
   if (_self->_ptr >= _self->_count * 2) {
     _self->_ptr = 0;
-  } else if (_self->_ptr >= _self->_count) {
-    shift = true;
   }
+  dark_factor = 100 + 50* _self->_ptr/_self->_count;
 
-  if (shift) {
-    return _self->_pal->color(_self->_ptr++ % _self->_count).dark();
-  }
-
-return _self->_pal->color(_self->_ptr++);
+  return _self->_pal->color(_self->_ptr++ % _self->_count).dark(dark_factor);
 }
 
 QColor KstColorSequence::next(const QColor& badColor) {
-  QColor suggestedColor = _self->next();
+  QColor suggestedColor;
   int sugH, sugS, sugV;
   int badH, badS, badV;
   suggestedColor.getHsv(sugH,sugS,sugV);
   badColor.getHsv(badH, badS, badV);
+  double r1, h1, f1, x1, y1, z1;
+  double r2, h2, f2, x2, y2, z2;
+  double dc;
 
-  while (1 - 1 / (sqrt((sugS*sin(sugH)-badS*sin(badH))*(sugS*sin(sugH)-badS*sin(badH)) +
-                       (sugS*cos(sugH)-badS*cos(badH))*(sugS*cos(sugH)-badS*cos(badH)) +
-                       (sugV-badV)*(sugV-badV))) < 0.8) {
+  // make sure that the new color is not close to badColor.
+  // to do this imagine HSV as defining a cone.
+  // The distance from the point of the cone is R = V / 255
+  // Angle of rotational symetry is Theta = H * 2PI/360
+  // The 2nd angle is phi = S*(PI/4)/255
+  // a color is acceptable if |C1-C2|>dcMin
+
+  r1 = badV/255.0;
+  h1 = badH*M_PI/180.0;
+  f1 = badS*M_PI/4.0/255.0;
+  x1 = r1*sin( h1 )*sin( f1 );
+  y1 = r1*cos( h1 )*sin( f1 );
+  z1 = r1*cos( f1 );
+  do {
     suggestedColor = _self->next();
     suggestedColor.getHsv(sugH,sugS,sugV);
-  }
+    r2 = sugV/255.0;
+    h2 = sugH*M_PI/180.0;
+    f2 = sugS*M_PI/4.0/255.0;
+    x2 = r2*sin( h2 )*sin( f2 );
+    y2 = r2*cos( h2 )*sin( f2 );
+    z2 = r2*cos( f2 );
+    dc = sqrt( ( x1-x2 )*( x1-x2 ) + ( y1-y2 )*( y1-y2 ) + ( z1-z2 )*( z1-z2 ) );
+  } while ( dc < 0.3 );
+
   return QColor(suggestedColor);
 }
 
@@ -76,7 +93,7 @@ KstColorSequence::KstColorSequence() : _ptr(0) {
     for (int i = 0; i < colorcnt; i++) {
       _pal->addColor(QColor(colors[i]));
     }
-  } 
+  }
 
   _count = _pal->nrColors();
 }
