@@ -33,6 +33,7 @@ bool KST::dataTagNameNotUnique(const QString &tag, bool warn, void *p) {
   }
 
   /* verify that the tag name is not used by a data object */
+  KstReadLocker ml(&KST::dataObjectList.lock());
   if (KST::dataObjectList.findTag(tag) != KST::dataObjectList.end()) {
       if (warn) {
         KMessageBox::sorry(static_cast<QWidget*>(p), i18n("%1: this name is already in use. Change it to a unique name.").arg(tag));
@@ -53,6 +54,8 @@ bool KST::vectorTagNameNotUnique(const QString &tag, bool warn, void *p) {
   }
 
   /* verify that the tag name is not used by a data object */
+  KstReadLocker ml(&KST::vectorList.lock());
+  KstReadLocker ml2(&KST::scalarList.lock());
   if (KST::vectorList.findTag(tag) != KST::vectorList.end() ||
       KST::scalarList.findTag(tag) != KST::scalarList.end()) {
       if (warn) {
@@ -82,7 +85,7 @@ int KST::vectorToFile(KstVectorPtr v, QFile *f, void *p) {
   modval = QMAX(_size/100, 100);
 
   for (int i = 0; i < _size; i++) {
-    int l = snprintf(buf, BSIZE, "%d %g\n", i, _v[i]);
+    int l = snprintf(buf, BSIZE, "%g\n", _v[i]);
     f->writeBlock(buf, l);
     if (i % 100 == 0) {
       kpd->progressBar()->setProgress(i);
