@@ -20,7 +20,6 @@
 
 #include "kstvector.h"
 #include "kstscalar.h"
-#include "kstfilter.h"
 #include "kstdataobject.h"
 #include "kstdatasource.h"
 
@@ -39,22 +38,22 @@ namespace KST {
   /** The list of data objects which are in use */
   extern KstDataObjectList dataObjectList;
 
-  /** The list of filter sets that are defined */
-  extern KstFilterSetList filterSetList;
-
   /** check that a tag has not been used by any other tags */
   extern bool tagNameNotUnique(const QString& tag, bool warn = true, void *parent = 0L);
   extern bool dataTagNameNotUnique(const QString& tag, bool warn = true, void *parent = 0L);
   extern bool vectorTagNameNotUnique(const QString& tag, bool warn = true, void *parent = 0L);
-
-  extern QString suggestPlotName();
-
-  extern QString suggestWinName();
+  extern bool vectorTagNameNotUniqueInternal(const QString& tag);
 
   /** Save a vector to a file */
   extern int vectorToFile(KstVectorPtr v, QFile *f, void *parent = 0L);
 
   extern void addVectorToList(KstVectorPtr v);
+
+  /** Bad choice for location - maybe move it later */
+  void *malloc(size_t size);
+  void *realloc(void *ptr, size_t size);
+
 }
 
 #endif
+// vim: ts=2 sw=2 et
