@@ -35,6 +35,7 @@
 #include <kpopupmenu.h>
 
 // application specific includes
+#include "imagedialog.h"
 #include "kst.h"
 #include "kst2dplot.h"
 #include "kstcurvedialog_i.h"
@@ -43,8 +44,11 @@
 #include "kstdoc.h"
 #include "ksteqdialog_i.h"
 #include "ksteventmonitor_i.h"
-#include "kstfilteredvector.h"
 #include "ksthsdialog_i.h"
+#include "kstimage.h"
+#include "kstimagedialog_i.h"
+#include "kstmatrix.h"
+#include "kstmatrixdialog_i.h"
 #include "kstplotdialog_i.h"
 #include "kstplugin.h"
 #include "kstplugindialog_i.h"
@@ -52,6 +56,7 @@
 #include "kstrvector.h"
 #include "kstvectordialog_i.h"
 #include "kstviewwindow.h"
+#include "matrixselector.h"
 #include "vectorselector.h"
 
 
@@ -61,18 +66,8 @@ static QMap<int,QString> FilterMap;
 #define RTTI_OBJ_VECTOR          4201
 #define RTTI_OBJ_OBJECT          4202
 #define RTTI_OBJ_DATA_VECTOR     4203
-#define RTTI_OBJ_FILTERED_VECTOR 4204
 
 
-KstObjectItem::KstObjectItem(QListView *parent, KstFilteredVectorPtr x, KstDataManagerI *dm)
-: QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_FILTERED_VECTOR), _name(x->tagName()), _dm(dm) {
-  assert(x);
-  setText(0, x->tagName());
-  setText(1, i18n("Filtered Vector"));
-  x = 0L; // keep the counts in sync
-  update();
-}
-
 KstObjectItem::KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI *dm)
 : QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_DATA_VECTOR), _name(x->tagName()), _dm(dm) {
   assert(x);
@@ -117,27 +112,13 @@ void KstObjectItem::update(bool recursive) {
         assert(dynamic_cast<KstRVector*>(px.data()));
         KstRVectorPtr x = static_cast<KstRVector*>(px.data());
         x->readLock();
-        setText(2, QString::number(x->getUsage() - 3)); // caller has a ptr
+        setText(2, QString::number(x->getUsage() - 2)); // caller has a ptr
         setText(3, QString::number(x->sampleCount()));
         setText(4, i18n("%3: %4 [%1..%2]").arg(x->reqStartFrame())
             .arg(x->reqStartFrame() + x->reqNumFrames())
             .arg(x->filename())
-            .arg(x->getField()));
-        _removable = x->getUsage() == 3;
-        x->readUnlock();
-        break;
-      }
-    case RTTI_OBJ_FILTERED_VECTOR:
-      {
-        KstVectorPtr px = *KST::vectorList.findTag(_name);
-        assert(px.data());
-        assert(dynamic_cast<KstFilteredVector*>(px.data()));
-        KstFilteredVectorPtr x = static_cast<KstFilteredVector*>(px.data());
-        x->readLock();
-        setText(2, QString::number(x->getUsage() - 3)); // caller has a ptr
-        setText(3, QString::number(x->sampleCount()));
-        setText(4, i18n("[%1..%2]").arg(x->min()).arg(x->max()));
-        _removable = x->getUsage() == 3;
+            .arg(x->field()));
+        _removable = x->getUsage() == 2;
         x->readUnlock();
         break;
       }
@@ -202,7 +183,7 @@ void KstObjectItem::update(bool recursive) {
 }
 
 void KstObjectItem::updateButtons() {
-  _dm->Edit->setEnabled(RTTI_OBJ_VECTOR != _rtti && RTTI_OBJ_FILTERED_VECTOR != _rtti);
+  _dm->Edit->setEnabled(RTTI_OBJ_VECTOR != _rtti);
   _dm->Delete->setEnabled(_removable);
 }
 
@@ -224,7 +205,18 @@ void KstObjectItem::makeCurve() {
   KstCurveDialogI::globalInstance()->_yVector->setSelection(_name);
 }
 
+void KstObjectItem::makeMatrix() {
+  KstMatrixDialogI::globalInstance()->show_New();
+  KstMatrixDialogI::globalInstance()->_zVector->setSelection(_name);
+}
+
+void KstObjectItem::makeImage() {
+  KstImageDialogI::globalInstance()->show_New();
+  KstImageDialogI::globalInstance()->_matrix->setSelection(_name);
+}
+
 void KstObjectItem::applyFilter(int id) {
+/* Replace with new filtering logic
   KST::filterSetList.lock().readLock();
   KstFilterSetPtr p = KST::filterSetList.find(FilterMap[id]);
   KST::filterSetList.lock().readUnlock();
@@ -235,6 +227,7 @@ void KstObjectItem::applyFilter(int id) {
     KstFilteredVectorPtr fv = new KstFilteredVector(v, p);
     emit updated();
   }
+*/
 }
 
 void KstObjectItem::addToPlot(int id) {
@@ -255,6 +248,24 @@ void KstObjectItem::removeFromPlot(int id) {
   }
 }
 
+void KstObjectItem::addImageToPlot(int id) {
+  Kst2DPlotPtr p = PlotMap[id];
+  KstImagePtr img = dynamic_cast<KstImage*>(dataObject().data());
+  if (p && img) {
+    p->addImage(img);
+    emit updated();
+  }
+}
+
+void KstObjectItem::removeImageFromPlot(int id) {
+  Kst2DPlotPtr p = PlotMap[id];
+  KstImagePtr img = dynamic_cast<KstImage*>(dataObject().data());
+  if (p && img) {
+    p->removeImage(img);
+    emit updated();
+  }
+}
+
 
 KstDataManagerI::KstDataManagerI(KstDoc *in_doc, QWidget* parent, const char* name, bool modal, WFlags fl)
 : KstDataManager(parent, name, modal, fl) {
@@ -287,6 +298,12 @@ KstDataManagerI::KstDataManagerI(KstDoc *in_doc, QWidget* parent, const char* na
     connect(OpenPlotDialog, SIGNAL(clicked()),
             KstApp::inst(), SLOT(showPlotDialog()));
 
+    connect(NewImage, SIGNAL(clicked()),
+            KstImageDialogI::globalInstance(), SLOT(show_New()));
+
+    connect(NewMatrix, SIGNAL(clicked()),
+            KstMatrixDialogI::globalInstance(), SLOT(show_New()));
+
 // FIXME: Filters disabled for now...
 //  connect(Filter, SIGNAL(clicked()), this, SLOT(filter_I()));
 
@@ -383,23 +400,6 @@ void KstDataManagerI::update() {
     }
   }
 
-  KstFilteredVectorList fvl = kstObjectSubList<KstVector,KstFilteredVector>(KST::vectorList);
-  for (KstFilteredVectorList::iterator it = fvl.begin(); it != fvl.end(); ++it) {
-    bool found = false;
-    for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
-      KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-      if (oi->rtti() == RTTI_OBJ_FILTERED_VECTOR && oi->tagName() == (*it)->tagName()) {
-        oi->update();
-        found = true;
-        break;
-      }
-    }
-    if (!found) {
-        KstObjectItem *i = new KstObjectItem(DataView, *it, this);
-        connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
-    }
-  }
-
   // is this really necessary?  I would think not.....
   for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
     if (i == currentItem) {
@@ -419,7 +419,7 @@ void KstDataManagerI::update() {
 
 
 void KstDataManagerI::filter_I() {
-  static_cast<KstApp*>(doc->parent())->showFilterListEditor();
+//  static_cast<KstApp*>(doc->parent())->showFilterListEditor();
 }
 
 
@@ -454,9 +454,6 @@ void KstDataManagerI::delete_I() {
   } else if (qi->rtti() == RTTI_OBJ_DATA_VECTOR) {
     KST::vectorList.removeTag(koi->tagName());
     doUpdates();
-  } else if (qi->rtti() == RTTI_OBJ_FILTERED_VECTOR) {
-    KST::vectorList.removeTag(koi->tagName());
-    doUpdates();
   }
 }
 
@@ -467,24 +464,30 @@ void KstDataManagerI::delete_I() {
 
 void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
   Q_UNUSED(col)
+
+  if (!i) {
+    return;
+  }
+  
   KstObjectItem *koi = static_cast<KstObjectItem*>(i);
   KstBaseCurvePtr c;
+  KstImagePtr img;
 
   KPopupMenu *m = new KPopupMenu(this);
 
   m->setTitle(koi->text(0));
 
   int id;
-  if (koi->rtti() != RTTI_OBJ_VECTOR && koi->rtti() != RTTI_OBJ_FILTERED_VECTOR) {
+  if (koi->rtti() != RTTI_OBJ_VECTOR) {
     id = m->insertItem(i18n("&Edit..."), this, SLOT(edit_I()));
   }
 
   if (koi->rtti() == RTTI_OBJ_DATA_VECTOR ||
-      koi->rtti() == RTTI_OBJ_FILTERED_VECTOR ||
       koi->rtti() == RTTI_OBJ_VECTOR) {
     KPopupMenu *fMenu = new KPopupMenu(this);
     id = 500;
     FilterMap.clear();
+#if 0 
     KST::filterSetList.lock().readLock();
     for (KstFilterSetList::Iterator f = KST::filterSetList.begin(); f != KST::filterSetList.end(); ++f) {
       fMenu->insertItem((*f)->name(), koi, SLOT(applyFilter(int)), 0, id);
@@ -492,17 +495,20 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
       ++id;
     }
     KST::filterSetList.lock().readUnlock();
+#endif
     id = m->insertItem(i18n("Apply &Filter"), fMenu);
     m->setItemEnabled(id, !FilterMap.isEmpty());
   }
 
   if (koi->rtti() == RTTI_OBJ_DATA_VECTOR) {
     id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
+    id = m->insertItem(i18n("Make M&atrix..."), koi, SLOT(makeMatrix()));
     id = m->insertItem(i18n("&Reload"), koi, SLOT(reload()));
   } else if (koi->rtti() == RTTI_OBJ_VECTOR) {
     id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
-  } else if (koi->rtti() == RTTI_OBJ_FILTERED_VECTOR) {
-    id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
+    id = m->insertItem(i18n("Make M&atrix..."), koi, SLOT(makeMatrix()));
+  } else if (dynamic_cast<KstMatrix*>(koi->dataObject().data())) {
+    id = m->insertItem(i18n("Make &Image..."), koi, SLOT(makeImage()));
   } else if ((c = dynamic_cast<KstBaseCurve*>(koi->dataObject().data()))) {
     KPopupMenu *addMenu = new KPopupMenu(this);
     KPopupMenu *removeMenu = new KPopupMenu(this);
@@ -532,7 +538,41 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
     }
 
     app->deleteIterator(it);
-    
+
+    id = m->insertItem(i18n("&Add To Plot"), addMenu);
+    m->setItemEnabled(id, haveAdd);
+    id = m->insertItem(i18n("&Remove From Plot"), removeMenu);
+    m->setItemEnabled(id, haveRemove);
+  } else if ((img = dynamic_cast<KstImage*>(koi->dataObject().data()))) {
+    KPopupMenu *addMenu = new KPopupMenu(this);
+    KPopupMenu *removeMenu = new KPopupMenu(this);
+    PlotMap.clear();
+    id = 100;
+    bool haveAdd = false, haveRemove = false;
+
+    KstApp *app = KstApp::inst();
+    KMdiIterator<KMdiChildView*> *it = app->createIterator();
+    while (it->currentItem()) {
+      KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
+      if (v) {
+        Kst2DPlotList plots = v->view()->findChildrenType<Kst2DPlot>();
+        for (Kst2DPlotList::Iterator i = plots.begin(); i != plots.end(); ++i) {
+          Kst2DPlotPtr plot = *i;
+          if (!plot->hasImage(img)) {
+            addMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tagName()), koi, SLOT(addImageToPlot(int)), 0, id);
+            haveAdd = true;
+          } else {
+            removeMenu->insertItem(i18n("%1 - %2").arg(v->caption()).arg(plot->tagName()), koi, SLOT(removeImageFromPlot(int)), 0, id);
+            haveRemove = true;
+          }
+          PlotMap[id++] = plot;
+        }
+      }
+      it->next();
+    }
+
+    app->deleteIterator(it);
+
     id = m->insertItem(i18n("&Add To Plot"), addMenu);
     m->setItemEnabled(id, haveAdd);
     id = m->insertItem(i18n("&Remove From Plot"), removeMenu);
