@@ -25,9 +25,11 @@
 #include <kservice.h>
 #include <kservicetype.h>
 
+#include <qdeepcopy.h>
 #include <qfile.h>
 #include <qstylesheet.h>
 
+#include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "stdinsource.h"
 
@@ -40,16 +42,24 @@ namespace KST {
       Plugin(KService::Ptr svc) : KstShared(), service(svc), _lib(0L) {
         assert(service);
         _plugLib = service->property("X-Kst-Plugin-Library").toString();
+        //kdDebug() << "Create plugin " << (void*)this << " " << service->property("Name").toString() << endl;
       }
 
-      virtual ~Plugin() { if (_lib) { _lib->unload(); } }
+      virtual ~Plugin() {
+        //kdDebug() << "Destroy plugin " << (void*)this << " " << service->property("Name").toString() << endl;
+        if (_lib) {
+          _lib->unload();
+        }
+      }
 
       KstDataSource *create(const QString& filename, const QString& type = QString::null) const {
         KstDataSource *(*sym)(const QString&, const QString&) = (KstDataSource*(*)(const QString&, const QString&))symbol("create");
         if (sym) {
           //kdDebug() << "Trying to create " << filename << " type=" << type << " with " << service->property("Name").toString() << endl;
           KstDataSource *ds = (sym)(filename, type);
-          ds->_source = service->property("Name").toString();
+          if (ds) {
+            ds->_source = service->property("Name").toString();
+          }
           //kdDebug() << (ds ? "SUCCESS" : "FAILED") << endl;
           return ds;
         }
@@ -57,11 +67,27 @@ namespace KST {
         return 0L;
       }
 
-      bool understands(const QString& filename) const {
-        bool (*sym)(const QString&) = (bool(*)(const QString&))symbol("understands");
+      QStringList fieldList(const QString& filename, const QString& type = QString::null) const {
+        QStringList (*sym)(const QString&, const QString&) = (QStringList(*)(const QString&, const QString&))symbol("fieldList");
+        if (sym) {
+          return (sym)(filename, type);
+        }
+
+        // fallback incase the helper isn't implemented
+        //  (note: less efficient now)
+        KstDataSourcePtr ds = create(filename, type);
+        if (ds) {
+          return ds->fieldList();
+        }
+
+        return QStringList();
+      }
+
+      int understands(const QString& filename) const {
+        int (*sym)(const QString&) = (int(*)(const QString&))symbol("understands");
         if (sym) {
           //kdDebug() << "Checking if " << service->property("Name").toString() << " understands " << filename << endl;
-          bool rc = (sym)(filename);
+          int rc = (sym)(filename);
           //kdDebug() << "result: " << rc << endl;
           return rc;
         }
@@ -142,7 +168,8 @@ static void scanPlugins() {
 
   // This cleans up plugins that have been uninstalled and adds in new ones.
   // Since it is a shared pointer it can't dangle anywhere.
-  pluginInfo = tmpList;
+  pluginInfo.clear();
+  pluginInfo = QDeepCopy<KST::PluginInfoList>(tmpList);
 }
 
 
@@ -161,13 +188,29 @@ QStringList KstDataSource::pluginList() {
 }
 
 
+namespace {
+class PluginSortContainer {
+  public:
+    KstSharedPtr<KST::Plugin> plugin;
+    int match;
+    int operator<(const PluginSortContainer& x) const {
+      return match > x.match; // yes, this is by design.  biggest go first
+    }
+    int operator==(const PluginSortContainer& x) const {
+      return match == x.match;
+    }
+};
+}
+
 static KstDataSourcePtr findPluginFor(const QString& filename, const QString& type) {
   KstDataSourcePtr plugin;
 
   scanPlugins();
 
+  KST::PluginInfoList info = QDeepCopy<KST::PluginInfoList>(pluginInfo);
+
   if (!type.isEmpty()) {
-    for (KST::PluginInfoList::ConstIterator it = pluginInfo.begin(); it != pluginInfo.end(); ++it) {
+    for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
       if ((*it)->provides(type)) {
         plugin = (*it)->create(filename, type);
         break;
@@ -177,13 +220,23 @@ static KstDataSourcePtr findPluginFor(const QString& filename, const QString& ty
     return plugin;
   }
 
-  for (KST::PluginInfoList::ConstIterator it = pluginInfo.begin(); it != pluginInfo.end(); ++it) {
-    if ((*it)->understands(filename)) {
-      plugin = (*it)->create(filename);
+  QValueList<PluginSortContainer> bestPlugins;
+
+  for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
+    PluginSortContainer psc;
+    if ((psc.match = (*it)->understands(filename)) > 0) {
+      psc.plugin = *it;
+      bestPlugins.append(psc);
+    }
+  }
+
+  qHeapSort(bestPlugins);
+
+  for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
+      plugin = (*i).plugin->create(filename);
       if (plugin) {
         break;
       }
-    }
   }
 
   return plugin;
@@ -203,6 +256,47 @@ KstDataSourcePtr KstDataSource::loadSource(const QString& filename, const QStrin
 }
 
 
+QStringList KstDataSource::fieldListForSource(const QString& filename, const QString& type) {
+  if (filename.isEmpty() || filename == "stdin" || filename == "-") {
+    return QStringList();
+  }
+
+  scanPlugins();
+
+  KST::PluginInfoList info = QDeepCopy<KST::PluginInfoList>(pluginInfo);
+
+  if (!type.isEmpty()) {
+    for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
+      if ((*it)->provides(type)) {
+        return (*it)->fieldList(filename, type);
+      }
+    }
+  }
+
+  QValueList<PluginSortContainer> bestPlugins;
+
+  for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
+    PluginSortContainer psc;
+    if ((psc.match = (*it)->understands(filename)) > 0) {
+      psc.plugin = *it;
+      bestPlugins.append(psc);
+    }
+  }
+
+  qHeapSort(bestPlugins);
+
+  QStringList rc;
+  for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
+      rc = (*i).plugin->fieldList(filename);
+      if (!rc.isEmpty()) {
+        break;
+      }
+  }
+
+  return rc;
+}
+
+
 KstDataSourcePtr KstDataSource::loadSource(QDomElement& e) {
   QString filename, type;
 
@@ -230,15 +324,21 @@ KstDataSourcePtr KstDataSource::loadSource(QDomElement& e) {
   return findPluginFor(filename, type);
 }
 
- 
+
 KstDataSource::KstDataSource(const QString& filename, const QString& type)
 : KstObject(), _filename(filename) {
   Q_UNUSED(type)
   _valid = false;
+  d = new KstDataSourcePrivate;
+  d->numFrames = new KstScalar(filename + i18n("-frames"));
 }
 
-
 KstDataSource::~KstDataSource() {
+  KST::scalarList.lock().writeLock();
+  KST::scalarList.remove(d->numFrames);
+  KST::scalarList.lock().writeUnlock();
+  d->numFrames = 0;
+  delete d;
 }
 
 
@@ -247,6 +347,9 @@ KstObject::UpdateType KstDataSource::update(int u) {
   return KstObject::NO_CHANGE;
 }
 
+void KstDataSource::updateNumFramesScalar() {
+  d->numFrames->setValue(frameCount());
+}
 
 int KstDataSource::readField(double *v, const QString& field, int s, int n) {
   Q_UNUSED(v)
@@ -274,7 +377,8 @@ int KstDataSource::samplesPerFrame(const QString &field) {
 }
 
 
-int KstDataSource::frameCount() const {
+int KstDataSource::frameCount(const QString& field) const {
+  Q_UNUSED(field)
   return 0;
 }
 
@@ -308,7 +412,7 @@ void KstDataSource::virtual_hook(int id, void *data) {
 
 
 void *KstDataSource::bufferMalloc(size_t size) {
-  return ::malloc(size);
+  return KST::malloc(size);
 }
 
 
@@ -318,8 +422,27 @@ void KstDataSource::bufferFree(void *ptr) {
 
 
 void *KstDataSource::bufferRealloc(void *ptr, size_t size) {
-  return ::realloc(ptr, size);
+  return KST::realloc(ptr, size);
 }
 
 
+QGuardedPtr<QWidget> KstDataSource::configWidget() const {
+  return 0L;
+}
+
+
+bool KstDataSource::fieldListIsComplete() const {
+  return true;
+}
+
+
+bool KstDataSource::isEmpty() const {
+  return true;
+}
+
+
+bool KstDataSource::reset() {
+  return false;
+}
+
 // vim: ts=2 sw=2 et
