@@ -35,8 +35,8 @@ KstDebug *KstDebug::self() {
 }
 
 KstDebug::KstDebug() {
-  _bApplyLimit = false;
-  _iLimit      = 10000;
+  _applyLimit = false;
+  _limit = 10000;
 }
 
 KstDebug::~KstDebug() {
@@ -55,28 +55,30 @@ void KstDebug::log(const QString& msg, LogLevel level) {
   message.msg   = msg;
   message.level = level;
       
-  _messages.append( message );
-  if( _bApplyLimit ) {
-    if( (int)_messages.size( ) > _iLimit ) {
+  _messages.append(message);
+  if (_applyLimit) {
+    if ((int)_messages.size() > _limit) {
       QValueListIterator<LogMessage> first;
       QValueListIterator<LogMessage> last;
-      
-      first = _messages.begin( );
+
+      first = _messages.begin();
       last  = first;
-      last += _messages.size( ) - _iLimit; 
-      
-      _messages.erase( first, last );
+      last += _messages.size() - _limit; 
+
+      _messages.erase(first, last);
     }
   }
   
-  emit logAdded( );
+  emit logAdded();
 }
 
+
 void KstDebug::clear() {
   _messages.clear(); 
 }
 
-QString KstDebug::getLabel(LogLevel level) const {
+
+QString KstDebug::label(LogLevel level) const {
   QString str;
   
   switch (level) {
@@ -100,15 +102,12 @@ QString KstDebug::getLabel(LogLevel level) const {
   return str;
 }
 
-void KstDebug::getText(QString& body) {
-  unsigned int i;
+QString KstDebug::text() const {
+  // FIXME: this must be hard to internationalize
+  QString body = i18n("Kst version %1\n\n\nKst log:\n").arg(KSTVERSION);
   
-  body += i18n("Kst version %1\n\n\n").arg(KSTVERSION);
-
-  body += i18n("Kst log:\n");
-  
-  for( i=0; i<_messages.count(); i++ ) {
-    body += i18n("%1 %2: %3\n").arg(_messages[i].date.toString()).arg(getLabel(_messages[i].level)).arg(_messages[i].msg);
+  for (unsigned i = 0; i < _messages.count(); i++ ) {
+    body += i18n("%1 %2: %3\n").arg(_messages[i].date.toString()).arg(label(_messages[i].level)).arg(_messages[i].msg);
   }
   
   body += i18n("\n\nData-source plugins:");
@@ -118,19 +117,16 @@ void KstDebug::getText(QString& body) {
     body += *it;
   }
   body += "\n\n";
+  return body;
 }
 
-void KstDebug::setLimit( bool bApplyLimit, int iLimit ) {
-  _bApplyLimit = bApplyLimit;
-  _iLimit      = iLimit;
+void KstDebug::setLimit(bool applyLimit, int limit) {
+  _applyLimit = applyLimit;
+  _limit = limit;
 }
 
 void KstDebug::sendEmail() {
-  QString 			body;
-  
-  getText( body );
-
-  kapp->invokeMailer(QString::null, QString::null, QString::null, i18n("Kst Debugging Information"), body);
+  kapp->invokeMailer(QString::null, QString::null, QString::null, i18n("Kst Debugging Information"), text());
 }
 
 
