@@ -19,8 +19,6 @@
 #define KSTDEBUG_H
 
 #include <qobject.h>
-#include <qstring.h>
-#include <qstringlist.h>
 #include <qdatetime.h>
 
 #include <kstaticdeleter.h>
@@ -41,13 +39,13 @@ class KstDebug : public QObject {
 
     void clear();
     void log(const QString& msg, LogLevel level = Notice);
-    void setLimit( bool bApplyLimit, int iLimit );   
-    void getText(QString& body);
+    void setLimit(bool applyLimit, int limit);   
+    QString text() const;
     void sendEmail();
     
-    QValueList<LogMessage>* messages( ) { return &_messages; }
+    const QValueList<LogMessage>& messages() { return _messages; }
     QStringList dataSourcePlugins() const;
-    QString getLabel(LogLevel level) const;
+    QString label(LogLevel level) const;
 
   signals: 
     void logAdded();
@@ -58,8 +56,8 @@ class KstDebug : public QObject {
     
     static KstDebug *_self;
     QValueList<LogMessage> _messages;
-    bool  _bApplyLimit;
-    int 	_iLimit;
+    bool  _applyLimit;
+    int 	_limit;
 };
 
 
