@@ -39,7 +39,6 @@
 #include <qstring.h>
 #include <qtextstream.h>
 #include <qwidget.h>
-#include <qmessagebox.h>
 
 // include files for KDE
 #include <kapplication.h>
@@ -47,11 +46,11 @@
 #include <kdebug.h>
 #include <kfiledialog.h>
 #include <klocale.h>
+#include <kmdimainfrm.h>
 #include <kmessagebox.h>
 #include <kprogress.h>
 #include <ksavefile.h>
 #include <kurl.h>
-#include <kmdimainfrm.h>
 
 // application specific includes
 #include "kst.h"
@@ -60,13 +59,13 @@
 #include "kstdebug.h"
 #include "kstdoc.h"
 #include "kstequationcurve.h"
-#include "kstfilteredvector.h"
 #include "ksthistogram.h"
+#include "kstimage.h"
+#include "kstmatrix.h"
 #include "kstplugin.h"
 #include "kstpoint.h"
 #include "kstpsdcurve.h"
 #include "kstvcurve.h"
-#include "kstfitcurve.h"
 #include "kstvectordefaults.h"
 #include "kstviewwindow.h"
 #include "threadevents.h"
@@ -113,7 +112,9 @@ bool KstDoc::saveModified(bool doDelete) {
     switch (want_save) {
       case KMessageBox::Yes:
         if (_title == i18n("Untitled")) {
-          win->slotFileSaveAs();
+          if (!win->slotFileSaveAs()) {
+            return false;
+          }
         } else {
           saveDocument(absFilePath());
         }
@@ -197,6 +198,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   f.close();
 
   QDomElement docElem = doc.documentElement();
+  int columns = -1;
 
   QDomNode n = docElem.firstChild();
   while (!n.isNull()) {
@@ -216,8 +218,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         }
         static_cast<KstApp*>(parent())->resize(size);
       } else if (e.tagName() == "plotcols") {
-// FIXMEPLOTLIST
-//        KST::plotList.setPlotCols(e.text().toInt());
+        columns = e.text().toInt();
       } else if (e.tagName() == "updateDelay") {
 // FIXME XXXX
         setDelay(e.text().toInt());
@@ -228,7 +229,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         } else {
           file = KstDataSource::loadSource(o_file);
         }
-        if (file && (!file->isValid() || file->frameCount() < 1)) {
+        if (file && (!file->isValid() || file->isEmpty())) {
           KstDebug::self()->log(i18n("No data in file %1.").arg(file->fileName()), KstDebug::Warning);
         }
         if (file) {
@@ -240,13 +241,6 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         vector = new KstRVector(e, o_file, o_n, o_f, o_s, o_ave);
         KST::addVectorToList(KstVectorPtr(vector));
         // Vectors are automatically added to the global list.
-      } else if (e.tagName() == "filterset") {
-        KST::filterSetList.lock().writeLock();
-        KST::filterSetList.append(new KstFilterSet(e));
-        KST::filterSetList.lock().writeUnlock();
-      } else if (e.tagName() == "filteredvector") {
-        new KstFilteredVector(e);
-        // Vectors are automatically added to the global list.
       } else if (e.tagName() == "plugin") {
         KstDataObjectPtr p = new KstPlugin(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -255,10 +249,6 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         KstDataObjectPtr p = new KstVCurve(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
         KST::dataObjectList.append(p);
-      } else if (e.tagName() == "fit") {
-        KstDataObjectPtr p = new KstFitCurve(e);
-        KstWriteLocker dowl(&KST::dataObjectList.lock());
-        KST::dataObjectList.append(p);
       } else if (e.tagName() == "equation") {
         KstDataObjectPtr p = new KstEquationCurve(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -279,6 +269,14 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         KstBaseCurveList l = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
         Kst2DPlotPtr plotPtr = new Kst2DPlot(e);
         KstApp::inst()->plotHolderWhileOpeningDocument().insert(plotPtr->tagName(), plotPtr);
+      } else if (e.tagName() == "matrix") {
+        KstDataObjectPtr p = new KstMatrix(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
+      } else if (e.tagName() == "image") {
+        KstDataObjectPtr p = new KstImage(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
       } else if (e.tagName() == "window") {
         KstViewWindow *p = dynamic_cast<KstViewWindow*>(KstApp::inst()->activeWindow());
         KstViewWindow *toDelete = 0L;
@@ -311,32 +309,32 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   //  we are most likely reading an old style document, so we create
   //  a default view and fill it with whatever is left...
   //
-  if (KstApp::inst()->plotHolderWhileOpeningDocument().count() > 0) {
-    if (!KstApp::inst()->activeWindow()) {
-      Kst2DPlotMap::Iterator it;
-      Kst2DPlotPtr plot;
-      KstViewWindow *pWindow;
-      QString strWindow;
+  KstApp *app = KstApp::inst();
+  if (!app->plotHolderWhileOpeningDocument().isEmpty() && !app->activeWindow()) {
 
-      strWindow = KstApp::inst()->newWindow(QString::null);
-      pWindow = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(strWindow));
+    QString winName = app->newWindow(QString::null);
+    KstViewWindow *win = dynamic_cast<KstViewWindow*>(app->findWindow(winName));
 
-      if (pWindow) {
-        for ( it  = KstApp::inst()->plotHolderWhileOpeningDocument().begin();
-              it != KstApp::inst()->plotHolderWhileOpeningDocument().end(); ++it ) {
-          plot = *it;
+    if (win) {
+      if (columns != -1) {
+        win->view()->setOnGrid(true);
+        win->view()->setColumns(columns);
+      }
 
-          if (plot->_width > 0.0 && plot->_height > 0.0 ) {
-            pWindow->view()->appendChild(plot.data(), true);
-            if (plot->_width > 1.0 && plot->_height > 1.0) {
-              plot->resizeFromAspect( (double)plot->_pos_x / (double)plot->_width,
-                                     (double)plot->_pos_y / (double)plot->_height,
-                                     1.0 / (double)plot->_width,
-                                     1.0 / (double)plot->_height );
-            } else {
-              plot->resizeFromAspect( plot->_pos_x, plot->_pos_y,
-                                      plot->_width, plot->_height );
-            }
+      for (Kst2DPlotMap::Iterator it = app->plotHolderWhileOpeningDocument().begin();
+          it != app->plotHolderWhileOpeningDocument().end(); ++it ) {
+        Kst2DPlotPtr plot = *it;
+
+        if (plot->_width > 0.0 && plot->_height > 0.0 ) {
+          win->view()->appendChild(plot.data(), true);
+          if (plot->_width > 1.0 && plot->_height > 1.0) {
+            plot->resizeFromAspect(double(plot->_pos_x) / double(plot->_width),
+                double(plot->_pos_y) / double(plot->_height),
+                1.0 / double(plot->_width),
+                1.0 / double(plot->_height));
+          } else {
+            plot->resizeFromAspect(plot->_pos_x, plot->_pos_y,
+                plot->_width, plot->_height);
           }
         }
       }
@@ -397,8 +395,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
 }
 
 void KstDoc::saveDocument(QTextStream& ts) {
-  int i;
-
+  KstApp *app = KstApp::inst();
   ts << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
   ts << "<kstdoc>" << endl;
   // save window geometry for this kst file
@@ -407,17 +404,13 @@ void KstDoc::saveDocument(QTextStream& ts) {
   ts << "  <height>"<< ((KstApp *)parent())->height()<< "</height>" << endl;
   ts << " </windowsize>" << endl;
 
-  // save plot cols
-// FIXMEPLOTLIST
-//  ts << " <plotcols>" << KST::plotList.getPlotCols() << "</plotcols>" << endl;
-
   // save update delay
 //FIXME XXXX
   ts << " <updateDelay>" << delay() << "</updateDelay>" << endl;
 
   // save files
   KST::dataSourceList.lock().readLock();
-  for (i = 0; i < (int)KST::dataSourceList.count(); i++) {
+  for (uint i = 0; i < KST::dataSourceList.count(); i++) {
     ts << " <kstfile>" << endl;
     KST::dataSourceList[i]->save(ts);
     ts << " </kstfile>" << endl;
@@ -447,26 +440,8 @@ void KstDoc::saveDocument(QTextStream& ts) {
     (*it)->save(ts);
   }
   KST::dataObjectList.lock().readUnlock();
-
-  // save filters
-  KST::filterSetList.lock().readLock();
-  for (KstFilterSetList::Iterator it = KST::filterSetList.begin(); it != KST::filterSetList.end(); ++it) {
-    ts << " <filterset>" << endl;
-    (*it)->save(ts);
-    ts << " </filterset>" << endl;
-  }
-  KST::filterSetList.lock().readUnlock();
-
-  KstFilteredVectorList fvl = kstObjectSubList<KstVector,KstFilteredVector>(KST::vectorList);
-  for (KstFilteredVectorList::Iterator it = fvl.begin(); it != fvl.end(); ++it) {
-    ts << " <filteredvector>" << endl;
-    (*it)->save(ts);
-    ts << " </filteredvector>" << endl;
-  }
-
   // save plots
-  KstApp *pApp = KstApp::inst();
-  KMdiIterator<KMdiChildView*> *it = pApp->createIterator();
+  KMdiIterator<KMdiChildView*> *it = app->createIterator();
   while (it->currentItem()) {
     KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
     if (c) {
@@ -483,10 +458,10 @@ void KstDoc::saveDocument(QTextStream& ts) {
     }
     it->next();
   }
-  pApp->deleteIterator(it);
+  app->deleteIterator(it);
 
   // save windows
-  it = KstApp::inst()->createIterator();
+  it = app->createIterator();
   if (it) {
     while (it->currentItem()) {
       KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
@@ -497,7 +472,7 @@ void KstDoc::saveDocument(QTextStream& ts) {
       }
       it->next();
     }
-    KstApp::inst()->deleteIterator(it);
+    app->deleteIterator(it);
   }
 
   ts << "</kstdoc>" << endl;
@@ -531,25 +506,19 @@ bool KstDoc::saveDocument(const QString &filename) {
 }
 
 void KstDoc::deleteContents() {
-  KMdiIterator<KMdiChildView*>* pIterator;
 
   KST::vectorDefaults.sync();
 
-  KST::filterSetList.lock().writeLock();
-  KST::filterSetList.clear();
-  KST::filterSetList.lock().writeUnlock();
-
-  if (KstApp::inst()) {
-    pIterator = KstApp::inst()->createIterator();
-    if (pIterator) {
-      while (pIterator->currentItem()) {
-        KMdiChildView *pView = pIterator->currentItem();
-        pIterator->next();
-        if (pView) {
-          KstApp::inst()->closeWindow(pView);
-        }
+  KstApp *app = KstApp::inst();
+  if (app) { // Can be null on application exit
+    KMdiIterator<KMdiChildView*> *it = app->createIterator();
+    if (it) {
+      while (it->currentItem()) {
+        KMdiChildView *view = it->currentItem();
+        it->next();
+        app->closeWindow(view);
       }
-      KstApp::inst()->deleteIterator(pIterator);
+      app->deleteIterator(it);
     }
   }
 
@@ -667,26 +636,6 @@ void KstDoc::wasModified() {
 }
 
 RemoveStatus KstDoc::removeDataObject(const QString& tag) {
-  // FIXME: necessary?  We don't allow removal if it's in a plot...
-  /* delete the curve from all plots */
-// FIXMEPLOTLIST
-#if 0
-  for (unsigned i_plot = 0; i_plot < KST::plotList.count(); i_plot++) {
-    KstPlot *plot = KST::plotList.at(i_plot);
-    for (KstBaseCurveList::iterator it = plot->Curves.begin(); it != plot->Curves.end(); ++it) {
-      if ((*it)->tagName() == tag) {
-        if ((*it)->slaveVectorsUsed()) {
-          return SLAVE_USED;
-        } else {
-          KstBaseCurvePtr tmp = *it;
-          --it;
-          plot->Curves.remove(tmp);
-        }
-      }
-    }
-  }
-#endif
-
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.removeTag(tag);
   KST::dataObjectList.lock().writeUnlock();
@@ -699,10 +648,9 @@ RemoveStatus KstDoc::removeDataObject(const QString& tag) {
 void KstDoc::purge() {
   bool modified = false;
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-  KstFilteredVectorList fvl = kstObjectSubList<KstVector,KstFilteredVector>(KST::vectorList);
 
   KST::dataObjectList.lock().writeLock();
-  int cnt = rvl.count() + fvl.count() + KST::dataObjectList.count();
+  int cnt = rvl.count() + KST::dataObjectList.count();
   int prg = 0;
 
   KProgressDialog *dlg = new KProgressDialog((QWidget*)static_cast<KstApp*>(parent()), "Progress Dialog");
@@ -742,22 +690,6 @@ void KstDoc::purge() {
   }
 
   rvl.clear();
-
-  for (KstFilteredVectorList::Iterator it = fvl.begin(); it != fvl.end(); ++it) {
-    //kdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
-    if ((*it)->getUsage() == 1) {
-      //kdDebug() << "    -> REMOVED" << endl;
-      KST::vectorList.lock().writeLock();
-      KST::vectorList.remove((*it).data());
-      KST::vectorList.lock().writeUnlock();
-      modified = true;
-    }
-    dlg->progressBar()->setProgress(++prg);
-    kapp->processEvents();
-  }
-
-  fvl.clear();
-
   setModified(modified);
   emit updateDialogs();
 
@@ -771,9 +703,6 @@ void KstDoc::forceUpdate() {
 
 
 bool KstDoc::event(QEvent *e) {
-  KMdiIterator<KMdiChildView*>* pIterator;
-  KstViewWindow* pView;
-
   if (e->type() == KstEventTypeThread) {
     _updating = true; // block update thread from sending events 'till we're done
 
@@ -786,16 +715,16 @@ bool KstDoc::event(QEvent *e) {
       kdDebug() << "Update ALL dialogs" << endl;
 
       if (KstApp::inst()) {
-        pIterator = KstApp::inst()->createIterator();
-        if (pIterator) {
-          while (pIterator->currentItem()) {
-            pView = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
-            if (pView) {
-              pView->view()->update();
+        KMdiIterator<KMdiChildView*>* it = KstApp::inst()->createIterator();
+        if (it) {
+          while (it->currentItem()) {
+            KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
+            if (view) {
+              view->view()->update();
             }
-            pIterator->next();
+            it->next();
           }
-          KstApp::inst()->deleteIterator(pIterator);
+          KstApp::inst()->deleteIterator(it);
         }
       }
 
@@ -816,28 +745,29 @@ bool KstDoc::event(QEvent *e) {
 
 void KstDoc::createScalars() const {
   // Should not be orphans, just unparented so they are not saved
-  new KstScalar(i18n("CONST_MKSA_SPEED_OF_LIGHT"), 2.99792458e8 , false);
-  new KstScalar(i18n("CONST_MKSA_GRAVITATIONAL_CONSTANT"), 6.673e-11, false);
-  new KstScalar(i18n("CONST_MKSA_PLANCKS_CONSTANT_H"), 6.62606876e-34, false);
-  new KstScalar(i18n("CONST_MKSA_PLANCKS_CONSTANT_HBAR"), 1.05457159642e-34, false);
-  new KstScalar(i18n("CONST_MKSA_ASTRONOMICAL_UNIT"), 1.49597870691e11, false);
-  new KstScalar(i18n("CONST_MKSA_VACUUM_PERMITTIVITY"), 8.854187817e-12, false);
-  new KstScalar(i18n("CONST_MKSA_VACUUM_PERMEABILITY"), 1.25663706144e-6, false);
-  new KstScalar(i18n("CONST_MKSA_GRAV_ACCEL"), 9.80665e0, false);
-  new KstScalar(i18n("CONST_MKSA_MASS_MUON"), 1.88353109e-28, false);
-  new KstScalar(i18n("CONST_MKSA_MASS_PROTON"), 1.67262158e-27, false);
-  new KstScalar(i18n("CONST_MKSA_MASS_NEUTRON"), 1.67492716e-27, false);
-  new KstScalar(i18n("CONST_MKSA_RYDBERG"), 2.17987190389e-18, false);
-  new KstScalar(i18n("CONST_MKSA_BOLTZMANN"), 1.3806503e-23, false);
-  new KstScalar(i18n("CONST_MKSA_SOLAR_MASS"), 1.98892e30, false);
-  new KstScalar(i18n("CONST_MKSA_BOHR_RADIUS"), 5.291772083e-11, false);
-  new KstScalar(i18n("CONST_MKSA_ELECTRON_CHARGE"), 1.602176462e-19, false);
-  new KstScalar(i18n("CONST_MKSA_MOLAR_GAS"), 8.314472e0, false);
-  new KstScalar(i18n("CONST_MKSA_STANDARD_GAS_VOLUME"), 2.2710981e-2, false);
-  new KstScalar(i18n("CONST_PI"), 3.141592654e0, false);
+  new KstScalar("CONST_MKSA_SPEED_OF_LIGHT", 2.99792458e8 , false);
+  new KstScalar("CONST_MKSA_GRAVITATIONAL_CONSTANT", 6.673e-11, false);
+  new KstScalar("CONST_MKSA_PLANCKS_CONSTANT_H", 6.62606876e-34, false);
+  new KstScalar("CONST_MKSA_PLANCKS_CONSTANT_HBAR", 1.05457159642e-34, false);
+  new KstScalar("CONST_MKSA_ASTRONOMICAL_UNIT", 1.49597870691e11, false);
+  new KstScalar("CONST_MKSA_VACUUM_PERMITTIVITY", 8.854187817e-12, false);
+  new KstScalar("CONST_MKSA_VACUUM_PERMEABILITY", 1.25663706144e-6, false);
+  new KstScalar("CONST_MKSA_GRAV_ACCEL", 9.80665e0, false);
+  new KstScalar("CONST_MKSA_MASS_MUON", 1.88353109e-28, false);
+  new KstScalar("CONST_MKSA_MASS_PROTON", 1.67262158e-27, false);
+  new KstScalar("CONST_MKSA_MASS_NEUTRON", 1.67492716e-27, false);
+  new KstScalar("CONST_MKSA_RYDBERG", 2.17987190389e-18, false);
+  new KstScalar("CONST_MKSA_BOLTZMANN", 1.3806503e-23, false);
+  new KstScalar("CONST_MKSA_SOLAR_MASS", 1.98892e30, false);
+  new KstScalar("CONST_MKSA_BOHR_RADIUS", 5.291772083e-11, false);
+  new KstScalar("CONST_MKSA_ELECTRON_CHARGE", 1.602176462e-19, false);
+  new KstScalar("CONST_MKSA_MOLAR_GAS", 8.314472e0, false);
+  new KstScalar("CONST_MKSA_STANDARD_GAS_VOLUME", 2.2710981e-2, false);
+  new KstScalar("CONST_PI", 3.141592654e0, false);
 }
 
 
+// All code after this point is code copied from KDE libraries
 static int write_all(int fd, const char *buf, size_t len) {
    while (len > 0)
    {
@@ -860,8 +790,9 @@ static bool backupFile(const QString& qFilename, const QString& backupDir,
    const char *filename = cFilename.data();
 
    int fd = open(filename, O_RDONLY);
-   if (fd < 0)
+   if (fd < 0) {
       return false;
+   }
 
    struct stat buff;
    if ( fstat( fd, &buff) < 0 )
@@ -871,8 +802,9 @@ static bool backupFile(const QString& qFilename, const QString& backupDir,
    }
 
    QCString cBackup;
-   if ( backupDir.isEmpty() )
+   if ( backupDir.isEmpty() ) {
        cBackup = cFilename;
+   }
    else
    {
        QCString nameOnly;
@@ -882,8 +814,9 @@ static bool backupFile(const QString& qFilename, const QString& backupDir,
        else
 	   nameOnly = cFilename.mid(slash + 1);
        cBackup = QFile::encodeName(backupDir);
-       if ( backupDir[backupDir.length()-1] != '/' )
+       if ( backupDir[backupDir.length()-1] != '/' ) {
            cBackup += '/';
+       }
        cBackup += nameOnly;
    }
    cBackup += QFile::encodeName(backupExtension);
@@ -937,8 +870,9 @@ static bool backupFile(const QString& qFilename, const QString& backupDir,
 
     ::close( fd );
 
-    if (::close(fd2))
-        return false;
+    if (::close(fd2)) {
+      return false;
+    }
     return true;
 }
 
