@@ -18,16 +18,15 @@
 #ifndef KSTDOC_H
 #define KSTDOC_H
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 // include files for QT
 #include <qobject.h>
-#include <qstring.h>
 #include <qptrlist.h>
+#include <qstring.h>
+
+#include <kapplication.h>
 #include <kurl.h>
-#include <kapp.h>
 
 #include "kstobject.h"
 #include "kstlistenums.h"
