@@ -21,29 +21,29 @@
 #include <qcheckbox.h>
 #include <qlabel.h>
 #include <qlineedit.h>
+#include <qmessagebox.h>
 #include <qpushbutton.h>
 #include <qradiobutton.h>
 #include <qspinbox.h>
 #include <qstring.h>
 #include <qwidget.h>
-#include <qmessagebox.h>
 
 // include files for KDE
 #include <kcombobox.h>
 #include <kdebug.h>
 #include <klocale.h>
+#include <kmdimainfrm.h>
 #include <kmessagebox.h>
 #include <kstaticdeleter.h>
-#include <kmdimainfrm.h>
 
 // application specific includes
-#include "ksteqdialog_i.h"
 #include "curveappearancewidget.h"
 #include "curveplacementwidget.h"
 #include "kst.h"
 #include "kst2dplot.h"
-#include "kstdoc.h"
 #include "kstdatacollection.h"
+#include "kstdoc.h"
+#include "ksteqdialog_i.h"
 #include "kstviewwindow.h"
 #include "scalarselector.h"
 #include "vectorselector.h"
@@ -81,11 +81,7 @@ KstEquationCurvePtr KstEqDialogI::_getPtr(const QString &tagin) {
   KstEquationCurveList c =
     kstObjectSubList<KstDataObject, KstEquationCurve>(KST::dataObjectList);
 
-  if ( c.findTag(tagin) == c.end() ) {
-    return 0L;
-  } else {
-    return *c.findTag(tagin);
-  }
+  return *c.findTag(tagin);
 }
 
 void KstEqDialogI::updateWindow() {
@@ -93,8 +89,12 @@ void KstEqDialogI::updateWindow() {
 }
 
 void KstEqDialogI::_fillFieldsForEdit() {
-  if ( DP == 0L ) return; // shouldn't be needed
-  _tagName->setText( DP->tagName() );
+  if (DP == 0L) {
+    return; // shouldn't be needed
+  }
+
+  DP->readLock();
+  _tagName->setText(DP->tagName());
 
   Equation->setText(DP->equation());
 
@@ -116,11 +116,12 @@ void KstEqDialogI::_fillFieldsForEdit() {
     XMax->setText(QString::number(DP->vX()->max()));
   }
 
-
   _curveAppearance->setValue(
     DP->hasLines(), DP->hasPoints(), DP->getColor(),
     DP->Point.getType(), DP->lineWidth(), DP->lineStyle());
 
+  DP->readUnlock();
+
   _curvePlacement->hide();
 
   adjustSize();
@@ -128,14 +129,10 @@ void KstEqDialogI::_fillFieldsForEdit() {
 }
 
 void KstEqDialogI::_fillFieldsForNew() {
-  KstEquationCurveList eqs =
-    kstObjectSubList<KstDataObject, KstEquationCurve>(KST::dataObjectList);
+  KstEquationCurveList eqs = kstObjectSubList<KstDataObject, KstEquationCurve>(KST::dataObjectList);
 
   /* set tag name */
-  QString new_label;
-  new_label.sprintf("Eq%d-", signed(eqs.count())+1);
-  new_label += i18n("<New_Equation>");
-  _tagName->setText(new_label);
+  _tagName->setText(QString("E%1-<New_Equation>").arg(eqs.count()+1));
 
   /* set the curve placement window  */
   _curvePlacement->update();
@@ -161,12 +158,9 @@ void KstEqDialogI::update() {
 
 bool KstEqDialogI::new_I() {
   KstEquationCurvePtr eq;
-  int n;
-  double x0, x1;
-  bool ok;
 
   QString tag_name = _tagName->text();
-  tag_name.replace(i18n("<New_Equation>"), Equation->text());
+  tag_name.replace("<New_Equation>", Equation->text());
 
   /* verify that the curve name is unique */
   if (KST::dataTagNameNotUnique(tag_name)) {
@@ -174,7 +168,9 @@ bool KstEqDialogI::new_I() {
     return false;
   }
 
-  if ( !_checkEntries() ) return false;
+  if (!_checkEntries()) {
+    return false;
+  }
 
   if (UseExistingVector->isChecked()) {
     KST::vectorList.lock().readLock();
@@ -192,9 +188,10 @@ bool KstEqDialogI::new_I() {
                               *i, DoInterpolation->isChecked(),
                               _curveAppearance->color());
   } else {
-    n = N->value();
-    x0 = XMin->text().toDouble(&ok);
-    x1 = XMax->text().toDouble(&ok);
+    bool ok;
+    int n = N->value();
+    double x0 = XMin->text().toDouble(&ok);
+    double x1 = XMax->text().toDouble(&ok);
     eq = new KstEquationCurve(tag_name, Equation->text(),
                               x0, x1, n, _curveAppearance->color());
   }
@@ -212,14 +209,13 @@ bool KstEqDialogI::new_I() {
   eq->setLineStyle(_curveAppearance->lineStyle());
   eq->Point.setType(_curveAppearance->pointType());
 
-  Kst2DPlotPtr plot;
   KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_curvePlacement->_plotWindow->currentText()));
   if (!w) {
-    QString newName = KST::suggestWinName();
-    QString n = KstApp::inst()->newWindow(newName);
+    QString n = KstApp::inst()->newWindow(KST::suggestWinName());
     w = static_cast<KstViewWindow*>(KstApp::inst()->findWindow(n));
   }
   if (w) {
+    Kst2DPlotPtr plot;
     if (_curvePlacement->existingPlot()) {
       /* assign curve to plot */
       plot = dynamic_cast<Kst2DPlot*>(w->view()->findChild(_curvePlacement->plotName()).data());
@@ -263,23 +259,21 @@ bool KstEqDialogI::_checkEntries() {
       return false;
     }
   } else {
-    int n;
-    double x0, x1;
     bool ok;
 
-    n = N->value();
+    int n = N->value();
     if (n < 2) {
       KMessageBox::sorry(0L, i18n("The equation must be evaluated at more "
                                   "than one point.  Increase Num Samples."));
       return false;
     }
-    x0 = XMin->text().toDouble(&ok);
+    double x0 = XMin->text().toDouble(&ok);
     if (!ok) {
       KMessageBox::sorry(0L, i18n("The text you entered for the minimum X "
                                   "value is not a number."));
       return false;
     }
-    x1 = XMax->text().toDouble(&ok);
+    double x1 = XMax->text().toDouble(&ok);
     if (!ok) {
       KMessageBox::sorry(0L, i18n("The text you entered for the maximum X "
                                   "value is not a number."));
@@ -296,24 +290,26 @@ bool KstEqDialogI::_checkEntries() {
 }
 
 bool KstEqDialogI::edit_I() {
-
   /* verify that the curve name is unique */
+  DP->writeLock();
   if (_tagName->text() != DP->tagName()) {
     if (KST::dataTagNameNotUnique(_tagName->text())) {
+      DP->writeUnlock();
       return false;
     }
   }
 
   DP->setTagName(_tagName->text());
-
   DP->setColor(_curveAppearance->color());
   DP->setHasPoints(_curveAppearance->showPoints());
   DP->setHasLines(_curveAppearance->showLines());
   DP->setLineWidth(_curveAppearance->lineWidth());
   DP->setLineStyle(_curveAppearance->lineStyle());
   DP->Point.setType(_curveAppearance->pointType());
-
-  if ( !_checkEntries() ) return false;
+  if (!_checkEntries()) {
+    DP->writeUnlock();
+    return false;
+  }
 
   if (UseExistingVector->isChecked()) {
     KST::vectorList.lock().readLock();
@@ -324,25 +320,28 @@ bool KstEqDialogI::edit_I() {
       kdFatal() << "Bug in kst: the Vector field in plotDialog (Eq) "
                 << "refers to a non existant vector..." << endl;
     }
+
+    KstVectorPtr vp = *i;
     KST::vectorList.lock().readUnlock();
 
-    DP->setExistingXVector(*i, DoInterpolation->isChecked());
+    DP->setExistingXVector(vp, DoInterpolation->isChecked());
   } else {
     bool ok;
-    int n = N->value();
     double x0 = XMin->text().toDouble(&ok);
     double x1 = XMax->text().toDouble(&ok);
-    DP->setStaticXVector(x0, x1, n);
+    KstVectorPtr v = KstVector::generateVector(x0, x1, N->value(), QString::null);
+    DP->setStaticXVector(v);
   }
 
   DP->setEquation(Equation->text());
   if (!DP->isValid()) {
     KMessageBox::sorry(0L, i18n("There is a syntax error in the equation "
                                 "you entered.  Please fix it."));
+    DP->writeUnlock();
     return false;
   }
-
   DP->update();
+  DP->writeUnlock();
 
   emit modified();
 
