@@ -52,7 +52,7 @@ KstEquationCurve::KstEquationCurve(const QString &in_tag,
                                    int nx, const QColor &in_color)
 : KstBaseCurve() {
   _staticX = true;
-  DoInterp = false;
+  _doInterp = false;
   _xVector = _inputVectors.insert(XVECTOR, KstVector::generateVector(x0, x1, nx, QString::null));
   commonConstructor(in_tag, equation, in_color);
   update();
@@ -65,7 +65,7 @@ KstEquationCurve::KstEquationCurve(const QString &in_tag,
                                    const QColor &in_color)
 : KstBaseCurve() {
   _staticX = false;
-  DoInterp = do_interp; //false;
+  _doInterp = do_interp; //false;
   _xVector = _inputVectors.insert(XVECTOR, xvector);
 
   commonConstructor(in_tag, equation, in_color);
@@ -85,7 +85,7 @@ KstEquationCurve::KstEquationCurve(QDomElement &e)
   setHasLines(true);
 
   _staticX = false;
-  DoInterp = false;
+  _doInterp = false;
 
   /* parse the DOM tree */
   QDomNode n = e.firstChild();
@@ -113,7 +113,7 @@ KstEquationCurve::KstEquationCurve(QDomElement &e)
       } else if (e.tagName() == "hasPoints") {
         HasPoints = (e.text() != "0");
       } else if (e.tagName() == "interpolate") {
-        DoInterp = true;
+        _doInterp = true;
       } else if (e.tagName() == "pointType") {
         Point.setType(e.text().toInt());
       }
@@ -129,7 +129,7 @@ KstEquationCurve::KstEquationCurve(QDomElement &e)
       x1 = x0 + 2;
     }
     _staticX = true;
-    DoInterp = false;
+    _doInterp = false;
     _xVector = _inputVectors.insert(XVECTOR, KstVector::generateVector(x0, x1, ns, xvtag));
   } else {
     _xVector = _inputVectors.end();
@@ -178,14 +178,14 @@ void KstEquationCurve::commonConstructor(const QString &in_tag,
   _yVector = _outputVectors.insert(OUTVECTOR, yv);
   yv->zero();
 
-  IsValid = false;
-  NumNew = NumShifted = 0;
+  _isValid = false;
+  _numNew = _numShifted = 0;
 
-  Equation = in_equation;
+  setEquation(in_equation);
 }
 
 bool KstEquationCurve::isValid() const {
-  return IsValid;
+  return _isValid;
 }
 
 
@@ -217,7 +217,7 @@ KstObject::UpdateType KstEquationCurve::update(int update_counter) {
   MeanX= v->mean();
   MinPosX = v->minPos();
 
-  IsValid = FillY(force);
+  _isValid = FillY(force);
 
   v = *_yVector;
 
@@ -231,7 +231,7 @@ KstObject::UpdateType KstEquationCurve::update(int update_counter) {
   return UPDATE;
 }
 
-void KstEquationCurve::getPoint(int i, double &x, double &y) {
+void KstEquationCurve::point(int i, double &x, double &y) {
   x = (*_xVector)->interpolate(i, NS);
   y = (*_yVector)->value()[i];
 }
@@ -239,7 +239,7 @@ void KstEquationCurve::getPoint(int i, double &x, double &y) {
 void KstEquationCurve::save(QTextStream &ts) {
   ts << " <equation>" << endl;
   ts << "  <tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
-  ts << "  <equation>" << QStyleSheet::escape(Equation) << "</equation>" << endl;
+  ts << "  <equation>" << QStyleSheet::escape(_equation) << "</equation>" << endl;
   if (_staticX) {
     ts << "  <x0>" << QString::number((*_xVector)->min()) << "</x0>" << endl;
     ts << "  <x1>" << QString::number((*_xVector)->max()) << "</x1>" << endl;
@@ -247,7 +247,7 @@ void KstEquationCurve::save(QTextStream &ts) {
     ts << "  <xvtag>" << QStyleSheet::escape((*_xVector)->tagName()) << "</xvtag>" << endl;
   } else {
     ts << "  <xvector>" << QStyleSheet::escape((*_xVector)->tagName()) << "</xvector>" << endl;
-    if (DoInterp) {
+    if (_doInterp) {
       ts << "  <interpolate/>" << endl;
     }
   }
@@ -260,16 +260,36 @@ void KstEquationCurve::save(QTextStream &ts) {
 }
 
 void KstEquationCurve::setEquation(const QString& in_fn) {
-  Equation = in_fn;
+  _equation = in_fn;
   VectorsUsed.clear();
   NS = 2; // reset the updating
-  if (_pe) {
-    delete _pe;
-    _pe = 0L;
+  delete _pe;
+  _pe = 0L;
+  if (!_equation.isEmpty()) {
+    Equation::Context ctx;
+    ctx.sampleCount = NS;
+
+    yy_scan_string(_equation.latin1());
+    int rc = yyparse();
+    if (rc == 0) {
+      _pe = static_cast<Equation::Node*>(ParsedEquation);
+      Equation::FoldVisitor vis(&ctx, _pe);
+      _pe->collectVectors(VectorsUsed);
+      ParsedEquation = 0L;
+    } else {
+      delete (Equation::Node*)ParsedEquation;
+      ParsedEquation = 0L;
+    }
   }
+  _isValid = _pe != 0L;
 }
 
 void KstEquationCurve::setExistingXVector(KstVectorPtr in_xv, bool do_interp) {
+  KstVectorPtr v = _inputVectors[XVECTOR];
+  if (v) {
+    v->writeUnlock();
+  }
+
   if (_staticX) {
     _xVector = _inputVectors.end();
     KST::vectorList.lock().writeLock();
@@ -277,27 +297,37 @@ void KstEquationCurve::setExistingXVector(KstVectorPtr in_xv, bool do_interp) {
     KST::vectorList.lock().writeUnlock();
   }
 
+  _inputVectors.erase(XVECTOR);
+  in_xv->writeLock();
   _xVector = _inputVectors.insert(XVECTOR, in_xv);
 
   NS = 2; // reset the updating
   _staticX = false;
-  DoInterp = do_interp;
+  _doInterp = do_interp;
 }
 
-void KstEquationCurve::setStaticXVector(double xmin, double xmax, int n) {
+void KstEquationCurve::setStaticXVector(KstVectorPtr vp) {
+  KstVectorPtr v = _inputVectors[XVECTOR];
+  if (v) {
+    v->writeUnlock();
+  }
+
   if (_staticX) {
     _xVector = _inputVectors.end();
     KST::vectorList.lock().writeLock();
     KST::vectorList.remove(_inputVectors[XVECTOR]);
     KST::vectorList.lock().writeUnlock();
   }
+
+  _inputVectors.erase(XVECTOR);
   _staticX = true;
-  _xVector = _inputVectors.insert(XVECTOR, KstVector::generateVector(xmin, xmax, n, QString::null));
+  vp->writeLock();
+  _xVector = _inputVectors.insert(XVECTOR, vp);
 }
 
 
 QString KstEquationCurve::getYLabel() const {
-  return Equation;
+  return _equation;
 }
 
 QString KstEquationCurve::getXLabel() const {
@@ -333,36 +363,34 @@ bool KstEquationCurve::FillY(bool force) {
   int ns;
 
   // determine value of Interp
-  if (DoInterp) {
+  if (_doInterp) {
     ns = (*_xVector)->sampleCount();
     for (i_v = 0; i_v < VectorsUsed.count(); i_v++) {
       if (VectorsUsed[i_v]->sampleCount() > ns) {
         ns = VectorsUsed[i_v]->sampleCount();
       }
     }
-    if ((*_xVector)->sampleCount() > 0) {
-      Interp = (ns+1) / (*_xVector)->sampleCount();
-    } else { // avoid divide by zero
-      Interp = 1;
-    }
   } else {
-    Interp = 1;
+    ns = (*_xVector)->sampleCount();
   }
 
-  if (NS != (*_xVector)->sampleCount() || Interp != 1 ||
+  if (NS != (*_xVector)->sampleCount() || (ns != (*_xVector)->sampleCount()) ||
       (*_xVector)->numShift() != (*_xVector)->numNew()) {
-    NS = (*_xVector)->sampleCount()*Interp - 1;
+    NS = ns;
 
     KstVectorPtr yv = *_yVector;
-    yv->resize(NS);
+    if (!yv->resize(NS)) {
+      // FIXME: handle error?
+      return false;
+    }
     yv->zero();
     i0 = 0; // other vectors may have diffent lengths, so start over
     v_shift = NS;
   } else {
     // calculate shift and new samples
     // only do shift optimization if all used vectors are same size and shift
-    v_shift = (*_xVector)->numShift()*Interp;
-    v_new = (*_xVector)->numNew()*Interp;
+    v_shift = (*_xVector)->numShift();
+    v_new = (*_xVector)->numNew();
 
     for (i_v = 0; i_v < VectorsUsed.count(); i_v++) {
       if (v_shift != VectorsUsed[i_v]->numShift()) {
@@ -388,17 +416,17 @@ bool KstEquationCurve::FillY(bool force) {
     }
   }
 
-  NumShifted = (*_yVector)->numShift() + v_shift;
-  if (NumShifted > NS) {
-    NumShifted = NS;
+  _numShifted = (*_yVector)->numShift() + v_shift;
+  if (_numShifted > NS) {
+    _numShifted = NS;
   }
 
-  NumNew = NS - i0 + (*_yVector)->numNew();
-  if (NumNew > NS) {
-    NumNew = NS;
+  _numNew = NS - i0 + (*_yVector)->numNew();
+  if (_numNew > NS) {
+    _numNew = NS;
   }
 
-  (*_yVector)->SetNewAndShift(NumNew, NumShifted);
+  (*_yVector)->setNewAndShift(_numNew, _numShifted);
 
   double *rawv = (*_yVector)->value();
   KstVectorPtr iv = (*_xVector);
@@ -407,11 +435,11 @@ bool KstEquationCurve::FillY(bool force) {
   ctx.sampleCount = NS;
 
   if (!_pe) {
-    if (Equation.isEmpty()) {
+    if (_equation.isEmpty()) {
       return true;
     }
 
-    yy_scan_string(Equation.latin1());
+    yy_scan_string(_equation.latin1());
     int rc = yyparse();
     if (rc == 0) {
       _pe = static_cast<Equation::Node*>(ParsedEquation);
