@@ -46,14 +46,14 @@ public:
 
   virtual UpdateType update(int update_counter = -1);
 
-  virtual void getPoint(int i, double &x1, double &y1);
+  virtual void point(int i, double &x1, double &y1);
   virtual void save(QTextStream &ts);
   virtual QString propertyString() const;
 
   /** equations used to edit the vector */
   void setEquation(const QString &Equation);
   void setExistingXVector(KstVectorPtr xvector, bool do_interp);
-  void setStaticXVector(double xmin, double xmax, int n);
+  void setStaticXVector(KstVectorPtr v);
 
   bool isXStatic() const { return _staticX; }
 
@@ -62,11 +62,11 @@ public:
 
   virtual KstCurveType type() const;
 
-  virtual QString equation() const { return Equation; }
+  virtual QString equation() const { return _equation; }
   KstVectorPtr vX() { return _inputVectors[XVECTOR]; }
 
   virtual bool slaveVectorsUsed() const;
-  bool doInterp() const { return DoInterp; }
+  bool doInterp() const { return _doInterp; }
 
   bool isValid() const;
 
@@ -76,25 +76,23 @@ public:
 
   virtual void _showDialog();
 
+  virtual QString getXVTag() const {return (*_xVector)->tagName();}
+  virtual QString getYVTag() const {return (*_yVector)->tagName();}
+
 private:
-  QString Equation;
-
-  char String[1024];
-
-  bool _staticX;
+  QString _equation;
 
   KstVectorList VectorsUsed;
 
-  void commonConstructor(const QString &in_tag, const QString &equation,
-                         const QColor &in_color);
+  void commonConstructor(const QString& in_tag, const QString &equation,
+                         const QColor& in_color);
 
   bool FillY(bool force = false);
-  bool IsValid;
+  bool _isValid : 1;
+  bool _staticX : 1;
+  bool _doInterp : 1;
 
-  int NumNew, NumShifted;
-
-  bool DoInterp;
-  int Interp;
+  int _numNew, _numShifted, _interp;
 
   static const QString XVECTOR;
   static const QString OUTVECTOR;
