@@ -15,6 +15,10 @@
  *                                                                         *
  ***************************************************************************/
 
+// include files for KDE
+#include <kmdimainfrm.h>
+
+// application specific includes
 #include "kstextension.h"
 #include "kst.h"
 
@@ -23,6 +27,7 @@ KstExtension::KstExtension(QObject *parent, const char *name, const QStringList&
 
 
 KstExtension::~KstExtension() {
+  emit unregister();
 }
 
 
@@ -44,4 +49,5 @@ KstApp* KstExtension::app() const {
   return static_cast<KstApp*>(parent());
 }
 
+#include "kstextension.moc"
 // vim: ts=2 sw=2 et
