@@ -1,8 +1,9 @@
 /***************************************************************************
                      kstfitdialog_i.cpp  -  Part of KST
                              -------------------
-    begin                : Mon May 12 2003
-    copyright            : (C) 2003 The University of British Columbia
+    begin                : Wed Jul 28 2004
+    copyright            : (C) 2003 The University of Toronto
+                           (C) 2004 The University of British Columbia
     email                :
  ***************************************************************************/
 
@@ -25,6 +26,7 @@
 #include <qlayout.h>
 #include <qlineedit.h>
 #include <qlistview.h>
+#include <qmessagebox.h>
 #include <qobjectlist.h>
 #include <qpushbutton.h>
 #include <qspinbox.h>
@@ -34,32 +36,31 @@
 #include <qtooltip.h>
 #include <qwhatsthis.h>
 #include <qwidget.h>
-#include <qmessagebox.h>
+#include <qregexp.h>
 
 // include files for KDE
 #include <kcolorbutton.h>
 #include <kcolordialog.h>
 #include <kdebug.h>
 #include <klocale.h>
+#include <kmdimainfrm.h>
 #include <kmessagebox.h>
 #include <kstaticdeleter.h>
-#include <kmdimainfrm.h>
 
 // application specific includes
+#include "curveappearancewidget.h"
+#include "kst.h"
+#include "kst2dplot.h"
+#include "kstdatacollection.h"
+#include "kstdoc.h"
 #include "kstfitdialog_i.h"
 #include "kstvcurve.h"
-#include "kstfitcurve.h"
-#include "curveappearancewidget.h"
 #include "kstviewwindow.h"
-#include "kst2dplot.h"
-#include "kst.h"
 #include "plugincollection.h"
 #include "plugin.h"
-#include "kstdoc.h"
+#include "pluginmanager.h"
 #include "scalarselector.h"
 #include "vectorselector.h"
-#include "kstdatacollection.h"
-#include "pluginmanager.h"
 
 KstFitDialogI *KstFitDialogI::_inst = 0L;
 static KStaticDeleter<KstFitDialogI> _plFitInst;
@@ -73,15 +74,15 @@ KstFitDialogI *KstFitDialogI::globalInstance() {
 }
 
 
-const QString newFitPluginString = i18n("<New_Plugin>");
+const QString newFitPluginString = "<Auto Name>";
 
 KstFitDialogI::KstFitDialogI(QWidget* parent, const char* name,
-                                   bool modal, WFlags fl)
+                             bool modal, WFlags fl)
 : KstFitDialog(parent, name, modal, fl) {
-  connect(Select, SIGNAL(activated(int)), this, SLOT(updatePlugin(int)));
-  connect(New, SIGNAL(clicked()), this, SLOT(new_I()));
-  connect(Edit, SIGNAL(clicked()), this, SLOT(edit_I()));
-  connect(Delete, SIGNAL(clicked()), this, SLOT(delete_I()));
+  Init();
+
+  connect(_OK, SIGNAL(clicked()), this, SLOT(OK()));
+  connect(_cancel, SIGNAL(clicked()), this, SLOT(close()));
   connect(PluginCombo, SIGNAL(activated(int)), this, SLOT(pluginChanged(int)));
   connect(_pluginManager, SIGNAL(clicked()), this, SLOT(showPluginManager()));
 
@@ -94,22 +95,35 @@ KstFitDialogI::~KstFitDialogI() {
 }
 
 
-void KstFitDialogI::show_I(const QString &field) {
-  int new_index = -1;
-
-  KstPluginList plugins = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
-  new_index = plugins.findIndexTag(field);
-
-  updatePluginList();
-  updatePlugin(new_index);
-  show();
-  raise();
+void KstFitDialogI::Init() {
 }
 
 
-void KstFitDialogI::setCurve(const QString& strCurve, const QString& strPlotName, const QString& strWindow) {
-  KstVCurvePtr  curve;
-  KstVCurveList curves = kstObjectSubList<KstDataObject, KstVCurve>(KST::dataObjectList);
+void KstFitDialogI::OK() {
+  if (new_I()) {
+    close();
+  }
+}
+
+
+void KstFitDialogI::close() {
+  QDialog::close();
+}
+
+
+void KstFitDialogI::reject() {
+  QDialog::reject();
+}
+
+
+void KstFitDialogI::show_setCurve(const QString& strCurve,
+                                  const QString& strPlotName,
+                                  const QString& strWindow) {
+
+  KstBaseCurvePtr  curve;
+  KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
+  KstPluginList c = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
+  QString new_label;
 
   _strWindow   = strWindow;
   _strPlotName = strPlotName;
@@ -120,6 +134,17 @@ void KstFitDialogI::setCurve(const QString& strCurve, const QString& strPlotName
     _yvector = curve->getYVTag();
     _evector = curve->getYETag();
   }
+
+  if ( _xvector=="<None>" ) return;
+  if ( _yvector=="<None>" ) return;
+
+   _tagName->setText(newFitPluginString);
+
+  updatePluginList();
+  pluginChanged(PluginCombo->currentItem());
+
+  show();
+  raise();
 }
 
 
@@ -135,7 +160,7 @@ void KstFitDialogI::updatePluginList() {
   _pluginList.clear();
   for (it = _pluginMap.begin(); it != _pluginMap.end(); ++it) {
     if (it.data()._isFit) {
-      if (!it.data()._isFitWeighted || _evector != i18n("<None>")) {
+      if (!it.data()._isFitWeighted || _evector != "<None>") {
         _pluginList += it.data()._name;
         PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName)
                                 .arg(it.data()._version)
@@ -157,34 +182,21 @@ void KstFitDialogI::updatePluginList() {
 }
 
 
-void KstFitDialogI::show_New() {
-  updatePluginList();
-  updatePlugin(-2);
-  show();
-  raise();
-}
-
-void KstFitDialogI::update(int new_index) {
-  Q_UNUSED(new_index);
-
+void KstFitDialogI::update() {
   QString selectedVector;
   QString selectedScalar;
-  KstSharedPtr<Plugin> pPtr;
-
-  pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
-  if( pPtr != NULL ) {
+  KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
+  if (pPtr) {
     const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
-      if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
+      if ((*it)._type == Plugin::Data::IOValue::TableType) {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
-        assert(field);
         if (field) {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
           vs->update();
         }
       } else {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
-        assert(field);
         if (field) {
           ScalarSelector *ss = static_cast<ScalarSelector*>(field);
           ss->update();
@@ -194,154 +206,6 @@ void KstFitDialogI::update(int new_index) {
   }
 }
 
-void KstFitDialogI::setInputVectors( ) {
-  int iVector = 0;
-
-  KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
-  if( pPtr != 0 ) {
-    // Update input vector and scalar combos
-    const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
-    for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
-      if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
-        assert(field);
-        if (field) {
-          VectorSelector *vs = static_cast<VectorSelector*>(field);
-          switch( iVector) {
-            case 0:
-              vs->setSelection(_xvector);
-              vs->setEnabled(false);
-              break;
-            case 1:
-              vs->setSelection(_yvector);
-              vs->setEnabled(false);
-              break;
-            case 2:
-              vs->setSelection(_evector);
-              vs->setEnabled(false);
-              break;
-            default:
-              break;
-          }
-          iVector++;
-        }
-      }
-    }
-  }
-}
-
-void KstFitDialogI::updatePlugin(int new_index) {
-  QString selectedVector;
-  QString selectedScalar;
-  int oldSelect = Select->currentItem();
-  int oldCount = Select->count();
-
-  KstPluginList pl = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
-
-  Select->clear();
-
-  for (KstPluginList::iterator it = pl.begin(); it != pl.end(); ++it) {
-    Select->insertItem((*it)->tagName());
-  }
-
-  if (new_index == -2 || Select->count() == 0) {
-    Select->insertItem(i18n("P%1-%2").arg(Select->count()+1).arg(newFitPluginString));
-    Select->setCurrentItem(Select->count()-1);
-  } else if (new_index >= 0 && new_index < Select->count()) {
-    Select->setCurrentItem(new_index);
-  } else if (oldCount != Select->count()) {
-    Select->setCurrentItem(Select->count()-1);
-  } else {
-    Select->setCurrentItem(oldSelect < Select->count() ? oldSelect : Select->count()-1);
-  }
-
-  if (_pluginList.isEmpty()) {
-    updatePluginList();
-  }
-
-  KstPluginPtr plugin = pl[Select->currentItem()];
-  if (plugin) {
-    int oldPluginIndex = PluginCombo->currentItem();
-    int newPluginIndex = _pluginList.findIndex(plugin->plugin()->data()._name);
-    bool updateCombos = (newPluginIndex != oldPluginIndex);
-
-    if (updateCombos) {
-      PluginCombo->setCurrentItem(newPluginIndex);
-      pluginChanged(PluginCombo->currentItem());
-    }
-
-    KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
-    if( pPtr != 0 ) {
-      // Update input vector and scalar combos
-      const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
-      for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
-        if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
-          QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
-          assert(field);
-          if (field) {
-            VectorSelector *vs = static_cast<VectorSelector*>(field);
-            selectedVector = vs->selectedVector();
-            if (!updateCombos) {
-              vs->update();
-            }
-            setInputVectors( );
-          }
-        } else {
-          QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
-          assert(field);
-          if (field) {
-            ScalarSelector *ss = static_cast<ScalarSelector*>(field);
-            selectedScalar = ss->selectedScalar( );
-            if (!updateCombos) {
-              ss->update();
-            }
-            if (plugin.data() && plugin->isValid()) {
-              ss->setSelection(plugin->inputScalars()[(*it)._name]->tagName());
-            } else {
-              ss->setSelection(selectedScalar);
-            }
-          }
-        }
-      }
-
-      // Update output vector and scalar lineedits
-      //    const QValueList<Plugin::Data::IOValue>& otable = plugin->plugin()->data()._outputs;
-      if (plugin.data() && plugin->isValid()) {
-        const QValueList<Plugin::Data::IOValue>& otable = pPtr->data()._outputs;
-        for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
-          QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
-          assert(field);
-          if (field) {
-            QLineEdit *li = static_cast<QLineEdit*>(field);
-            QString ts;
-            if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
-              ts = plugin->outputVectors()[(*it)._name]->tagName();
-            } else { // scalar
-              ts = plugin->outputScalars()[(*it)._name]->tagName();
-            }
-            li->setText(ts);
-          }
-        }
-      }
-    } else { // invalid plugin
-      PluginCollection *pc = PluginCollection::self();
-      QString cur = _pluginList[PluginCombo->currentItem()];
-      Plugin::Data pdata = pc->pluginList()[pc->pluginNameList()[cur]];
-      for (QValueList<Plugin::Data::IOValue>::ConstIterator it = pdata._outputs.begin(); it != pdata._outputs.end(); ++it) {
-        QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
-        if (field) {
-          static_cast<QLineEdit*>(field)->setText(QString::null);
-        }
-      }
-    }
-
-    QTimer::singleShot(0, this, SLOT(fixupLayout()));
-  } else {
-    setInputVectors( );
-    kdWarning() << "Couldn't find plugin in list: " << Select->currentItem() << endl;
-  }
-}
-
 
 void KstFitDialogI::fixupLayout() {
   _frameWidget->resize(_frameWidget->sizeHint());
@@ -352,33 +216,79 @@ void KstFitDialogI::fixupLayout() {
 
 
 bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
+  int iInputVector = 0;
+
   KST::vectorList.lock().readLock();
   KST::scalarList.lock().readLock();
   const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
-      assert(field);
-      VectorSelector *vs = static_cast<VectorSelector*>(field);
-      KstVectorPtr v = *KST::vectorList.findTag(vs->selectedVector());
-      plugin->inputVectors().insert((*it)._name, v);
+      if (iInputVector < 2 || (plugin->plugin()->data()._isFitWeighted && iInputVector < 3) ) {
+        KstVectorPtr v;
+
+        switch (iInputVector) {
+          case 0:
+            v = *KST::vectorList.findTag(_xvector);
+            break;
+          case 1:
+            v = *KST::vectorList.findTag(_yvector);
+            break;
+          case 2:
+            v = *KST::vectorList.findTag(_evector);
+            break;
+        }
+        if (v) {
+          v->writeLock(); // to match with plugin->writeLock()
+        }
+        if (plugin->inputVectors().contains((*it)._name)) {
+          plugin->inputVectors()[(*it)._name]->writeUnlock();
+        }
+        plugin->inputVectors().insert((*it)._name, v);
+        iInputVector++;
+      } else {
+        QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
+        if (field) {
+          VectorSelector *vs = static_cast<VectorSelector*>(field);
+          KstVectorPtr v = *KST::vectorList.findTag(vs->selectedVector());
+          if (v) {
+            v->writeLock(); // to match with plugin->writeLock()
+          }
+          if (plugin->inputVectors().contains((*it)._name)) {
+            plugin->inputVectors()[(*it)._name]->writeUnlock();
+          }
+          plugin->inputVectors().insert((*it)._name, v);
+        }
+      }
     } else {
       QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
-      assert(field);
-      ScalarSelector *ss = static_cast<ScalarSelector*>(field);
-      KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
-      if (s == *KST::scalarList.end()) {
-        bool ok;
-        double val = ss->_scalar->currentText().toDouble(&ok);
+      if (field) {
+        ScalarSelector *ss = static_cast<ScalarSelector*>(field);
+        KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
+        if (s == *KST::scalarList.end()) {
+          bool ok;
+          double val = ss->_scalar->currentText().toDouble(&ok);
 
-        if (ok) {
-          KstScalarPtr pSNew = new KstScalar(ss->_scalar->currentText(), val, true, false, false);
-          plugin->inputScalars().insert((*it)._name, pSNew);
+          if (ok) {
+            KstScalarPtr newScalar = new KstScalar(ss->_scalar->currentText(), val, true, false, false);
+            newScalar->writeLock(); // to match with plugin->writeLock()
+            if (plugin->inputScalars().contains((*it)._name)) {
+              plugin->inputScalars()[(*it)._name]->writeUnlock();
+            }
+            plugin->inputScalars().insert((*it)._name, newScalar);
+          } else {
+            s->writeLock(); // to match with plugin->writeLock()
+            if (plugin->inputScalars().contains((*it)._name)) {
+              plugin->inputScalars()[(*it)._name]->writeUnlock();
+            }
+            plugin->inputScalars().insert((*it)._name, s);
+          }
         } else {
+          s->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputScalars().contains((*it)._name)) {
+            plugin->inputScalars()[(*it)._name]->writeUnlock();
+          }
           plugin->inputScalars().insert((*it)._name, s);
         }
-      } else {
-        plugin->inputScalars().insert((*it)._name, s);
       }
     }
   }
@@ -399,8 +309,8 @@ bool KstFitDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
     QLineEdit *li = static_cast<QLineEdit*>(field);
 
     if (li->text().isEmpty()) {
-      QString tagName = Select->currentText();
-      if (tagName.endsWith(newFitPluginString)) {
+      QString tagName = _tagName->text();
+      if (tagName == newFitPluginString) {
         tagName = plugin->tagName();
       }
       li->setText(tagName+"-"+(*it)._name);
@@ -436,55 +346,74 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
   KstVectorPtr xVector;
   KstVectorPtr yVector;
   KstVectorPtr wVector;
+  KstLabel* label;
   KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
-  KstFitCurvePtr fit;
+  KstVCurvePtr fit;
   QString c_name;
+  QString strLabel;
   bool bRetVal = false;
 
   // Generate new fit name
-  c_name = "F" + QString::number(curves.count()+1) + "-" + _strCurve;
-  while (curves.findTag(c_name) != curves.end()) {
-    c_name += "'";
-  }
-  while (KST::vectorList.findTag(c_name) != KST::vectorList.end()) {
-    c_name += "'";
-  }
+  c_name = KST::suggestCurveName(plugin->tagName());
 
-  if(plugin->inputVectors().find("X Array") != plugin->inputVectors().end()) {
+  if (plugin->inputVectors().find("X Array") != plugin->inputVectors().end()) {
     xVector = plugin->inputVectors()["X Array"];
   }
-  if(plugin->outputVectors().find("Y Fitted") != plugin->outputVectors().end()) {
+  if (plugin->outputVectors().find("Y Fitted") != plugin->outputVectors().end()) {
     yVector = plugin->outputVectors()["Y Fitted"];
   }
-  if (!_evector.isEmpty() && _evector.compare(i18n("<None>")) != 0) {
-    if(plugin->outputVectors().find("Y Fitted") != plugin->inputVectors().end()) {
-      wVector = plugin->inputVectors()["Weight Array"];
-    }
-  }
 
   if (xVector && yVector) {
     plugin->update(-1);
-    fit = new KstFitCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(wVector), _curveAppearance->color());
+    fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), _curveAppearance->color());
     if (fit) {
-      fit->setPlugin(plugin);
+      Kst2DPlotPtr plot;
+
       fit->setHasPoints(_curveAppearance->showPoints());
       fit->setHasLines(_curveAppearance->showLines());
       fit->setLineWidth(_curveAppearance->lineWidth());
       fit->setLineStyle(_curveAppearance->lineStyle());
       fit->Point.setType(_curveAppearance->pointType());
 
-      Kst2DPlotPtr plot;
-      KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_strWindow));
-      if (w) {
-        if( w->view()->findChild(_strPlotName) ) {
-          plot = dynamic_cast<Kst2DPlot*>((w->view()->findChild(_strPlotName)).data());
-          if (plot) {
-            plot->addCurve(fit.data());
-            w->view()->paint(P_DATA);
+      //
+      // create the necessary label...
+      //
+      strLabel = i18n("%2: %1\\n").
+                 arg(_strCurve.replace( '_',  "\\_" ).remove( QRegExp( "^[C][0-9]{1,2}\\-" ) )).
+                 arg(plugin->plugin()->data()._readableName);
+
+       KstVectorPtr vectorParam = plugin->outputVectors()["Parameters"];
+      if( vectorParam ) {
+        QString strParamName;
+
+        for( int i=0; i<vectorParam->length(); i++ ) {
+          strParamName = plugin->plugin()->parameterName(i);
+          if( !strParamName.isEmpty() ) {
+            if (plugin->outputScalars().contains(strParamName)) {
+              KstScalarPtr scalar = plugin->outputScalars()[strParamName];
+
+              if (scalar) {
+                strLabel += i18n("%1: [%2]\\n").arg(strParamName).arg(scalar->tagName());
+              }
+            }
           }
         }
       }
 
+      //
+      // add the label and the curve to the plot...
+      //
+      KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_strWindow));
+      if (w && w->view()->findChild(_strPlotName)) {
+        plot = dynamic_cast<Kst2DPlot*>((w->view()->findChild(_strPlotName)).data());
+        if (plot) {
+          label = new KstLabel(strLabel, LxTy, 0.0, 0.05, 0.05);
+          plot->addLabel(label);
+          plot->addCurve(fit.data());
+          w->view()->paint(P_DATA);
+        }
+      }
+
       KST::dataObjectList.lock().writeLock();
       KST::dataObjectList.append(fit.data());
       KST::dataObjectList.lock().writeUnlock();
@@ -497,133 +426,74 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
 }
 
 
-void KstFitDialogI::new_I() {
-  KstPluginPtr plugin;
-  QString tagName = Select->currentText();
+bool KstFitDialogI::new_I() {
+  QString tagName = _tagName->text();
 
-  if (KST::dataTagNameNotUnique(tagName, true, this)) {
-    Select->setFocus();
-  } else {
-    int pitem = PluginCombo->currentItem();
-
-    if (pitem >= 0 && PluginCombo->count() > 0) {
-      KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
-      if (pPtr != NULL) {
-        plugin = new KstPlugin;
-        if( plugin ) {
-          if (saveInputs(plugin, pPtr)) {
-            plugin->setPlugin(pPtr);
-
-            if (tagName.endsWith(newFitPluginString)) {
-              QString tmpTagName;
-              int i = 0;
-
-              do {
-                tmpTagName = i18n("P%1-%2").arg(++i).arg(_pluginList[pitem]);
-              } while (KST::dataTagNameNotUnique(tmpTagName, false));
-              tagName = tmpTagName;
-            }
-
-            plugin->setTagName(tagName);
-            if (saveOutputs(plugin, pPtr)) {
-              KST::dataObjectList.lock().writeLock();
-              KST::dataObjectList.append(plugin.data());
-              KST::dataObjectList.lock().writeUnlock();
-
-              if (plugin->isValid()) {
-                if (!createCurve(plugin)) {
-                  KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
-                }
-              } else {
-                KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
-              }
-            }
-          }
-          plugin = 0L;
-        } else {
-          KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
-        }
-      }
+  if (tagName != newFitPluginString) {
+    if (KST::dataTagNameNotUnique(tagName, true, this)) {
+      _tagName->setFocus();
+      return( false );
     }
   }
-  emit modified();
-}
 
+  int pitem = PluginCombo->currentItem();
 
-void KstFitDialogI::edit_I() {
-  int index = Select->currentItem();
+  if (pitem >= 0 && PluginCombo->count() > 0) {
+    KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
 
-  if (Select->count() != 0 && index >= 0) {
-    QString tagName = Select->currentText();
-    KstPluginList pl = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
-    KstPluginPtr plugin = pl[index];
+    if (pPtr) {
+      KstPluginPtr plugin = new KstPlugin;
+      plugin->writeLock();
+      if (saveInputs(plugin, pPtr)) {
+        plugin->setPlugin(pPtr);
 
-    if (plugin.data()) {
-      if( plugin->isValid()) {
-        int pitem = PluginCombo->currentItem();
-
-        if (tagName.endsWith(newFitPluginString)) {
-          QString tmpTagName;
-          int i = 0;
-
-          do {
-            tmpTagName = i18n("P%1-%2").arg(++i).arg(_pluginList[pitem]);
-          } while (KST::dataTagNameNotUnique(tmpTagName, false));
-          tagName = tmpTagName;
+        if (tagName == newFitPluginString) {
+          tagName = KST::suggestPluginName(_pluginList[pitem], _strCurve);
         }
 
-        if (tagName != plugin->tagName() && KST::dataTagNameNotUnique(tagName, true, this)) {
-          Select->setFocus();
-        } else {
-          plugin->setTagName(tagName);
-          plugin->inputVectors().clear();
-          plugin->inputScalars().clear();
+        plugin->setTagName(tagName);
+        if (saveOutputs(plugin, pPtr)) {
+          plugin->writeUnlock();
+          KST::dataObjectList.lock().writeLock();
+          KST::dataObjectList.append(plugin.data());
+          KST::dataObjectList.lock().writeUnlock();
 
-          KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
-
-          // Save the vectors and scalars
-          if (saveInputs(plugin, pPtr)) {
-            if (pitem >= 0 && PluginCombo->count() > 0) {
-              plugin->setPlugin(pPtr);
-            }
-
-            if (saveOutputs(plugin, pPtr)) {
-              emit modified();
+          plugin->readLock();
+          if (plugin->isValid()) {
+            if (!createCurve(plugin)) {
+              KMessageBox::sorry(this, i18n("Could not create curve from fit, possibly due to a bad plugin."));
+              plugin->readUnlock();
+              plugin = 0l;
+              return false;
             }
+            plugin->readUnlock();
+          } else {
+            KMessageBox::sorry(this, i18n("There is something wrong (i.e, there is a bug) with the creation of the fit plugin."));
+            plugin->readUnlock();
+            plugin = 0l;
+            return false;
           }
+        } else {
+          plugin->writeUnlock();
         }
       } else {
-        KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
+        plugin->writeUnlock();
       }
     } else {
-      new_I();
+      KMessageBox::sorry(this, i18n("There is something wrong (i.e, there is a bug) with"
+                                    " the selected plugin.\n"));
+      return false;
     }
-    plugin = 0L; // release
-    pl.clear(); // release
-  } else {
-    new_I();
+
+    emit modified();
   }
-  emit modified();
+  return true;
 }
 
 
-void KstFitDialogI::delete_I() {
-  QString tag = Select->currentText();
-
-  if (tag.isEmpty()) {
-    KMessageBox::sorry(this, i18n("You need to select an active plugin to delete."));
-  } else {
-    KST::dataObjectList.lock().writeLock();
-    KST::dataObjectList.removeTag(tag);
-    KST::dataObjectList.lock().writeUnlock();
-  }
-
-  emit modified();
-}
-
-
-void KstFitDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
-QString scalarLabelTemplate, vectorLabelTemplate;
+void KstFitDialogI::generateEntries(bool isWeighted, bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
+  QString scalarLabelTemplate, vectorLabelTemplate;
+  int iInputVector = 0;
 
   if (input) {
     scalarLabelTemplate = i18n("Input Scalar - %1:");
@@ -636,13 +506,18 @@ QString scalarLabelTemplate, vectorLabelTemplate;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
     QString labellabel;
     bool scalar = false;
+    bool fixed = false;
     switch ((*it)._type) {
       case Plugin::Data::IOValue::FloatType:
         labellabel = scalarLabelTemplate.arg((*it)._name);
         scalar = true;
         break;
       case Plugin::Data::IOValue::TableType:
-        if ((*it)._subType == Plugin::Data::IOValue::FloatSubType) {
+        if (input && ( iInputVector < 2 || ( isWeighted && iInputVector < 3 ) ) ) {
+          fixed = true;
+        }
+        if ((*it)._subType == Plugin::Data::IOValue::FloatSubType ||
+            (*it)._subType == Plugin::Data::IOValue::FloatNonVectorSubType) {
           labellabel = vectorLabelTemplate.arg((*it)._name);
         } else {
           // unsupported
@@ -655,7 +530,6 @@ QString scalarLabelTemplate, vectorLabelTemplate;
     }
 
     QLabel *label = new QLabel(labellabel, parent, input ? "Input label" : "Output label");
-
     QWidget *widget = 0L;
 
     if (input) {
@@ -670,8 +544,24 @@ QString scalarLabelTemplate, vectorLabelTemplate;
           QToolTip::add(w->_scalar, QString::number(p->value()));
         }
       } else {
-        widget = new VectorSelector(parent, (*it)._name.latin1());
-        connect(widget, SIGNAL(newVectorCreated(const QString&)), this, SIGNAL(modified()));
+        if (fixed) {
+          widget = new QLabel(parent, (*it)._name.latin1());
+          switch (iInputVector) {
+            case 0:
+              dynamic_cast<QLabel*>(widget)->setText(_xvector);
+              break;
+            case 1:
+              dynamic_cast<QLabel*>(widget)->setText(_yvector);
+              break;
+            case 2:
+              dynamic_cast<QLabel*>(widget)->setText(_evector);
+              break;
+          }
+          iInputVector++;
+        } else {
+          widget = new VectorSelector(parent, (*it)._name.latin1());
+          connect(widget, SIGNAL(newVectorCreated(const QString&)), this, SIGNAL(modified()));
+        }
       }
     } else {
       widget = new QLineEdit(parent, (*it)._name.latin1());
@@ -709,42 +599,43 @@ void KstFitDialogI::pluginChanged(int idx) {
   if (idx >= 0 && PluginCombo->count() > 0) {
     const QString& pluginName = _pluginList[idx];
     const Plugin::Data& pluginData = PluginCollection::self()->pluginList()[PluginCollection::self()->pluginNameList()[pluginName]];
-    int cnt = 3;
+    int cnt = 0;
 
     // Setup the grid and the "static" entries
     int variables = pluginData._inputs.count() + pluginData._outputs.count();
     QGridLayout *grid = new QGridLayout(_frameWidget, 4+variables, 2);
     grid->setMargin(6);
     grid->setSpacing(5);
-    grid->addWidget(new QLabel(i18n("Plugin Name:"), _frameWidget), 0, 0);
-    grid->addWidget(new QLabel(pluginData._name, _frameWidget), 0, 1);
-    grid->addWidget(new QLabel(i18n("Description:"), _frameWidget), 1, 0);
+    grid->addWidget(new QLabel(i18n("Plugin Name:"), _frameWidget), cnt, 0);
+    grid->addWidget(new QLabel(pluginData._readableName, _frameWidget), cnt, 1);
+    grid->addWidget(new QLabel(i18n("Description:"), _frameWidget), ++cnt, 0);
     QLabel *lab = new QLabel(pluginData._description, _frameWidget);
     lab->setAlignment(lab->alignment() | Qt::WordBreak);
-    grid->addWidget(lab, 1, 1);
+    grid->addWidget(lab, cnt, 1);
 
     // Add a separator
     QFrame* line = new QFrame(_frameWidget);
     line->setFrameShadow(QFrame::Sunken);
     line->setFrameShape(QFrame::HLine);
-    grid->addMultiCellWidget(line, 2, 2, 0, 1);
+    cnt++;
+    grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
 
     // Generate the input values
-    generateEntries(true, cnt, _frameWidget, grid, pluginData._inputs);
-    setInputVectors( );
+    cnt++;
+    generateEntries(pluginData._isFitWeighted, true, cnt, _frameWidget, grid, pluginData._inputs);
 
     if (!pluginData._inputs.isEmpty() && !pluginData._outputs.isEmpty()) {
-      // Add a separator
       line = new QFrame(_frameWidget);
       line->setFrameShape(QFrame::HLine);
       line->setFrameShadow(QFrame::Sunken);
       line->setFrameShape(QFrame::HLine);
-      grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
       cnt++;
+      grid->addMultiCellWidget(line, cnt, cnt, 0, 1);
     }
 
     // Generate the output values
-    generateEntries(false, cnt, _frameWidget, grid, pluginData._outputs);
+    cnt++;
+    generateEntries(pluginData._isFitWeighted, false, cnt, _frameWidget, grid, pluginData._outputs);
   }
 
   // show it
@@ -757,9 +648,11 @@ void KstFitDialogI::pluginChanged(int idx) {
 
 void KstFitDialogI::showPluginManager() {
   PluginManager *pm = new PluginManager(this, "Plugin Manager");
-  pm->exec();
-  delete pm;
-  updatePluginList();
+  if (pm) {
+    pm->exec();
+    delete pm;
+    updatePluginList();
+  }
 }
 
 
