@@ -22,26 +22,27 @@
 
 class KstGraphFileDialogI : public KstGraphFileDialog {
   Q_OBJECT
-public:
-  KstGraphFileDialogI(QWidget* parent = 0, const char* name = 0,
-                     bool modal = FALSE, WFlags fl = 0 );
-  virtual ~KstGraphFileDialogI();
+  public:
+    KstGraphFileDialogI(QWidget* parent = 0, const char* name = 0,
+        bool modal = false, WFlags fl = 0 );
+    virtual ~KstGraphFileDialogI();
 
-public slots:
-  /** shows/raises the dialog */
-  void showGraphFileDialog();
-  void reqGraphFile();
+  public slots:
+    /** shows/raises the dialog */
+    void showGraphFileDialog();
+    void reqGraphFile();
 
-private slots:
-  void xsizeChanged(int x);
-  void setAutoSave();
-  void square();
+  private slots:
+    void xsizeChanged(int x);
+    void setAutoSave();
+    void square();
 
-private:
-  QTimer *autoSaveTimer;
+  private:
+    QTimer *autoSaveTimer;
 
-signals:
-  void graphFileReq(const QString &filename, const QString &format, int w, int h, bool bAll);
+  signals:
+    void graphFileReq(const QString &filename, const QString &format, int w, int h, bool all);
 };
 
 #endif
+// vim: ts=2 sw=2 et
