@@ -41,7 +41,7 @@ public:
 
   virtual UpdateType update(int update_counter = -1);
 
-  virtual void getPoint(int i, double &x1, double &y1);
+  virtual void point(int i, double &x1, double &y1);
   virtual void save(QTextStream &ts);
   virtual QString propertyString() const;
 
@@ -76,6 +76,11 @@ public:
 
   virtual void _showDialog();
 
+  virtual bool slaveVectorsUsed() const;
+
+  virtual QString getXVTag() const {return (*_bVector)->tagName();}
+  virtual QString getYVTag() const {return (*_hVector)->tagName();}
+
 private:
   KstVectorPtr V;
 
@@ -92,9 +97,12 @@ private:
                          const QColor &in_color);
 
   KstHsNormType NormMode;
+
+  KstVectorMap::Iterator _bVector, _hVector;
 };
 
 typedef KstSharedPtr<KstHistogram> KstHistogramPtr;
 typedef KstObjectList<KstHistogramPtr> KstHistogramList;
 
 #endif
+// vim: ts=2 sw=2 et
