@@ -40,8 +40,6 @@ k_dcop:
   virtual QStringList plotList(const QString& window) = 0;
   virtual QStringList pluginList() = 0;
   virtual QStringList filterList() = 0;
-  virtual QStringList filterSetList() = 0;
-  virtual QStringList filterSetContents(const QString& filter) = 0;
 
 
   virtual QStringList inputVectors(const QString& objectName) = 0;
@@ -190,6 +188,36 @@ k_dcop:
   virtual QStringList createPlugin(const QString& pluginName,
                             const QStringList& scalarInputs,
                             const QStringList& vectorInputs) = 0;
+
+  //create new KST matrix
+  //returns the name of the matrix, or empty QString if not successful
+  //if useMaxX is true, xNumSteps is ignored
+  //if useMaxX is false, xNumSteps * yNumSteps must be <= length of vector
+  //xNumSteps can = 0, yNumSteps must be >= 1
+  virtual QString createMatrix(const QString& name, const QString& vector,
+                              double xMin, double yMin,
+                              double xStepSize, double yStepSize,
+                              uint xNumSteps, uint yNumSteps,
+                              bool useMaxX) = 0;
+
+  //create a new KST image
+  //returns the name of the image, or empty QString if not successful
+  //imageType 0 = colormap only
+  //          1 = contour map only
+  //          2 = both colormap and contour map
+  //if imageType == 0 or 1, unnneeded parameters are ignored
+  virtual QString createImage(const QString &name,
+                              const QString &matrix,
+                              double lowerZ,
+                              double upperZ,
+                              const QString &paletteName,
+                              int numContours,
+                              const QColor& contourColor,
+                              uint imageType) = 0;
+
+  //create a plot marker
+  //returns false if plotName is not valid or markerValue already exists, else returns true
+  virtual bool addPlotMarker(const QString &plotName, double markerValue) = 0;
 };
 
 #endif
