@@ -31,17 +31,21 @@
 #include "kst2dplot.h"
 #include "kstcolorsequence.h"
 #include "kstdatacollection.h"
+#include "kstdefaultnames.h"
 #include "kstdoc.h"
 #include "kstequationcurve.h"
+#include "ksteventmonitorentry.h"
+#include "ksthistogram.h"
 #include "kstiface_impl.h"
+#include "kstimage.h"
+#include "kstmatrix.h"
+#include "kstplugin.h"
+#include "kstpsdcurve.h"
 #include "kstscalar.h"
 #include "kstvcurve.h"
 #include "kstvector.h"
 #include "kstviewwindow.h"
 #include "plugincollection.h"
-#include "ksthistogram.h"
-#include "kstpsdcurve.h"
-#include "ksteventmonitorentry.h"
 
 KstIfaceImpl::KstIfaceImpl(KstDoc *doc, KstApp *app)
 : DCOPObject("KstIface"), _doc(doc), _app(app) {
@@ -149,7 +153,7 @@ QStringList KstIfaceImpl::pluginList() {
   QMap<QString,Plugin::Data>::ConstIterator it;
 
   for (it = pluginList.begin(); it != pluginList.end(); ++it) {
-    if (it.data()._filter == false) {
+    if (it.data()._isFilter == false) {
       rc += it.data()._name;
     }
   }
@@ -165,7 +169,7 @@ QStringList KstIfaceImpl::filterList() {
   QMap<QString,Plugin::Data>::ConstIterator it;
 
   for (it = pluginList.begin(); it != pluginList.end(); ++it) {
-    if (it.data()._filter) {
+    if (it.data()._isFilter) {
       rc += it.data()._name;
     }
   }
@@ -174,37 +178,6 @@ QStringList KstIfaceImpl::filterList() {
 }
 
 
-QStringList KstIfaceImpl::filterSetList() {
-  QStringList rc;
-  KstReadLocker ml(&KST::filterSetList.lock());
-
-  for (KstFilterSetList::Iterator it = KST::filterSetList.begin(); it != KST::filterSetList.end(); ++it) {
-    (*it)->lock().readLock();
-    rc += (*it)->name();
-    (*it)->lock().readUnlock();
-  }
-
-  return rc;
-}
-
-
-QStringList KstIfaceImpl::filterSetContents(const QString& filter) {
-  QStringList rc;
-  KstReadLocker ml(&KST::filterSetList.lock());
-
-  KstFilterSetPtr fs = KST::filterSetList.find(filter);
-  if (fs) {
-    for (KstFilterSet::Iterator it = fs->begin(); it != fs->end(); ++it) {
-      (*it)->readLock();
-      rc += (*it)->tagName();
-      (*it)->readUnlock();
-    }
-  }
-
-  return rc;
-}
-
-
 bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const QString& equation, const QString& plotName, const QColor& color) {
   Kst2DPlotPtr plot;
   QString etag, ptag;
@@ -555,8 +528,10 @@ bool KstIfaceImpl::resizeVector(const QString& name, int newSize) {
 
   if (it != KST::vectorList.end()) {
     (*it)->writeLock();
-    (*it)->resize(newSize);
-    bool rc = (*it)->length() == newSize;
+    bool rc = (*it)->resize(newSize);
+    if (rc) {
+      rc = (*it)->length() == newSize;
+    }
     (*it)->writeUnlock();
     return rc;
   }
@@ -609,27 +584,7 @@ QString KstIfaceImpl::createPlot(const QString& window, const QString& name) {
   if (pWindow) {
     pView = dynamic_cast<KstViewWindow*>(pWindow);
     if (pView) {
-      pTLV = pView->view();
-      Kst2DPlotList plots = pTLV->findChildrenType<Kst2DPlot>(true);
-
-      if (plots.findTag(name) != plots.end()) {
-        //suggestPlotName does not check existing plot names
-        //plotName = KST::suggestPlotName();
-        int i = plots.count() + 1;
-        QString stringnum;
-        stringnum = stringnum.setNum(i);
-        plotName = "P" + stringnum;
-        while (plots.findTag(plotName) != plots.end()) {
-          stringnum = stringnum.setNum(++i);
-          plotName = "P" + stringnum;
-        }
-      } else {
-        plotName = name;
-      }
-
-      p = pTLV->createPlot<Kst2DPlot>(plotName);
-      plotName = p->tagName();
-
+      plotName = pView->createPlot<Kst2DPlot>(name, false);
       _doc->forceUpdate();
       _doc->setModified();
     }
@@ -901,7 +856,7 @@ const QString& KstIfaceImpl::loadVector(const QString& file, const QString& fiel
       KST::dataSourceList.lock().writeUnlock();
       return QString::null;
     }
-    if (src->frameCount() < 1) {
+    if (src->isEmpty()) {
       KST::dataSourceList.lock().writeUnlock();
       return QString::null;
     }
@@ -1334,7 +1289,7 @@ QStringList KstIfaceImpl::createPlugin(const QString& pluginName,
   QStringList::ConstIterator scalarParamIter;
   QStringList::ConstIterator vectorParamIter;
 
-  KstPluginPtr kstplug_ptr = new KstPlugin();
+  KstPluginPtr kstplug_ptr = new KstPlugin;
 
   //iterate through inputs
   IOIter = plug_data._inputs.begin();
@@ -1457,5 +1412,162 @@ QStringList KstIfaceImpl::createPlugin(const QString& pluginName,
   return outputList;
 }
 
+QString KstIfaceImpl::createMatrix(const QString& name, const QString& vector, double xMin, double yMin,
+                                                   double xStepSize, double yStepSize,
+                                                   uint xNumSteps, uint yNumSteps,
+                                                   bool useMaxX) {
+  //get the vector
+  KST::vectorList.lock().readLock();
+  KstVectorList::Iterator iter = KST::vectorList.findTag(vector);
+  KST::vectorList.lock().readUnlock();
 
+  if (iter == KST::vectorList.end()) {
+    return QString();
+  }
+
+  //suggest a name if not supplied
+  QString artag;
+  if (name.isEmpty()) {
+    artag = KST::suggestMatrixName(vector);
+  } else {
+    QString artag_end = QString(name);
+    //count number of data objects and make a unique name
+    int i = KST::dataObjectList.count() + 1;
+    QString artag = "A" + QString::number(i) + "-" + artag_end;
+  }
+  while (KST::dataTagNameNotUnique(artag, false)) {
+    artag += "\'";
+  }
+
+  //basic check to see if parameters are consistent with vector
+  (*iter)->readLock();
+  if (!useMaxX && (*iter)->sampleCount() < xNumSteps * yNumSteps) {
+    (*iter)->readUnlock();
+    return QString();
+  }
+
+  //can't let yNumSteps be less than 1, or will divide by 0
+  if (yNumSteps < 1) {
+    (*iter)->readUnlock();
+    return QString();
+  }
+
+  if (xStepSize <= 0 || yStepSize <= 0) {
+    (*iter)->readUnlock();
+    return QString();
+  }
+  (*iter)->readUnlock();
+
+  //create the matrix
+  KstMatrixPtr matrix = new KstMatrix(artag, *iter, xNumSteps, yNumSteps, xMin, yMin, xStepSize, yStepSize, useMaxX);
+
+  KST::dataObjectList.lock().writeLock();
+  KST::dataObjectList.append(KstDataObjectPtr(matrix));
+  KST::dataObjectList.lock().writeUnlock();
+
+  _doc->forceUpdate();
+  _doc->setModified();
+
+  return artag;
+}
+
+QString KstIfaceImpl::createImage(const QString &name,
+                                  const QString &in_matrix,
+                                  double lowerZ,
+                                  double upperZ,
+                                  const QString &paletteName,
+                                  int numContours,
+                                  const QColor& contourColor,
+                                  uint imageType) {
+  //get the matrix
+  KstMatrixList matrices = kstObjectSubList<KstDataObject, KstMatrix>(KST::dataObjectList);
+  KstMatrixPtr matrix = *matrices.findTag(in_matrix);
+  if (!matrix) {
+    return QString::null;
+  }
+
+  //make a name if necessary
+  QString imgtag;
+  if (name.isEmpty()) {
+    imgtag = KST::suggestImageName(in_matrix);
+  } else {
+    QString imgtag_end = QString(name);
+    //count number of data objects and make a unique name
+    int i = KST::dataObjectList.count() + 1;
+    QString imgtag = "Img" + QString::number(i) + "-" + imgtag_end;
+  }
+  while (KST::dataTagNameNotUnique(imgtag, false)) {
+    imgtag += "\'";
+  }
+
+  //determine the image type
+  KstImagePtr image;
+  if (imageType == 0) {
+    //need a colormap
+    if (lowerZ > upperZ) {
+      return QString::null;
+    }
+    KPalette* pal = new KPalette(paletteName);
+    matrix->readLock();
+    image = new KstImage(imgtag, matrix, lowerZ, upperZ, pal);
+    matrix->readUnlock();
+  } else if (imageType == 1) {
+    //need a contourmap
+    if (numContours < 1) {
+      return QString::null;
+    }
+    matrix->readLock();
+    image = new KstImage(imgtag, matrix, numContours, contourColor.isValid() ? contourColor : QColor("darkBlue"));
+    matrix->readUnlock();
+  } else if (imageType == 2) {
+    //need both contourmap and colormap
+    if (lowerZ > upperZ) {
+      return QString::null;
+    }
+    KPalette* pal = new KPalette(paletteName);
+    if (!pal) {
+      return QString::null;
+    }
+    if (numContours < 1) {
+      return QString::null;
+    }
+    matrix->readLock();
+    image = new KstImage(imgtag, matrix, lowerZ, upperZ, pal,
+                         numContours, contourColor.isValid() ? contourColor : QColor("darkBlue"));
+    matrix->readUnlock();
+  } else {
+    return QString::null;
+  }
+
+  KST::dataObjectList.lock().writeLock();
+  KST::dataObjectList.append(KstDataObjectPtr(image));
+  KST::dataObjectList.lock().writeUnlock();
+
+  _doc->forceUpdate();
+  _doc->setModified();
+
+  return imgtag;
+
+}
+
+bool KstIfaceImpl::addPlotMarker(const QString &plotName, double markerValue) {
+  //find the plot
+  KstApp *app = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *iter = app->createIterator();
+  while (iter->currentItem()) {
+    KMdiChildView *childview = iter->currentItem();
+    KstViewWindow *viewwindow = dynamic_cast<KstViewWindow*>(childview);
+    if (viewwindow) {
+      Kst2DPlotList plotlist = viewwindow->view()->findChildrenType<Kst2DPlot>(false);
+      Kst2DPlotList::Iterator plot_iter=plotlist.findTag(plotName);
+      if (plot_iter != plotlist.end() && (*plot_iter)->setPlotMarker(markerValue)) {
+        app->deleteIterator(iter);
+        return true;
+      }
+    }
+    iter->next();
+  }
+  app->deleteIterator(iter);
+  return false;
+}
 // vim: ts=2 sw=2 et
