@@ -18,11 +18,8 @@
 #ifndef KSTLABEL_H
 #define KSTLABEL_H
 
-#include "kstscalar.h"
-#include <qdom.h>
 #include <qdragobject.h>
 #include <qpainter.h>
-#include <qstring.h>
 
 #include "kstplotbase.h"
 
@@ -47,16 +44,26 @@ public:
   void setRelPosition(float new_x, float new_y);
   void offsetRelPosition(float offset_x, float offset_y);
   void setJustification(KstJustifyType Justify);
-
+  void setResized( );
+  
   KstJustifyType justification() const {return Justify;}
 
   /** Return width and height of the label (from font metrics) */
   int width(QPainter &p);
   int ascent(QPainter &p);
+  int rotatedWidth(QPainter &p);
+  int rotatedHeight(QPainter &p);
   int lineSpacing(QPainter &p);
 
+  const QString& text() const;
   float rotation() const {return Rotation;}
-
+  double rotationRadians() const {return 3.1415926535897932333796 * (double)Rotation / 180.0;}
+  
+  const QColor& color() const;
+  void setColor(const QColor &color);
+  bool usePlotColor() const;
+  void setUsePlotColor(bool usePlotColor);
+  
   /** draw the label with painter at px, py */
   void draw(QPainter &p, int px, int py, bool bJustify=true, bool doDraw=true);
 
@@ -71,8 +78,6 @@ public:
   void save(QTextStream &ts);
   void read(QDomElement &e);
 
-  const QString& text() const;
-
   void setDoScalarReplacement(bool in_do);
 
   inline double x() const { return _x; }
@@ -92,6 +97,8 @@ private:
 
   QString SymbolFontName;
   QString FontName;
+  bool _usePlotColor;
+  QColor _color;
   int Size;
   float Width; /* set by draw */
 
