@@ -33,10 +33,8 @@
 
 KstLegend::KstLegend(
                    KstJustifyType in_j,
-                   float in_rotation,
                    float in_X, float in_Y,
                    bool is_sample) {
-  Rotation = in_rotation;
   _x = in_X;
   _y = in_Y;
   Justify = in_j;
@@ -79,10 +77,6 @@ void KstLegend::setDoScalarReplacement(bool in_do) {
   doScalarReplacement = in_do;
 }
 
-void KstLegend::setRotation(float in_rotation) {
-  Rotation = in_rotation;
-}
-
 void KstLegend::setShow(bool bShow) {
   ShowLegend = bShow;
 }
@@ -227,7 +221,6 @@ void KstLegend::save(QTextStream &ts) {
 }
 
 void KstLegend::read(QDomElement &e) {
-  Rotation = 0.0;
   Justify = CxBy;
   _x = _y = 0.0;
   FontName = "helvetica";
