@@ -18,14 +18,7 @@
 #ifndef KSTLEGEND_H
 #define KSTLEGEND_H
 
-#include "kstbasecurve.h"
-#include "kstlabel.h"
-#include "kstscalar.h"
 #include "kst2dplot.h"
-#include <qdom.h>
-#include <qdragobject.h>
-#include <qpainter.h>
-#include <qstring.h>
 
 typedef enum {LeftTop, CentreTop, RightTop, LeftCentre, CentreCentre,
               RightCentre, LeftBottom, CentreBottom, RightBottom} KstLegendLayoutType;
@@ -34,11 +27,9 @@ typedef enum {AlignmentLeft, AlignmentCentre, AlignmentRight} KstLegendAlignment
 class KstLegend {
 public:
   KstLegend(KstJustifyType in_j = CxBy,
-           float in_rotation = 0.0, float in_X=0.015,
-           float in_Y=0.02, bool is_sample = false);
+           float in_X=0.015, float in_Y=0.02, bool is_sample = false);
   ~KstLegend();
 
-  void setRotation(float new_rotation);
   void setRelPosition(float new_x, float new_y);
   void offsetRelPosition(float offset_x, float offset_y);
   void setJustification(KstJustifyType Justify);
@@ -52,7 +43,6 @@ public:
   QColor getColorForeground() { return colorForeground; }
   void setLayout(KstLegendLayoutType type) { LayoutType = type;}
   void setAlignment(KstLegendAlignmentType type) { AlignmentType = type;}
-  float rotation() const {return Rotation;}
   KstJustifyType justification() const {return Justify;}
 
   /** draw the label with painter at px, py */
@@ -78,7 +68,6 @@ private:
   /** Computes the font size, given the size of the painter, and Size */
   int fontSize(QPainter &p);
 
-  float Rotation;
   KstJustifyType Justify;
 
   QString SymbolFontName;
