@@ -22,13 +22,13 @@
 #include <qfontdatabase.h>
 #include <qlineedit.h>
 #include <qmessagebox.h>
+#include <qpushbutton.h>
+#include <qradiobutton.h>
 #include <qspinbox.h>
 #include <qstring.h>
 #include <qtabwidget.h>
 #include <qtextstream.h>
 #include <qwidget.h>
-#include <qpushbutton.h>
-#include <qradiobutton.h>
 
 // include files for KDE
 #include <kcolorbutton.h>
@@ -37,8 +37,8 @@
 #include <kfontcombo.h>
 #include <klistbox.h>
 #include <klocale.h>
-#include <kmessagebox.h>
 #include <kmdimainfrm.h>
+#include <kmessagebox.h>
 
 // application specific includes
 #include "kst.h"
@@ -48,6 +48,7 @@
 #include "kstplotdialog_i.h"
 #include "kstsettings.h"
 #include "kstviewwindow.h"
+#include "plotlistbox.h"
 
 struct LegendLayout {
   QString Label;
@@ -77,8 +78,8 @@ const LegendLayout LegendLayouts[] = {
   { i18n("Bottom right"), RightBottom }
 };
 
-const unsigned int iNumLegendLayouts = sizeof( LegendLayouts ) / sizeof( LegendLayout );
-const unsigned int iNumLegendAlignments = sizeof( LegendAlignments ) / sizeof( LegendAlignment );
+const unsigned int numLegendLayouts = sizeof( LegendLayouts ) / sizeof( LegendLayout );
+const unsigned int numLegendAlignments = sizeof( LegendAlignments ) / sizeof( LegendAlignment );
 
 /*
  *  Constructs a KstPlotDialogI which is a child of 'parent', with the
@@ -90,7 +91,6 @@ const unsigned int iNumLegendAlignments = sizeof( LegendAlignments ) / sizeof( L
 KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
                                const char* name, bool modal, WFlags fl)
 : KstPlotDialog(parent, name, modal, fl ) {
-  QString str;
   unsigned int i;
 
   doc = in_doc;
@@ -114,12 +114,10 @@ KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
   /* adding/removing curves */
   connect(CurveUnplot, SIGNAL(clicked()),
           this, SLOT(removeDisplayedCurve()));
-  connect(DisplayedCurveList, SIGNAL(executed(QListBoxItem*)),
-          this, SLOT(removeDisplayedCurve(QListBoxItem*)));
   connect(CurvePlot, SIGNAL(clicked()),
           this, SLOT(addDisplayedCurve()));
-  connect(AvailableCurveList, SIGNAL(executed(QListBoxItem*)),
-          this, SLOT(addDisplayedCurve(QListBoxItem*)));
+
+  connect(_newWindow, SIGNAL(clicked()), this, SLOT(newWindow()));
 
   /* labels */
   connect(TopLabelText, SIGNAL(textChanged(const QString &)),
@@ -186,23 +184,38 @@ KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
           this, SLOT(updateSampLegendLabel()));
 
   LegendFontComboBox->setFonts(qfd.families());
-
   LegendPosition->hide();
   LegendPositionLabel->hide();
   LegendPosition->setEditable(false);
-  for (i = 0; i < iNumLegendLayouts; i++) {
-    str = LegendLayouts[i].Label;
-    LegendPosition->insertItem(str);
+  for (i = 0; i < numLegendLayouts; i++) {
+    LegendPosition->insertItem(LegendLayouts[i].Label);
   }
 
   LegendAlignment->setEditable(false);
-  for (i = 0; i < iNumLegendAlignments; i++) {
-    str = LegendAlignments[i].Label;
-    LegendAlignment->insertItem(str);
+  for (i = 0; i < numLegendAlignments; i++) {
+    LegendAlignment->insertItem(LegendAlignments[i].Label);
   }
 
-  radioButtonThisPlot->setChecked( true );
-  radioButtonLegendThisPlot->setChecked( true );
+  radioButtonThisPlot->setChecked(true);
+  radioButtonLegendThisPlot->setChecked(true);
+  
+  /* Plot Markers */
+  connect(AddPlotMarker, SIGNAL(clicked()),
+          this, SLOT(addPlotMarker()));
+  connect(RemovePlotMarker, SIGNAL(clicked()),
+          this, SLOT(removePlotMarker()));
+  connect(RemoveAllPlotMarkers, SIGNAL(clicked()),
+          this, SLOT(removeAllPlotMarkers()));
+  connect(PlotMarkerList, SIGNAL(clicked(QListBoxItem*)),
+          this, SLOT(updateButtons()));
+  connect(PlotMarkerList, SIGNAL(selectionChanged()),
+          this, SLOT(updateButtons()));
+  connect(NewPlotMarker, SIGNAL(returnPressed()),
+          this, SLOT(addPlotMarker()));
+  connect(NewPlotMarker, SIGNAL(textChanged(const QString &)),
+          this, SLOT(updateButtons()));
+  connect(UseCurve, SIGNAL(clicked()),
+          this, SLOT(updateButtons()));
 }
 
 KstPlotDialogI::~KstPlotDialogI() {
@@ -210,9 +223,9 @@ KstPlotDialogI::~KstPlotDialogI() {
   SampleLabel = 0L;
 }
 
-void KstPlotDialogI::show_I(const QString& strWindow, const QString& strPlot) {
-  _strWindow = strWindow;
-  _strPlot = strPlot;
+void KstPlotDialogI::show_I(const QString& window, const QString& plot) {
+  _windowName = window;
+  _plotName = plot;
 
   show();
   raise();
@@ -235,13 +248,7 @@ void KstPlotDialogI::updateWindow() {
 }
 
 void KstPlotDialogI::update(int new_index) {
-  Q_UNUSED(new_index);
-
-  int i_plot;
-  KstApp *app = KstApp::inst();
-  Kst2DPlotPtr plot;
-  KstBaseCurvePtr curve;
-  double xmin, ymin, xmax, ymax;
+  Q_UNUSED(new_index)
 
   if (!isShown()) {
     return;
@@ -251,15 +258,16 @@ void KstPlotDialogI::update(int new_index) {
   updateWindowList();
   updatePlotList();
   updateCurveLists();
+  updatePlotMarkers();
 
+  int i_plot;
   if (Select->count() > 0) {
     i_plot = Select->currentItem();
-    PlotPosition->setValue(i_plot);
   } else {
     i_plot = -1;
-    PlotPosition->setValue(0);
   }
 
+  KstApp *app = KstApp::inst();
   KMdiChildView *c = app->findWindow(_window->currentText());
 
   if (c) {
@@ -270,7 +278,10 @@ void KstPlotDialogI::update(int new_index) {
 
   if (i_plot >= 0 && c) {
     KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Select->currentText());
-    plot = dynamic_cast<Kst2DPlot*>(obj.data());
+    Kst2DPlotPtr plot = dynamic_cast<Kst2DPlot*>(obj.data());
+    if (!plot) {
+      return;
+    }
 
     /* insert the current plot name in the plot name edit box */
     Name->setText(plot->tagName());
@@ -279,6 +290,7 @@ void KstPlotDialogI::update(int new_index) {
     XIsLog->setChecked(plot->isXLog());
     YIsLog->setChecked(plot->isYLog());
 
+    double xmin, ymin, xmax, ymax;
     plot->getScale(xmin, ymin, xmax, ymax);
 
     XFixedMin->setText(QString::number(xmin));
@@ -317,7 +329,7 @@ void KstPlotDialogI::update(int new_index) {
       YAuto->setChecked(true);
     }
 
-    NumberFontSize->setValue(plot->TickLabel->size());
+    NumberFontSize->setValue(plot->XTickLabel->size());
 
     XAxisText->setText(plot->XLabel->text());
     XLabelFontSize->setValue(plot->XLabel->size());
@@ -353,9 +365,9 @@ void KstPlotDialogI::update(int new_index) {
   updateButtons();
 }
 
-void KstPlotDialogI::fill2DPlotList( Kst2DPlotList& plots ) {
-  KstApp* pApp = KstApp::inst();
-  KMdiIterator<KMdiChildView*> *it = pApp->createIterator();
+void KstPlotDialogI::fill2DPlotList(Kst2DPlotList& plots) {
+  KstApp *app = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *it = app->createIterator();
   while (it->currentItem()) {
     KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
     if (c) {
@@ -369,19 +381,16 @@ void KstPlotDialogI::fill2DPlotList( Kst2DPlotList& plots ) {
     }
     it->next();
   }
-  pApp->deleteIterator(it);
+  app->deleteIterator(it);
 }
 
 
 void KstPlotDialogI::applyLabels(Kst2DPlotPtr plot) {
-  QString strAlignment;
-  QString strLayout;
   Kst2DPlotList plots;
-  unsigned int i;
 
-  plot->XLabel->setText((XAxisText)->text());
-  plot->YLabel->setText((YAxisText)->text());
-  plot->TopLabel->setText((TopLabelText)->text());
+  plot->XLabel->setText(XAxisText->text());
+  plot->YLabel->setText(YAxisText->text());
+  plot->TopLabel->setText(TopLabelText->text());
 
   if (radioButtonThisPlot->isChecked()) {
     plots += plot;
@@ -391,10 +400,10 @@ void KstPlotDialogI::applyLabels(Kst2DPlotPtr plot) {
       plots  = static_cast<KstViewWindow*>(c)->view()->findChildrenType<Kst2DPlot>(true);
     }
   } else {
-    fill2DPlotList( plots );
+    fill2DPlotList(plots);
   }
 
-  for (i=0; i<plots.size(); i++) {
+  for (uint i = 0; i < plots.size(); i++) {
     plot = plots[i];
 
     plot->XLabel->setFontName(FontComboBox->currentText());
@@ -406,15 +415,19 @@ void KstPlotDialogI::applyLabels(Kst2DPlotPtr plot) {
     plot->TopLabel->setFontName(FontComboBox->currentText());
     plot->TopLabel->setSize(TopLabelFontSize->value());
 
-    plot->TickLabel->setFontName(FontComboBox->currentText());
-    plot->TickLabel->setSize(NumberFontSize->value());
+    plot->XTickLabel->setFontName(FontComboBox->currentText());
+    plot->XTickLabel->setSize(NumberFontSize->value());
+    plot->XFullTickLabel->setFontName(FontComboBox->currentText());
+    plot->XFullTickLabel->setSize(NumberFontSize->value());
+    plot->YTickLabel->setFontName(FontComboBox->currentText());
+    plot->YTickLabel->setSize(NumberFontSize->value());
+    plot->YFullTickLabel->setFontName(FontComboBox->currentText());
+    plot->YFullTickLabel->setSize(NumberFontSize->value());
   }
 }
 
 
 void KstPlotDialogI::applyLegends(Kst2DPlotPtr plot) {
-  QString strAlignment;
-  QString strLayout;
   KstLegendLayoutType layout = LeftTop;
   KstLegendAlignmentType alignment = AlignmentRight;
   Kst2DPlotList plots;
@@ -425,29 +438,29 @@ void KstPlotDialogI::applyLegends(Kst2DPlotPtr plot) {
   } else if (radioButtonAllPlotsInWindow->isChecked()) {
     KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
     if (c) {
-      plots  = static_cast<KstViewWindow*>(c)->view()->findChildrenType<Kst2DPlot>(true);
+      plots = static_cast<KstViewWindow*>(c)->view()->findChildrenType<Kst2DPlot>(true);
     }
   } else {
-    fill2DPlotList( plots );
+    fill2DPlotList(plots);
   }
 
-  strLayout = LegendPosition->currentText();
-  for (i = 0; i < iNumLegendLayouts; i++) {
-    if (strLayout.compare(LegendLayouts[i].Label) == 0) {
+  const QString& layoutName = LegendPosition->currentText();
+  for (i = 0; i < numLegendLayouts; i++) {
+    if (layoutName == LegendLayouts[i].Label) {
       layout = LegendLayouts[i].type;
       break;
     }
   }
 
-  strAlignment = LegendAlignment->currentText();
-  for (i = 0; i < iNumLegendAlignments; i++) {
-    if (strAlignment.compare(LegendAlignments[i].Label) == 0) {
+  const QString& alignmentName = LegendAlignment->currentText();
+  for (i = 0; i < numLegendAlignments; i++) {
+    if (alignmentName == LegendAlignments[i].Label) {
       alignment = LegendAlignments[i].type;
       break;
     }
   }
 
-  for (i=0; i<plots.size(); i++) {
+  for (i = 0; i < plots.size(); i++) {
     plot = plots[i];
 
     plot->Legend->setFontName(LegendFontComboBox->currentText());
@@ -461,8 +474,8 @@ void KstPlotDialogI::applyLegends(Kst2DPlotPtr plot) {
   }
 }
 
-void KstPlotDialogI::applyLimits(Kst2DPlotPtr plot) {
 
+void KstPlotDialogI::applyLimits(Kst2DPlotPtr plot) {
   /* do X Scale */
   if (XAC->isChecked()) {
     plot->setXScaleMode(AC);
@@ -503,37 +516,27 @@ void KstPlotDialogI::applyLimits(Kst2DPlotPtr plot) {
 void KstPlotDialogI::applyPlotColors(Kst2DPlotPtr plot) {
   plot->setForegroundColor(plotColors->foreground());
   plot->setBackgroundColor(plotColors->background());
-
 }
 
 
 void KstPlotDialogI::new_I() {
-  Kst2DPlotPtr plot;
-
   KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
   if (!c) {
     QString name = KstApp::inst()->newWindow();
+    if (name.isEmpty()) {
+      return;
+    }
     c = KstApp::inst()->findWindow(name);
     _window->insertItem(name);
     _window->setCurrentText(name);
   }
 
-  // FIXME: might want to ensure that the plot name is not in any other window
-  /** verify that the plotname is unique */
-  if (c) {
-    KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Name->text());
-    if (obj) {
-      QString message = i18n("Could not create a new plot.\n"
-          "%1: not a unique plot name.\n"
-          "Change it to a unique name.").arg(Name->text());
-
-      KMessageBox::sorry(0L, message);
-      Name->setFocus();
-      return;
-    }
+  //check plot name
+  if (!checkPlotName()) {
+    return;
   }
 
-  plot = static_cast<KstViewWindow*>(c)->view()->createPlot<Kst2DPlot>(Name->text());
+  Kst2DPlotPtr plot = static_cast<KstViewWindow*>(c)->view()->createPlot<Kst2DPlot>(Name->text());
 
   if (_reGrid->isChecked()) {
     static_cast<KstViewWindow*>(c)->view()->cleanup(_plotColumns->value());
@@ -549,6 +552,16 @@ void KstPlotDialogI::new_I() {
     }
   }
   curves.clear();
+  //also add the images
+  KstImageList images = kstObjectSubList<KstDataObject, KstImage>(KST::dataObjectList);
+  plot->removeAllImages();
+  for (unsigned i = 0; i < DisplayedCurveList->count(); i++) {
+    KstImageList::Iterator it = images.findTag(DisplayedCurveList->text(i));
+    if (it != images.end()) {
+      plot->addImage(*it);
+    }
+  }
+  images.clear();
 
   applyLabels(plot);
   applyLegends(plot);
@@ -559,28 +572,33 @@ void KstPlotDialogI::new_I() {
 //   plot->setForegroundColor(Colors->foreground());
 //   plot->setBackgroundColor(Colors->background());
   applyPlotColors(plot);
+  applyPlotMarkers(plot);
+
 
   static_cast<KstViewWindow*>(c)->view()->paint(P_PLOT);
 
-  update();
-
-  changePosition(PlotPosition->value());
-
   //doc->setDelay(UpdateDelay->value());
 
+  _plotName = Name->text();
   update();
+
   emit docChanged();
 }
 
 void KstPlotDialogI::edit_I() {
-  Kst2DPlotPtr plot;
-
   int index = Select->currentItem();
   if (index < 0) {
     KMessageBox::sorry(0L, i18n("You need to select an active plot to edit."));
     return;
   }
 
+  //check the plot name
+  if (Name->text() != Select->currentText()) {
+    if (!checkPlotName()) {
+    return;
+    }
+  }
+
   KMdiChildView *c = KstApp::inst()->findWindow(_window->currentText());
   if (!c) {
     return;
@@ -591,8 +609,7 @@ void KstPlotDialogI::edit_I() {
   if (index >= (int)plots.count()) {
     new_I();
   } else {
-    plot = *plots.at(index);
-
+    Kst2DPlotPtr plot = *plots.at(index);
 
     plot->setTagName(Name->text());
     /* add the curves */
@@ -605,11 +622,22 @@ void KstPlotDialogI::edit_I() {
       }
     }
     curves.clear();
+    //also add the images
+    KstImageList images = kstObjectSubList<KstDataObject, KstImage>(KST::dataObjectList);
+    plot->removeAllImages();
+    for (unsigned i = 0; i < DisplayedCurveList->count(); i++) {
+      KstImageList::Iterator it = images.findTag(DisplayedCurveList->text(i));
+      if (it != images.end()) {
+        plot->addImage(*it);
+      }
+    }
+    images.clear();
 
     applyLabels(plot);
     applyLegends(plot);
     applyLimits(plot);
     applyPlotColors(plot);
+    applyPlotMarkers(plot);
 
 //  FIXME: why is this commented out?
 //     if (ApplyToAll->isChecked()) {
@@ -621,7 +649,6 @@ void KstPlotDialogI::edit_I() {
 //       plot->setForegroundColor(Colors->foreground());
 //       plot->setBackgroundColor(Colors->background());
 //     }
-    changePosition(PlotPosition->value());
 
     if (_reGrid->isChecked()) {
       static_cast<KstViewWindow*>(c)->view()->cleanup(_plotColumns->value());
@@ -630,6 +657,7 @@ void KstPlotDialogI::edit_I() {
     plot->setDirty();
     static_cast<KstViewWindow*>(c)->view()->paint(P_PLOT);
 
+    _plotName = Name->text().stripWhiteSpace();
     update();
     emit docChanged();
   }
@@ -662,7 +690,7 @@ void KstPlotDialogI::delete_I() {
 }
 
 void KstPlotDialogI::updateSampNumLabel() {
-  if (FontComboBox->currentText() != 0) {
+  if (!FontComboBox->currentText().isEmpty()) {
     QPixmap pix(SampleNumLabel->size());
     QPainter p(&pix);
 
@@ -686,7 +714,7 @@ void KstPlotDialogI::updateSampNumLabel() {
 }
 
 void KstPlotDialogI::updateSampTopLabel() {
-  if (FontComboBox->currentText() != 0) {
+  if (!FontComboBox->currentText().isEmpty()) {
     QPixmap pix(SampleTopLabel->size());
     QPainter p(&pix);
 
@@ -707,7 +735,7 @@ void KstPlotDialogI::updateSampTopLabel() {
 }
 
 void KstPlotDialogI::updateSampXLabel() {
-  if (FontComboBox->currentText() != 0) {
+  if (!FontComboBox->currentText().isEmpty()) {
     QPixmap pix(SampleXLabel->size());
     QPainter p(&pix);
 
@@ -728,7 +756,7 @@ void KstPlotDialogI::updateSampXLabel() {
 }
 
 void KstPlotDialogI::updateSampYLabel() {
-  if (FontComboBox->currentText() != 0) {
+  if (!FontComboBox->currentText().isEmpty()) {
     QPixmap pix(SampleYLabel->size());
     QPainter p(&pix);
 
@@ -755,8 +783,9 @@ void KstPlotDialogI::updateLabels() {
   updateSampNumLabel();
 }
 
+
 void KstPlotDialogI::updateSampLegendLabel() {
-  if (LegendFontComboBox->currentText() != 0) {
+  if (!LegendFontComboBox->currentText().isEmpty()) {
     QPixmap pix(SampleLegendLabel->size());
     QPainter p(&pix);
 
@@ -773,8 +802,8 @@ void KstPlotDialogI::updateSampLegendLabel() {
     SampleLabel->setFontName(LegendFontComboBox->currentText());
     SampleLabel->setSize(LegendFontSize->value());
     SampleLabel->setText(i18n("Curve name"));
-    SampleLabel->draw(p,SampleXLabel->width()/2,
-                      SampleXLabel->height()/2,true);
+    SampleLabel->draw(p, SampleLegendLabel->width()/2,
+                      SampleLegendLabel->height()/2, true);
 
     SampleLegendLabel->setPixmap(pix);
   } else {
@@ -783,13 +812,61 @@ void KstPlotDialogI::updateSampLegendLabel() {
   }
 }
 
+void KstPlotDialogI::updatePlotMarkers() {
+  KstApp* app = KstApp::inst();
+
+  PlotMarkerList->clear();
+
+  KMdiChildView *c = app->findWindow(_window->currentText());
+  if (Select->count() > 0 && c) {
+    KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Select->currentText());
+    Kst2DPlotPtr plot = dynamic_cast<Kst2DPlot*>(obj.data());
+
+    if (plot) {
+      for (QValueList<double>::ConstIterator it = plot->plotMarkers().begin(); it != plot->plotMarkers().end(); it++) {
+        PlotMarkerList->insertItem(QString::number(*it, 'g', 64));
+      }
+      //update the auto-generation settings
+      KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
+      CurveCombo->clear();
+      for (KstBaseCurveList::iterator curves_iter = curves.begin(); curves_iter != curves.end(); curves_iter++) {
+        (*curves_iter)->readLock();
+        CurveCombo->insertItem((*curves_iter)->tagName());
+        (*curves_iter)->readUnlock();
+      }
+      if (plot->hasCurveToMarkers()) {
+        UseCurve->setChecked(true);
+        Both->setChecked(false);
+        Falling->setChecked(false);
+        Rising->setChecked(false);
+        if (plot->curveToMarkersFallingDetect()) {
+          if (plot->curveToMarkersRisingDetect()) {
+            Both->setChecked(true);
+          } else {
+            Falling->setChecked(true);
+          }
+        } else {
+          Rising->setChecked(true);
+        }
+        int curveComboIndex;
+        for (curveComboIndex = 0; curveComboIndex < CurveCombo->count(); curveComboIndex++) {
+          if (CurveCombo->text(curveComboIndex) == plot->curveToMarkers()->tagName()) {
+            break;
+          }
+        }
+        CurveCombo->setCurrentItem(curveComboIndex);
+      }
+    }
+  }
+
+}
+
 
 void KstPlotDialogI::removeDisplayedCurve() {
   uint count = DisplayedCurveList->count();
-  int i;
 
-  if (count>0) {
-    for (i=count-1; i>=0; i--) {
+  if (count > 0) {
+    for (int i = count-1; i >= 0; i--) {
       if (DisplayedCurveList->isSelected(i)) {
         AvailableCurveList->insertItem(DisplayedCurveList->text(i));
         DisplayedCurveList->removeItem(i);
@@ -800,13 +877,10 @@ void KstPlotDialogI::removeDisplayedCurve() {
 }
 
 
-void KstPlotDialogI::removeDisplayedCurve(QListBoxItem* pItem){
-  QString str;
-  int index;
-
-  index = DisplayedCurveList->index(pItem);
-  if (index!=-1) {
-    str = DisplayedCurveList->text(index);
+void KstPlotDialogI::removeDisplayedCurve(QListBoxItem* item){
+  int index = DisplayedCurveList->index(item);
+  if (index != -1) {
+    QString str = DisplayedCurveList->text(index);
     DisplayedCurveList->removeItem(index);
     AvailableCurveList->insertItem(str);
     updateButtons();
@@ -816,10 +890,9 @@ void KstPlotDialogI::removeDisplayedCurve(QListBoxItem* pItem){
 
 void KstPlotDialogI::addDisplayedCurve() {
   uint count = AvailableCurveList->count();
-  int i;
 
-  if (count>0) {
-    for (i=count-1; i>=0; i--) {
+  if (count > 0) {
+    for (int i = count-1; i >= 0; i--) {
       if (AvailableCurveList->isSelected(i)) {
         DisplayedCurveList->insertItem(AvailableCurveList->text(i));
         AvailableCurveList->removeItem(i);
@@ -831,12 +904,9 @@ void KstPlotDialogI::addDisplayedCurve() {
 
 
 void KstPlotDialogI::addDisplayedCurve(QListBoxItem* pItem){
-  QString str;
-  int index;
-
-  index = AvailableCurveList->index(pItem);
-  if (index!=-1) {
-    str = AvailableCurveList->text(index);
+  int index = AvailableCurveList->index(pItem);
+  if (index != -1) {
+    QString str = AvailableCurveList->text(index);
     AvailableCurveList->removeItem(index);
     DisplayedCurveList->insertItem(str);
     updateButtons();
@@ -858,56 +928,60 @@ void KstPlotDialogI::applyAutoLabels() {
   }
 }
 
-void KstPlotDialogI::changePosition(int ni) {
-// FIXMEPLOTLIST - not applicable anymore?
-#if 0
-  if (ni >= Select->count()) {
-    ni = Select->count() - 1;
-  }
-
-  int ci = Select->currentItem();
-  KST::plotList.insert(ni, KST::plotList.take(ci));
-  Select->setCurrentItem(ni);
-#endif
-}
 
 void KstPlotDialogI::updateButtons() {
-  bool bSelected;
-  uint count;
-  uint i;
+  bool selected = false;
+  uint count = AvailableCurveList->count();
 
-  bSelected = false;
-  count = AvailableCurveList->count( );
-  for (i=0; i<count; i++) {
-    if( AvailableCurveList->isSelected(i) ) {
-      bSelected = true;
+  for (uint i = 0; i < count; i++) {
+    if (AvailableCurveList->isSelected(i)) {
+      selected = true;
     }
   }
-  if (bSelected && !CurvePlot->isEnabled()) {
-    CurvePlot->setEnabled(TRUE);
-  } else if (!bSelected && CurvePlot->isEnabled()) {
-    CurvePlot->setEnabled(FALSE);
+  if (selected && !CurvePlot->isEnabled()) {
+    CurvePlot->setEnabled(true);
+  } else if (!selected && CurvePlot->isEnabled()) {
+    CurvePlot->setEnabled(false);
   }
 
-  bSelected = false;
-  count = DisplayedCurveList->count( );
-  for (i=0; i<count; i++) {
-    if( DisplayedCurveList->isSelected(i) ) {
-      bSelected = true;
+  selected = false;
+  count = DisplayedCurveList->count();
+  for (uint i = 0; i < count; i++) {
+    if (DisplayedCurveList->isSelected(i)) {
+      selected = true;
     }
   }
-  if (bSelected && !CurveUnplot->isEnabled()) {
-    CurveUnplot->setEnabled(TRUE);
-  } else if (!bSelected && CurveUnplot->isEnabled()) {
-    CurveUnplot->setEnabled(FALSE);
+  if (selected && !CurveUnplot->isEnabled()) {
+    CurveUnplot->setEnabled(true);
+  } else if (!selected && CurveUnplot->isEnabled()) {
+    CurveUnplot->setEnabled(false);
   }
+
+  /** updates for Plot Markers tab **/
+  AddPlotMarker->setEnabled(!NewPlotMarker->text().isEmpty());
+
+  selected = false;
+  count = PlotMarkerList->count();
+  for (uint i = 0; i < count; i++) {
+    if (PlotMarkerList->isSelected(i)) {
+      selected = true;
+    }
+  }
+  RemovePlotMarker->setEnabled(selected);
+  RemoveAllPlotMarkers->setEnabled(count > 0);
+
+  //updates for auto-generation marker curve section
+  CurveCombo->setEnabled(UseCurve->isChecked());
+  Rising->setEnabled(UseCurve->isChecked());
+  Falling->setEnabled(UseCurve->isChecked());
+  Both->setEnabled(UseCurve->isChecked());
+
 }
 
 void KstPlotDialogI::updateCurveLists() {
   KstApp* app = KstApp::inst();
-  Kst2DPlotPtr plot;
   KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
-  KstBaseCurveList::iterator it;
+  KstImageList images = kstObjectSubList<KstDataObject, KstImage>(KST::dataObjectList);
 
   DisplayedCurveList->clear();
   AvailableCurveList->clear();
@@ -915,29 +989,52 @@ void KstPlotDialogI::updateCurveLists() {
   KMdiChildView *c = app->findWindow(_window->currentText());
   if (Select->count() && c) {
     KstViewObjectPtr obj = static_cast<KstViewWindow*>(c)->view()->findChild(Select->currentText());
-    plot = dynamic_cast<Kst2DPlot*>(obj.data());
-
-    for (it = curves.begin(); it != curves.end(); it++) {
+    Kst2DPlotPtr plot = dynamic_cast<Kst2DPlot*>(obj.data());
+    if (!plot) {
+      return;
+    }
+    for (KstBaseCurveList::iterator it = curves.begin(); it != curves.end(); it++) {
+      (*it)->readLock();
       if (plot->Curves.find(*it) == plot->Curves.end()) {
         AvailableCurveList->insertItem((*it)->tagName());
       } else {
         DisplayedCurveList->insertItem((*it)->tagName());
       }
+      (*it)->readUnlock();
+    }
+    //add images as well
+    for (KstImageList::iterator it = images.begin(); it != images.end(); it++) {
+      (*it)->readLock();
+      if (plot->hasImage(*it)) {
+        DisplayedCurveList->insertItem((*it)->tagName());
+      } else {
+        AvailableCurveList->insertItem((*it)->tagName());
+      }
+      (*it)->readUnlock();
     }
   } else {
-    for (it = curves.begin(); it != curves.end(); it++) {
+    for (KstBaseCurveList::iterator it = curves.begin(); it != curves.end(); it++) {
+      (*it)->readLock();
       AvailableCurveList->insertItem((*it)->tagName());
+      (*it)->readUnlock();
+    }
+    //add images as well
+    for (KstImageList::iterator it = images.begin(); it != images.end(); it++) {
+      (*it)->readLock();
+      AvailableCurveList->insertItem((*it)->tagName());
+      (*it)->readUnlock();
     }
   }
 }
 
 void KstPlotDialogI::updateScalarCombo() {
-  int index;
-
   ScalarList->clear();
   KST::scalarList.lock().readLock();
-  for (index = 0; index < (int)KST::scalarList.count(); index++) {
-    ScalarList->insertItem(KST::scalarList[index]->tagLabel());
+  for (int index = 0; index < (int)KST::scalarList.count(); index++) {
+    KstScalarPtr s = KST::scalarList[index];
+    s->readLock();
+    ScalarList->insertItem(s->tagLabel());
+    s->readUnlock();
   }
   KST::scalarList.lock().readUnlock();
 }
@@ -945,10 +1042,10 @@ void KstPlotDialogI::updateScalarCombo() {
 void KstPlotDialogI::updateWindowList() {
   KstApp *app = KstApp::inst();
   KMdiIterator<KMdiChildView*> *it = app->createIterator();
-  QString strOld;
+  QString oldText;
 
   if (_window->count()) {
-    strOld = _window->currentText();
+    oldText = _window->currentText();
   }
 
   _window->clear();
@@ -957,16 +1054,12 @@ void KstPlotDialogI::updateWindowList() {
     KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
     if (v) {
       _window->insertItem(v->caption());
-      if (!strOld.isEmpty()) {
-        if (strOld.compare(v->caption()) == 0) {
-          _window->setCurrentItem(_window->count()-1);
-        }
+      if (!oldText.isEmpty() && oldText == v->caption()) {
+        _window->setCurrentItem(_window->count() - 1);
       }
-      if (!_strWindow.isEmpty()) {
-        if (_strWindow.compare(v->caption()) == 0) {
-          _window->setCurrentItem(_window->count()-1);
-          strOld = "";
-        }
+      if (!_windowName.isEmpty() && _windowName == v->caption()) {
+        _window->setCurrentItem(_window->count() - 1);
+        oldText = QString::null;
       }
     }
     it->next();
@@ -974,42 +1067,155 @@ void KstPlotDialogI::updateWindowList() {
 
   app->deleteIterator(it);
 
-  _strWindow = "";
+  _windowName = QString::null;
 }
 
 void KstPlotDialogI::updatePlotList() {
   KstApp *app = KstApp::inst();
-  KMdiChildView *c = app->findWindow(_window->currentText());
-  QString strOld;
+  QString oldName;
 
   if (Select->count()) {
-    strOld = Select->currentText();
+    oldName = Select->currentText();
   }
 
   Select->clear();
 
+  KMdiChildView *c = app->findWindow(_window->currentText());
   if (c) {
     Kst2DPlotList plots = static_cast<KstViewWindow*>(c)->view()->findChildrenType<Kst2DPlot>(true);
 
     for (Kst2DPlotList::Iterator i = plots.begin(); i != plots.end(); ++i) {
       Select->insertItem((*i)->tagName());
-      if (!strOld.isEmpty()) {
-        if (strOld.compare((*i)->tagName()) == 0) {
-          Select->setCurrentItem(Select->count()-1);
-        }
+      if (!oldName.isEmpty() && oldName == (*i)->tagName()) {
+        Select->setCurrentItem(Select->count() - 1);
       }
-      if (!_strPlot.isEmpty()) {
-        if (_strPlot.compare((*i)->tagName()) == 0) {
-          Select->setCurrentItem(Select->count()-1);
-          strOld = "";
-        }
+      if (!_plotName.isEmpty() && _plotName == (*i)->tagName()) {
+        Select->setCurrentItem(Select->count() - 1);
+        oldName = QString::null;
       }
     }
   }
 
-  _strPlot = "";
+  _plotName = QString::null;
 }
 
+void KstPlotDialogI::addPlotMarker() {
+  //silently do nothing if there is no text, to be consistent
+  //with "Add" button disabled
+  if (NewPlotMarker->text().isEmpty()) {
+    return;
+  }
+
+  bool ok;
+  double newMarkerVal = NewPlotMarker->text().toDouble(&ok);
+
+  if (ok) {
+    uint i = 0;
+    QString stringnum;
+    stringnum.setNum(newMarkerVal, 'g', 64);
+    while (i < PlotMarkerList->count() && PlotMarkerList->text(i).toDouble() < newMarkerVal) {
+      i++;
+    }
+    if (i == PlotMarkerList->count()) {
+      PlotMarkerList->insertItem(stringnum, -1);
+      NewPlotMarker->clear();
+      return;
+    }
+    if (newMarkerVal != PlotMarkerList->text(i).toDouble()) {
+      PlotMarkerList->insertItem(stringnum, i);
+      NewPlotMarker->clear();
+      return;
+    }
+    KMessageBox::sorry(this,
+                       i18n("A plot marker with equal (or very close) value already exists."),
+                       i18n("Kst"));
+    return;
+  }
+  KMessageBox::sorry(this,
+                     i18n("The text you have entered is not a valid number. Please enter a valid number."),
+                     i18n("Kst"));
+}
+
+void KstPlotDialogI::removePlotMarker() {
+  uint count = PlotMarkerList->count();
+  if (count > 0) {
+    for (int i = count-1; i >= 0; i--) {
+      if (PlotMarkerList->isSelected(i)) {
+        PlotMarkerList->removeItem(i);
+      }
+    }
+    updateButtons();
+  }
+}
+
+void KstPlotDialogI::removeAllPlotMarkers() {
+  uint count = PlotMarkerList->count();
+  if (count > 0) {
+    for (int i = count-1; i >= 0; i--) {
+      PlotMarkerList->removeItem(i);
+    }
+    updateButtons();
+  }
+}
+
+void KstPlotDialogI::applyPlotMarkers(Kst2DPlotPtr plot) {
+  QValueList<double> newMarkers;
+  for (unsigned i = 0; i < PlotMarkerList->count(); i++) {
+    newMarkers.append(PlotMarkerList->text(i).toDouble());
+  }
+  plot->setPlotMarkerList(newMarkers);
+
+  //apply the autogeneration settings
+  if (UseCurve->isChecked()) {
+    KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
+    KstBaseCurveList::iterator curves_iter = curves.findTag(CurveCombo->currentText());
+    bool falling = Falling->isChecked(), rising = Rising->isChecked();
+    if (Both->isChecked()) {
+      falling = true;
+      rising = true;
+    }
+    plot->setCurveToMarkers(*curves_iter, rising, falling);
+  } else {
+    plot->removeCurveToMarkers();
+  }
+}
+
+void KstPlotDialogI::newWindow() {
+  QString name = KstApp::inst()->newWindow();
+  if (!name.isEmpty()) {
+    _window->setCurrentText(name);
+    updatePlotList();
+  }
+}
+
+bool KstPlotDialogI::checkPlotName() {
+  KstApp *app = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *iter;
+  QString name = Name->text();
+  //check the name
+  KstViewObjectPtr rc;
+  iter = app->createIterator();
+  while (iter->currentItem()) {
+    KMdiChildView *childview = iter->currentItem();
+    KstViewWindow *viewwindow = dynamic_cast<KstViewWindow*>(childview);
+    if (viewwindow) {
+      rc = viewwindow->view()->findChild(name);
+      if (rc) {
+        QString message = i18n("Could not create a new plot.\n"
+            "%1: not a unique plot name.\n"
+            "Change it to a unique name.").arg(Name->text());
+
+        KMessageBox::sorry(this, message);
+        Name->setFocus();
+        app->deleteIterator(iter);
+        return false;
+      }
+    }
+    iter->next();
+  }
+  app->deleteIterator(iter);
+  return true;
+}
 
 #include "kstplotdialog_i.moc"
 // vim: et ts=2 sw=2
