@@ -15,6 +15,17 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <stdlib.h>
+
+// include files for Qt
+#include <qmessagebox.h>
+
+// include files for KDE
+#include <kdebug.h>
+#include <klocale.h>
+#include <kmessagebox.h>
+
+// application specific includes
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "kstdoc.h"
@@ -22,33 +33,6 @@
 #include "kstplugindialog_i.h"
 #include "plugincollection.h"
 
-#include <kdebug.h>
-#include <klocale.h>
-#include <kmessagebox.h>
-
-#include <stdlib.h>
-
-
-static void countScalarsAndVectors(const QValueList<Plugin::Data::IOValue>& table, unsigned& scalars, unsigned& vectors) {
-  scalars = 0;
-  vectors = 0;
-
-  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
-    switch ((*it)._type) {
-      case Plugin::Data::IOValue::FloatType:
-        scalars++;
-        break;
-      case Plugin::Data::IOValue::TableType:
-        if ((*it)._subType == Plugin::Data::IOValue::FloatSubType) {
-          vectors++;
-        }
-        break;
-      default:
-        break;
-    }
-  }
-}
-
 
 KstPlugin::KstPlugin() : KstDataObject() {
   commonConstructor();
@@ -74,10 +58,18 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
       } else if (e.tagName() == "iscalar") {
         _inputScalarLoadQueue.append(qMakePair(e.attribute("name"), e.text()));
       } else if (e.tagName() == "ovector") {
-        KstVectorPtr v = new KstVector(e.text());
+        KstVectorPtr v;
+
+        // FIXME: these might already exist!!
+        if (e.attribute("scalarList", "0").toInt()) {
+          v = new KstVector(e.text(), 0, true);
+        } else {
+          v = new KstVector(e.text(), 0, false);
+        }
         _outputVectors.insert(e.attribute("name"), v);
         KST::addVectorToList(v);
       } else if (e.tagName() == "oscalar") {
+        // FIXME: these might already exist!!
         _outputScalars.insert(e.attribute("name"), new KstScalar(e.text()));
       }
     }
@@ -99,7 +91,13 @@ KstPlugin::KstPlugin(QDomElement &e) : KstDataObject(e) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) {
         _outArrayCnt++;
         if (!_outputVectors.contains((*it)._name)) {
-          KstVectorPtr v = new KstVector(tagName() + " vector - " + (*it)._name);
+          KstVectorPtr v;
+
+          if ((*it)._subType == Plugin::Data::IOValue::FloatNonVectorSubType) {
+            v = new KstVector(tagName() + " vector - " + (*it)._name, 0, true);
+          } else {
+            v = new KstVector(tagName() + " vector - " + (*it)._name, 0, false);
+          }
           _outputVectors.insert((*it)._name, v);
           KST::addVectorToList(v);
         }
@@ -132,6 +130,28 @@ KstPlugin::~KstPlugin() {
 }
 
 
+void KstPlugin::countScalarsAndVectors(const QValueList<Plugin::Data::IOValue>& table, unsigned& scalars, unsigned& vectors) {
+  scalars = 0;
+  vectors = 0;
+
+  for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
+    switch ((*it)._type) {
+      case Plugin::Data::IOValue::FloatType:
+        scalars++;
+        break;
+      case Plugin::Data::IOValue::TableType:
+        if ((*it)._subType == Plugin::Data::IOValue::FloatSubType ||
+            (*it)._subType == Plugin::Data::IOValue::FloatNonVectorSubType) {
+          vectors++;
+        }
+        break;
+      default:
+        break;
+    }
+  }
+}
+
+
 KstObject::UpdateType KstPlugin::update(int update_counter) {
   if (!isValid()) {
     return NO_CHANGE;
@@ -142,7 +162,7 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
   }
 
   if (update_counter != -1) {
-    bool force = false;
+    //bool force = false;
     // FIXME: we should only update if one of our dependencies was updated
   }
 
@@ -240,6 +260,27 @@ KstObject::UpdateType KstPlugin::update(int update_counter) {
     }
   }
 
+  //
+  // if we have a fit plugin then create the necessary scalars from the parameter vector...
+  //
+  if( _plugin->data()._isFit ) {
+    KstVectorPtr vectorParam = _outputVectors["Parameters"];
+    if( vectorParam ) {
+      for( int i=0; i<vectorParam->length(); i++ ) {
+        const QString& strParamName = _plugin->parameterName(i);
+        if( !strParamName.isEmpty() ) {
+          QString strScalarName = i18n("%1-%2").arg(tagName()).arg(strParamName);
+          double dScalarValue = vectorParam->value(i);
+          if (!_outputScalars.contains(strParamName)) {
+            KstScalarPtr s = new KstScalar(strScalarName);
+            _outputScalars.insert(strParamName, s);
+          }
+          _outputScalars[strParamName]->setValue(dScalarValue);
+        }
+      }
+    }
+  }
+  
   CLEANUP();
 #undef CLEANUP
   return UPDATE;
@@ -262,7 +303,11 @@ void KstPlugin::save(QTextStream &ts) {
          << "</iscalar>" << endl;
     }
     for (KstVectorMap::Iterator i = _outputVectors.begin(); i != _outputVectors.end(); ++i) {
-      ts << "   <ovector name=\"" << i.key() << "\">"
+      ts << "   <ovector name=\"" << i.key();
+      if (i.data()->isScalarList()) {
+        ts << "\" scalarList=\"1";
+      }
+      ts << "\">"
          << i.data()->tagName()
          << "</ovector>" << endl;
     }
@@ -297,6 +342,7 @@ QString KstPlugin::propertyString() const {
 
 
 bool KstPlugin::setPlugin(KstSharedPtr<Plugin> plugin) {
+  // Assumes that this is called with a write lock in place on this object
   if (plugin == _plugin) {
     return true;
   }
@@ -328,12 +374,21 @@ bool KstPlugin::setPlugin(KstSharedPtr<Plugin> plugin) {
                                                          it != otable.end();
                                                                         ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      KstVectorPtr v = new KstVector;
+      KstVectorPtr v;
+
+      if ((*it)._subType == Plugin::Data::IOValue::FloatNonVectorSubType) {
+        v = new KstVector(QString::null, 0, true);
+      } else {
+        v = new KstVector;
+      }
       KST::addVectorToList(v);
+      v->writeLock();
       _outputVectors.insert((*it)._name, v);
       _outArrayCnt++;
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-      _outputScalars.insert((*it)._name, new KstScalar);
+      KstScalarPtr s = new KstScalar;
+      s->writeLock();
+      _outputScalars.insert((*it)._name, s);
       _outScalarCnt++;
     }
   }
