@@ -15,8 +15,10 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "kstplugindialog_i.h"
+#include <assert.h>
+#include <stdlib.h>
 
+// include files for Qt
 #include <qcheckbox.h>
 #include <qcombobox.h>
 #include <qlabel.h>
@@ -33,6 +35,7 @@
 #include <qwhatsthis.h>
 #include <qwidget.h>
 
+// include files for KDE
 #include <kcolorbutton.h>
 #include <kcolordialog.h>
 #include <kdebug.h>
@@ -40,16 +43,15 @@
 #include <kmessagebox.h>
 #include <kstaticdeleter.h>
 
-#include "plugincollection.h"
-#include "plugin.h"
+// application specific includes
+#include "kstdatacollection.h"
 #include "kstdoc.h"
+#include "kstplugindialog_i.h"
+#include "plugin.h"
+#include "plugincollection.h"
+#include "pluginmanager.h"
 #include "scalarselector.h"
 #include "vectorselector.h"
-#include "kstdatacollection.h"
-#include "pluginmanager.h"
-
-#include <assert.h>
-#include <stdlib.h>
 
 #define DIALOGTYPE KstPluginDialogI
 #define DTYPE "Plugin"
@@ -62,11 +64,11 @@ KstPluginDialogI *KstPluginDialogI::globalInstance() {
   if (!_inst) {
     _inst = _plPluginInst.setObject(new KstPluginDialogI);
   }
-return _inst;
+  return _inst;
 }
 
 
-const QString newPluginString = i18n("<New_Plugin>");
+static const QString newPluginString = "<New_Plugin>";
 
 KstPluginDialogI::KstPluginDialogI(QWidget* parent, const char* name,
                                    bool modal, WFlags fl)
@@ -85,14 +87,9 @@ KstPluginDialogI::~KstPluginDialogI() {
 }
 
 KstPluginPtr KstPluginDialogI::_getPtr(const QString &tagin) {
-  KstPluginList c =
-    kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
+  KstPluginList c = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
 
-  if ( c.findTag(tagin) == c.end() ) {
-    return 0L;
-  } else {
-    return *c.findTag(tagin);
-  }
+  return *c.findTag(tagin);
 }
 
 void KstPluginDialogI::updatePluginList() {
@@ -108,16 +105,12 @@ void KstPluginDialogI::updatePluginList() {
   for (QMap<QString,Plugin::Data>::ConstIterator it = _pluginMap.begin();
                                                   it != _pluginMap.end();
                                                                      ++it) {
-    if (it.data()._filter == false) {
-      _pluginList += it.data()._name;
-      PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName)
-                              .arg(it.data()._version)
-                              .arg(it.data()._description));
-      if (it.data()._name == previous) {
-        newFocus = cnt;
-      }
-      cnt++;
+    _pluginList += it.data()._name;
+    PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName).arg(it.data()._version).arg(it.data()._description));
+    if (it.data()._name == previous) {
+      newFocus = cnt;
     }
+    cnt++;
   }
 
   if (newFocus != -1) {
@@ -130,11 +123,9 @@ void KstPluginDialogI::updatePluginList() {
 
 
 void KstPluginDialogI::updateForm() {
-  KstSharedPtr<Plugin> pPtr;
-
-  pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
-  if( pPtr != NULL ) {
-    const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
+  KstSharedPtr<Plugin> plugin = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
+  if (plugin) {
+    const QValueList<Plugin::Data::IOValue>& itable = plugin->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
         QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
@@ -156,8 +147,11 @@ void KstPluginDialogI::updateForm() {
 }
 
 void KstPluginDialogI::_fillFieldsForEdit() {
-  if ( DP == 0L ) return; // shouldn't be needed
-  _tagName->setText( DP->tagName() );
+  if (DP == 0L) {
+    return; // shouldn't be needed
+  }
+  DP->readLock();
+  _tagName->setText(DP->tagName());
 
   updatePluginList();
 
@@ -166,6 +160,7 @@ void KstPluginDialogI::_fillFieldsForEdit() {
   pluginChanged(PluginCombo->currentItem());
 
   fillVectorScalarCombos(DP->plugin());
+  DP->readUnlock();
 
   show();
   fixupLayout();
@@ -174,8 +169,7 @@ void KstPluginDialogI::_fillFieldsForEdit() {
 void KstPluginDialogI::_fillFieldsForNew() {
   updatePluginList();
 
-  KstPluginList plugins =
-    kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
+  KstPluginList plugins = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
 
   /* set tag name */
   QString new_label;
@@ -185,31 +179,28 @@ void KstPluginDialogI::_fillFieldsForNew() {
 
   updatePluginList();
 
-  int i = 0;
-  PluginCombo->setCurrentItem(i);
+  PluginCombo->setCurrentItem(0);
   pluginChanged(PluginCombo->currentItem());
 
-  KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
+  KstSharedPtr<Plugin> plugin = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
 
-  fillVectorScalarCombos(pPtr);
+  fillVectorScalarCombos(plugin);
 
 }
 
-void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> pPtr) {
-  QString selectedVector;
-  QString selectedScalar;
+void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> plugin) {
   bool DPvalid = false;
 
-  if (DP != 0) {
-    if (DP.data() && DP->isValid()) {
-      DPvalid = true;
-    }
+  if (DP && DP->isValid()) {
+    DPvalid = true;
   }
 
-
-  if( pPtr != 0 ) {
+  if (plugin) {
+    if (DPvalid) {
+      DP->readLock();
+    }
     // Update input vector and scalar combos
-    const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
+    const QValueList<Plugin::Data::IOValue>& itable = plugin->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin();
          it != itable.end(); ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
@@ -217,7 +208,7 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> pPtr) {
         assert(field);
         if (field) {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
-          selectedVector = vs->selectedVector();
+          QString selectedVector = vs->selectedVector();
           vs->update();
           if (DPvalid) {
             vs->setSelection(DP->inputVectors()[(*it)._name]->tagName());
@@ -230,8 +221,8 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> pPtr) {
         assert(field);
         if (field) {
           ScalarSelector *ss = static_cast<ScalarSelector*>(field);
-          selectedScalar = ss->selectedScalar( );
-            ss->update();
+          QString selectedScalar = ss->selectedScalar();
+          ss->update();
           if (DPvalid) {
             ss->setSelection(DP->inputScalars()[(*it)._name]->tagName());
           } else {
@@ -243,7 +234,7 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> pPtr) {
 
     // Update output vector and scalar lineedits
     if (DPvalid) {
-      const QValueList<Plugin::Data::IOValue>& otable = pPtr->data()._outputs;
+      const QValueList<Plugin::Data::IOValue>& otable = plugin->data()._outputs;
       for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
         assert(field);
@@ -258,6 +249,7 @@ void KstPluginDialogI::fillVectorScalarCombos(KstSharedPtr<Plugin> pPtr) {
           li->setText(ts);
         }
       }
+      DP->readUnlock();
     }
   } else { // invalid plugin
     PluginCollection *pc = PluginCollection::self();
@@ -295,6 +287,12 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
       assert(field);
       VectorSelector *vs = static_cast<VectorSelector*>(field);
       KstVectorPtr v = *KST::vectorList.findTag(vs->selectedVector());
+      if (v) {
+        v->writeLock(); // to match with plugin->writeLock()
+      }
+      if (plugin->inputVectors().contains((*it)._name) && plugin->inputVectors()[(*it)._name] != v) {
+        plugin->inputVectors()[(*it)._name]->writeUnlock();
+      }
       plugin->inputVectors().insert((*it)._name, v);
     } else {
       QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
@@ -306,19 +304,31 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
         double val = ss->_scalar->currentText().toDouble(&ok);
 
         if (ok) {
-          KstScalarPtr pSNew = new KstScalar(ss->_scalar->currentText(), val, true, false, false);
-          plugin->inputScalars().insert((*it)._name, pSNew);
+          KstScalarPtr newScalar = new KstScalar(ss->_scalar->currentText(), val, true, false, false);
+          newScalar->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
+            plugin->inputScalars()[(*it)._name]->writeUnlock();
+          }
+          plugin->inputScalars().insert((*it)._name, newScalar);
         } else {
+          s->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
+            plugin->inputScalars()[(*it)._name]->writeUnlock();
+          }
           plugin->inputScalars().insert((*it)._name, s);
         }
       } else {
+        s->writeLock(); // to match with plugin->writeLock()
+        if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
+          plugin->inputScalars()[(*it)._name]->writeUnlock();
+        }
         plugin->inputScalars().insert((*it)._name, s);
       }
     }
   }
   KST::scalarList.lock().readUnlock();
   KST::vectorList.lock().readUnlock();
-return true;
+  return true;
 }
 
 
@@ -336,7 +346,7 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
       if (tagName.endsWith(newPluginString)) {
         tagName = plugin->tagName();
       }
-      li->setText(tagName+"-"+(*it)._name);
+      li->setText(tagName + "-" + (*it)._name);
     }
 
     QString nt = li->text();
@@ -347,16 +357,40 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         while (KST::vectorTagNameNotUnique(nt, false)) {
           nt += "'";
         }
-        plugin->outputVectors()[(*it)._name]->setTagName(nt);
+        KstVectorPtr v;
+        if (plugin->outputVectors().contains((*it)._name)) {
+          v = plugin->outputVectors()[(*it)._name];
+        } else {
+          v = new KstVector;
+          v->writeLock();
+          plugin->outputVectors().insert((*it)._name, v);
+        }
+        v->setTagName(nt);
       }
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
-        plugin->outputScalars()[(*it)._name]->setTagName(nt);
+        KstScalarPtr s;
+        if (plugin->outputScalars().contains((*it)._name)) {
+          s = plugin->outputScalars()[(*it)._name];
+        } else {
+          s = new KstScalar;
+          s->writeLock();
+          plugin->outputScalars().insert((*it)._name, s);
+        }
+        s->setTagName(nt);
       } else if (plugin->outputScalars()[(*it)._name]->tagName() != nt) {
         while (KST::vectorTagNameNotUnique(nt, false)) {
           nt += "'";
         }
-        plugin->outputScalars()[(*it)._name]->setTagName(nt);
+        KstScalarPtr s;
+        if (plugin->outputScalars().contains((*it)._name)) {
+          s = plugin->outputScalars()[(*it)._name];
+        } else {
+          s = new KstScalar;
+          s->writeLock();
+          plugin->outputScalars().insert((*it)._name, s);
+        }
+        s->setTagName(nt);
       }
     }
   }
@@ -369,19 +403,19 @@ bool KstPluginDialogI::new_I() {
   KstPluginPtr plugin;
   QString tagName = _tagName->text();
 
-  if (!tagName.endsWith(newPluginString)) {
-    if (KST::dataTagNameNotUnique(tagName, true, this)) {
-      _tagName->setFocus();
-      return false;
-    }
+  if (!tagName.endsWith(newPluginString) && KST::dataTagNameNotUnique(tagName, true, this)) {
+    _tagName->setFocus();
+    return false;
   }
 
   int pitem = PluginCombo->currentItem();
   if (pitem >= 0 && PluginCombo->count() > 0) {
     KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
-    if (pPtr != 0L) {
+    if (pPtr) {
       plugin = new KstPlugin;
+      plugin->writeLock();
       if (!saveInputs(plugin, pPtr)) {
+        plugin->writeUnlock();
         plugin = 0L;
         return false;
       }
@@ -398,9 +432,11 @@ bool KstPluginDialogI::new_I() {
       }
       plugin->setTagName(tagName);
       if (!saveOutputs(plugin, pPtr)) {
+        plugin->writeUnlock();
         plugin = 0L;
         return false;
       }
+      plugin->writeUnlock();
     }
   }
 
@@ -422,23 +458,29 @@ bool KstPluginDialogI::new_I() {
 
 
 bool KstPluginDialogI::edit_I() {
+  if (!DP) { // something is dreadfully wrong - this should never happen
+    return false;
+  }
 
-  if (_tagName->text() != DP->tagName()) {
-    if (KST::dataTagNameNotUnique(_tagName->text())) {
-      _tagName->setFocus();
-      return false;
-    }
+  DP->writeLock();
+  if (_tagName->text() != DP->tagName() && KST::dataTagNameNotUnique(_tagName->text())) {
+    _tagName->setFocus();
+    DP->writeUnlock();
+    return false;
   }
 
   DP->setTagName(_tagName->text());
 
-  if (!DP.data()) { // something is dreadfully wrong - this should never happen
-    return false;
-  }
-
   int pitem = PluginCombo->currentItem();
   KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
 
+  // Must unlock before clear()
+  for (KstVectorMap::Iterator i = DP->inputVectors().begin(); i != DP->inputVectors().end(); ++i) {
+    (*i)->writeUnlock();
+  }
+  for (KstScalarMap::Iterator i = DP->inputScalars().begin(); i != DP->inputScalars().end(); ++i) {
+    (*i)->writeUnlock();
+  }
   DP->inputVectors().clear();
   DP->inputScalars().clear();
 
@@ -446,6 +488,7 @@ bool KstPluginDialogI::edit_I() {
   if (!saveInputs(DP, pPtr)) {
     KMessageBox::sorry(this, i18n("There is an error in the plugin you "
                                   "entered. Please fix it."));
+    DP->writeUnlock();
     return false;
   }
 
@@ -456,14 +499,18 @@ bool KstPluginDialogI::edit_I() {
   if (!saveOutputs(DP, pPtr)) {
     KMessageBox::sorry(this, i18n("There is an error in the plugin you "
                                   "entered. Please fix it."));
+    DP->writeUnlock();
     return false;
   }
 
   if (!DP->isValid()) {
     KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
+    DP->writeUnlock();
     return false;
   }
   DP->update(-1);
+  DP->writeUnlock();
+
   emit modified();
   return true;
 }
@@ -488,7 +535,8 @@ QString scalarLabelTemplate, vectorLabelTemplate;
         scalar = true;
         break;
       case Plugin::Data::IOValue::TableType:
-        if ((*it)._subType == Plugin::Data::IOValue::FloatSubType) {
+        if ((*it)._subType == Plugin::Data::IOValue::FloatSubType ||
+            (*it)._subType == Plugin::Data::IOValue::FloatNonVectorSubType) {
           labellabel = vectorLabelTemplate.arg((*it)._name);
         } else {
           // unsupported
@@ -513,7 +561,9 @@ QString scalarLabelTemplate, vectorLabelTemplate;
         KstScalarPtr p = *KST::scalarList.findTag(w->_scalar->currentText());
         w->allowDirectEntry( true );
         if (p) {
+          p->readLock();
           QToolTip::add(w->_scalar, QString::number(p->value()));
+          p->readUnlock();
         }
       } else {
         widget = new VectorSelector(parent, (*it)._name.latin1());
@@ -563,7 +613,7 @@ void KstPluginDialogI::pluginChanged(int idx) {
     grid->setMargin(6);
     grid->setSpacing(5);
     grid->addWidget(new QLabel(i18n("Plugin name:"), _frameWidget), 0, 0);
-    grid->addWidget(new QLabel(pluginData._name, _frameWidget), 0, 1);
+    grid->addWidget(new QLabel(pluginData._readableName, _frameWidget), 0, 1);
     grid->addWidget(new QLabel(i18n("Description:"), _frameWidget), 1, 0);
     QLabel *lab = new QLabel(pluginData._description, _frameWidget);
     lab->setAlignment(lab->alignment() | Qt::WordBreak);
@@ -612,8 +662,10 @@ void KstPluginDialogI::updateScalarTooltip(const QString& n) {
   KstScalarPtr s = *KST::scalarList.findTag(n);
   QWidget *w = const_cast<QWidget*>(static_cast<const QWidget*>(sender()));
   if (s) {
+    s->readLock();
     QToolTip::remove(w);
     QToolTip::add(w, QString::number(s->value()));
+    s->readUnlock();
   } else {
     QToolTip::remove(w);
   }
