@@ -21,8 +21,6 @@
 #include <qstringlist.h>
 #include "kstplugin.h"
 
-class KstDoc;
-
 #include "plugindialog.h"
 
 class KstPluginDialogI : public KstPluginDialog {
@@ -33,15 +31,11 @@ public:
   virtual ~KstPluginDialogI();
 public slots:
   /** update the entries in the plugin dialog to represent current plugins */
-  void update(int new_index = -1);
+  void updateForm();
+  void update();
 
-  /** Calls update(), then shows/raises the dialog */
-  void show_I(const QString &field = QString::null);
-  void show_New();
-
-  void new_I();
-  void edit_I();
-  void delete_I();
+  bool new_I();
+  bool edit_I();
 
   void updatePluginList();
 
@@ -57,10 +51,30 @@ private:
   QStringList _pluginList;
   QWidget *_frameWidget;
   static KstPluginDialogI* _inst;
+  void fillVectorScalarCombos(KstSharedPtr<Plugin> pPtr);
+  bool saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p);
+  bool saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p);
+  void generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid,
+                       const QValueList<Plugin::Data::IOValue>& table);
+
+  KstPluginPtr _getPtr(const QString &tagin);
+
+  bool _newDialog;
+  KstPluginPtr DP;
+
+/***********************************/
+/** defined in dataobjectdialog.h **/
+public slots:
+  void show_Edit(const QString &field);
+  void show_New();
+  void OK();
+  void Init();
+  void close();
+  void reject();
+private:
+  void _fillFieldsForEdit();
+  void _fillFieldsForNew();
 
-  bool saveInputs(KstPluginPtr plugin, KSharedPtr<Plugin> p);
-  bool saveOutputs(KstPluginPtr plugin, KSharedPtr<Plugin> p);
-  void generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table);
 };
 
 #endif
