@@ -30,7 +30,7 @@ KstPoint::~KstPoint() {
 }
 
 void KstPoint::setType(int new_type) {
-  if ((new_type >=0) && (new_type <= KSTPOINT_MAXTYPE)) {
+  if (new_type >= 0 && new_type < KSTPOINT_MAXTYPE) {
     Type = new_type;
   }
 }
