@@ -9,7 +9,7 @@
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
+ *   it under the terms of the GNU Geeral Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
@@ -29,7 +29,7 @@
 #include "kstdoc.h"
 #include "kstpsdcurve.h"
 #include "kstpsddialog_i.h"
-#include "kstsettings.h"
+#include "kstvectordefaults.h"
 
 extern "C" void rdft(int n, int isgn, double *a);
 
@@ -166,7 +166,7 @@ void KstPSDCurve::commonConstructor(const QString &in_tag, KstVectorPtr in_V,
   last_n_subsets = 0;
   last_n_new = 0;
 
-  KstVectorPtr iv = new KstVector(in_tag+"-freq", 2);
+  KstVectorPtr iv = new KstVector(in_tag+"-freq", PSDLen);
   KST::addVectorToList(iv);
   _fVector = _outputVectors.insert(FVECTOR, iv);
 
@@ -220,7 +220,7 @@ KstObject::UpdateType KstPSDCurve::update(int update_counter) {
   bool force = false;
   KstVectorPtr iv = _inputVectors[INVECTOR];
 
-  double *psd;
+  double *psd,  *f;
 
   if (KstObject::checkUpdateCounter(update_counter))
     return NO_CHANGE;
@@ -244,9 +244,11 @@ KstObject::UpdateType KstPSDCurve::update(int update_counter) {
   }
 
   psd = (*_sVector)->value();
+  f = (*_fVector)->value();
 
   for (i_samp = 0; i_samp < PSDLen; i_samp++) {
     psd[i_samp] = 0;
+    f[i_samp] = i_samp*0.5*Freq/( PSDLen-1 );
   }
 
   for (i_subset = 0; i_subset < n_subsets; i_subset++) {
@@ -335,19 +337,14 @@ KstObject::UpdateType KstPSDCurve::update(int update_counter) {
   MinPosX = 1.0/double(NS) * MaxX;
   MeanX = MaxX/2.0;
 
-  double *f = (*_fVector)->value();
-  f[0] = 0;
-  f[1] = Freq/2.0;
-
   (*_sVector)->update(update_counter);
   (*_fVector)->update(update_counter);
 
   return UPDATE;
 }
 
-void KstPSDCurve::getPoint(int i, double &x, double &y) {
+void KstPSDCurve::point(int i, double &x, double &y) {
   x = (*_fVector)->interpolate(i, NS);
-//xxxdouble(i)/double(NS) * MaxX;
   y = (*_sVector)->interpolate(i, NS);
 }
 
@@ -403,7 +400,7 @@ void KstPSDCurve::setFreq(double in_freq) {
   if (in_freq > 0.0) {
     Freq = in_freq;
   } else {
-    Freq = KstSettings::globalSettings()->psdSampleRate;
+    Freq = KST::vectorDefaults.psdFreq();
   }
 }
 
@@ -424,6 +421,7 @@ void KstPSDCurve::_adjustLengths() {
   if (PSDLen != psdlen) {
     PSDLen = psdlen;
     (*_sVector)->resize(PSDLen);
+    (*_fVector)->resize(PSDLen);
 
     ALen = PSDLen*2;
     delete[] a;
