@@ -24,40 +24,41 @@
 
 class KstPsdDialogI : public KstPsdDialog {
   Q_OBJECT
-public:
-  KstPsdDialogI(QWidget* parent = 0, const char* name = 0,
-                bool modal = FALSE, WFlags fl = 0 );
-  virtual ~KstPsdDialogI();
-public slots:
-  /** update the entries in the psd dialog to represent current psds */
-  void update();
-  void updateWindow();
+  public:
+    KstPsdDialogI(QWidget* parent = 0, const char* name = 0,
+        bool modal = false, WFlags fl = 0 );
+    virtual ~KstPsdDialogI();
 
-  bool new_I();
-  bool edit_I();
+  public slots:
+    /** update the entries in the psd dialog to represent current psds */
+    void update();
+    void updateWindow();
 
-  static KstPsdDialogI *globalInstance();
-private:
-  static KstPsdDialogI *_inst;
+    bool new_I();
+    bool edit_I();
 
-  KstPSDCurvePtr _getPtr(const QString &tagin);
+    static KstPsdDialogI *globalInstance();
+  private:
+    static KstPsdDialogI *_inst;
 
-  bool _newDialog;
-  KstPSDCurvePtr DP;
+    KstPSDCurvePtr _getPtr(const QString& tagin);
 
-/***********************************/
-/** defined in dataobjectdialog.h **/
-public slots:
-  void show_Edit(const QString &field);
-  void show_New();
-  void OK();
-  void Init();
-  void close();
-  void reject();
-private:
-  void _fillFieldsForEdit();
-  void _fillFieldsForNew();
+    bool _newDialog;
+    KstPSDCurvePtr DP;
 
+    /***********************************/
+    /** defined in dataobjectdialog.h **/
+  public slots:
+    void show_Edit(const QString &field);
+    void show_New();
+    void OK();
+    void Init();
+    void close();
+    void reject();
+  private:
+    void _fillFieldsForEdit();
+    void _fillFieldsForNew();
 };
 
 #endif
+// vim: ts=2 sw=2 et
