@@ -17,19 +17,16 @@
  ***************************************************************************/
 
 #include <math.h>
-#include <stdio.h>
 #include <stdlib.h>
 
-#include <qmessagebox.h>
-
 #include <kdebug.h>
 #include <klocale.h>
 
-#include "rwlock.h"
 #include "enodes.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "kstrvector.h"
+#include "rwlock.h"
 
 /** Create a KstRVector: raw data from a file */
 KstRVector::KstRVector(KstDataSourcePtr in_file, const QString &in_field,
@@ -54,9 +51,9 @@ KstRVector::KstRVector(QDomElement &e, const QString &o_file,
 
   /* parse the DOM tree */
   QDomNode n = e.firstChild();
-  while( !n.isNull() ) {
-    QDomElement e = n.toElement(); // try to convert the node to an element.
-    if( !e.isNull() ) { // the node was really an element.
+  while (!n.isNull()) {
+    QDomElement e = n.toElement();
+    if (!e.isNull()) {
       if (e.tagName() == "tag") {
         setTagName(e.text());
       } else if (e.tagName() == "filename") {
@@ -125,7 +122,7 @@ void KstRVector::commonRVConstructor(KstDataSourcePtr in_file,
   Skip = in_skip;
   DoSkip = in_DoSkip;
   DoAve = in_DoAve;
-  Field = in_field;
+  _field = in_field;
 
   if (DoSkip && Skip < 1) {
     Skip = 1;
@@ -138,7 +135,7 @@ void KstRVector::commonRVConstructor(KstDataSourcePtr in_file,
   }
 
   if (_file) {
-    SPF = _file->samplesPerFrame(Field);
+    SPF = _file->samplesPerFrame(_field);
   }
 
   _dirty = true;
@@ -166,7 +163,7 @@ void KstRVector::change(KstDataSourcePtr in_file, const QString &in_field,
   _file = in_file;
   ReqF0 = in_f0;
   ReqNF = in_n;
-  Field = in_field;
+  _field = in_field;
   setTagName(in_tag);
 
   reset();
@@ -193,9 +190,8 @@ void KstRVector::changeFrames(int in_f0, int in_n,
   Skip = in_skip;
   DoSkip = in_DoSkip;
   DoAve = in_DoAve;
-  if (DoSkip) {
-    if (Skip < 1)
-      Skip = 1;
+  if (DoSkip && Skip < 1) {
+    Skip = 1;
   }
 
   ReqF0 = in_f0;
@@ -203,19 +199,17 @@ void KstRVector::changeFrames(int in_f0, int in_n,
 
   // ReqNF <=0 means read to end of file
   // ReqF0 < 0 means count back ReqNF from end of file
-  if (ReqNF <= 0) {
-    if (ReqF0 < 0)
-      ReqF0 = 0;
+  if (ReqNF <= 0 && ReqF0 < 0) {
+    ReqF0 = 0;
   }
 }
 
 KstRVector::~KstRVector() {  
   _file = 0;
 
-  if (AveReadBuf != NULL) {
+  if (AveReadBuf) {
     free(AveReadBuf);
-
-   AveReadBuf = NULL;
+    AveReadBuf = 0L;
   }
 }
 
@@ -270,7 +264,7 @@ void KstRVector::save(QTextStream &ts) {
 
   ts << "  <tag>" << tagName() << "</tag>" << endl;
   ts << "  <filename>" << _file->fileName() << "</filename>" << endl;
-  ts << "  <field>" << Field << "</field>" << endl;
+  ts << "  <field>" << _field << "</field>" << endl;
   ts << "  <start>" << ReqF0 << "</start>" << endl;
   ts << "  <num>" << ReqNF << "</num>" << endl;
   if (doSkip()) {
@@ -287,26 +281,26 @@ QString KstRVector::filename() const {
 }
 
 /** return the field */
-QString KstRVector::getField() const {
-  return Field;
+const QString& KstRVector::field() const {
+  return _field;
 }
 
 QString KstRVector::label() const {
   bool ok;
   QString label;
 
-  Field.toInt(&ok);
+  _field.toInt(&ok);
   if (ok && _file && _file->fileType() == "ASCII") {
-    label = i18n("Column %1").arg(Field);
+    label = i18n("Column %1").arg(_field);
   } else {
-    label = Field;
+    label = _field;
   }
 
   return label;
 }
 
 void KstRVector::reset() {
-  SPF = _file ? _file->samplesPerFrame(Field) : SPF;
+  SPF = _file ? _file->samplesPerFrame(_field) : SPF;
   F0 = NF = 0;
   resize(0);
   _numSamples = 0;
@@ -324,7 +318,7 @@ void KstRVector::checkIntegrity() {
   }
 
   /* if it looks like we have a new file, reset */
-  if (_file && (SPF != _file->samplesPerFrame(Field.latin1()) || _file->frameCount() < NF)) {
+  if (_file && (SPF != _file->samplesPerFrame(_field) || _file->frameCount(_field) < NF)) {
     reset();
   }
 
@@ -376,18 +370,18 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   //  > 1        >=0       Read ReqNF frames starting at frame ReqF0
   if (ReqNF < 1) { // read to end of file
     new_f0 = ReqF0;
-    new_nf = _file->frameCount() - new_f0;
+    new_nf = _file->frameCount(_field) - new_f0;
   } else if (ReqF0 < 0) { // count back from end of file
-    new_nf = _file->frameCount();
+    new_nf = _file->frameCount(_field);
     if (new_nf > ReqNF) {
       new_nf = ReqNF;
     }
-    new_f0 = _file->frameCount() - new_nf;
+    new_f0 = _file->frameCount(_field) - new_nf;
   } else {
     new_f0 = ReqF0;
     new_nf = ReqNF;
-    if (new_f0 + new_nf > _file->frameCount()) {
-      new_nf = _file->frameCount() - new_f0;
+    if (new_f0 + new_nf > _file->frameCount(_field)) {
+      new_nf = _file->frameCount(_field) - new_f0;
     }
     if (new_nf <= 0) {
       new_nf = new_f0 = 0;
@@ -429,7 +423,10 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   if (DoSkip) {
     /**** Reallocate V if necessary ****/
     if (new_nf / Skip != _size) {
-      resize(new_nf/Skip);
+      bool rc = resize(new_nf/Skip);
+      if (!rc) {
+        // FIXME: handle failed resize
+      }
     }
     n_read = 0;
     /** read each sample from the File */
@@ -440,8 +437,11 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
           N_AveReadBuf = Skip*SPF;
           AveReadBuf = static_cast<double*>
             (realloc(AveReadBuf, N_AveReadBuf*sizeof(double)));
+          if (!AveReadBuf) {
+            // FIXME: handle failed resize
+          }
         }
-        ave_nread = _file->readField(AveReadBuf, Field, new_f0+i, Skip);
+        ave_nread = _file->readField(AveReadBuf, _field, new_f0+i, Skip);
         for (k = 1; k < ave_nread; k++) {
           AveReadBuf[0] += AveReadBuf[k];
         }
@@ -450,13 +450,16 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
           n_read++;
         }
       } else {
-        n_read += _file->readField(_v + _numSamples + j, Field, new_f0 + i, -1);
+        n_read += _file->readField(_v + _numSamples + j, _field, new_f0 + i, -1);
       }
     }
   } else {
     /**** Reallocate V if necessary ****/
     if ((new_nf - 1)*SPF + 1 != _size) {
-      resize((new_nf - 1)*SPF + 1);
+      bool rc = resize((new_nf - 1)*SPF + 1);
+      if (!rc) {
+        // FIXME: handle failed resize
+      }
     }
 
     if (NF > 0) {
@@ -464,8 +467,8 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
     }
 
     /**** Read the new data from File ****/
-    n_read = _file->readField(_v+NF*SPF, Field, new_f0 + NF, new_nf - NF - 1);
-    n_read += _file->readField(_v+(new_nf-1)*SPF, Field, new_f0 + new_nf-1, -1);
+    n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF - 1);
+    n_read += _file->readField(_v+(new_nf-1)*SPF, _field, new_f0 + new_nf-1, -1);
   }
 
   NumNew = _size - _numSamples;
@@ -508,11 +511,11 @@ int KstRVector::samplesPerFrame() const {
 
 /** return true if it has a valid file and field, or false otherwise */
 bool KstRVector::isValid() const {
-  return _file && _file->isValidField(Field);
+  return _file && _file->isValidField(_field);
 }
 
 int KstRVector::fileLength() const {
-  return _file ? _file->frameCount() : 0;
+  return _file ? _file->frameCount(_field) : 0;
 }
 
 void KstRVector::reload() {
@@ -520,7 +523,12 @@ void KstRVector::reload() {
     return;
   }
 
-  // FIXME: inefficient
+  if (_file->reset()) { // Try the efficient way first
+    reset();
+    return;
+  }
+
+  // The inefficient way
   KstDataSourcePtr newsrc = KstDataSource::loadSource(_file->fileName(), _file->fileType());
   if (newsrc) {
     KST::dataSourceList.lock().writeLock();
@@ -532,4 +540,9 @@ void KstRVector::reload() {
   }
 }
 
+KstDataSourcePtr KstRVector::dataSource() const {
+  return _file;
+}
+
+
 // vim: ts=2 sw=2 et
