@@ -17,7 +17,6 @@
 
 #include "kstscalar.h"
 #include "kstdatacollection.h"
-#include "kstdoc.h"
 #include <klocale.h>
 
 #include <qstylesheet.h>
@@ -25,7 +24,7 @@
 static int iAnonymousScalarCounter = 0;
 
 /** Create the base scalar */
-KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displayable, bool bLock)
+KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displayable, bool doLock)
 : KstObject(), _orphan(orphan), _displayable(displayable) {
   QString _tag = in_tag;
   if (_tag.isEmpty()) {
@@ -33,20 +32,21 @@ KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displa
     
     do {
       _tag = nt.arg(iAnonymousScalarCounter++);
-    } while (KST::vectorTagNameNotUnique(_tag, false));
+    } while (KST::vectorTagNameNotUniqueInternal(_tag));
   } else {
-    while (KST::vectorTagNameNotUnique(_tag, false)) {
-      _tag += "'";
+    while (KST::vectorTagNameNotUniqueInternal(_tag)) {
+      _tag += '\'';
     }
   }
   setTagName(_tag);
   _value = val;
 
-  if (bLock) {
+  // FIXME: passing in a lock variable indicates a design problem
+  if (doLock) {
     KST::scalarList.lock().writeLock();
   }
   KST::scalarList.append(this);
-  if (bLock) {
+  if (doLock) {
     KST::scalarList.lock().writeUnlock();
   }
 }
