@@ -16,7 +16,6 @@ void KstSettingsDlg::init()
 
 void KstSettingsDlg::setSettings(const KstSettings *settings)
 {
-    _psdSampleRate->setValue(settings->psdSampleRate);
     _timer->setValue(settings->plotUpdateTimer);
     _colors->setBackground(settings->backgroundColor);
     _colors->setForeground(settings->foregroundColor);
@@ -46,7 +45,6 @@ void KstSettingsDlg::setClean()
 void KstSettingsDlg::save()
 {
     KstSettings s;
-    s.psdSampleRate = _psdSampleRate->value();
     s.plotUpdateTimer = _timer->value();
     s.backgroundColor = _colors->background();
     s.foregroundColor = _colors->foreground();
