@@ -2,7 +2,7 @@
                 ksttoplevelview.cpp: toplevel view objects
                              -------------------
     begin                : Mar 11, 2004
-    copyright            : (C) 2003 The University of Toronto
+    copyright            : (C) 2004 The University of Toronto
     email                :
  ***************************************************************************/
 
@@ -28,8 +28,10 @@
 #include <kprinter.h>
 
 // application specific includes
-#include "ksttoplevelview.h"
+#include "kst.h"
+#include "kstdoc.h"
 #include "kstplotgroup.h"
+#include "ksttoplevelview.h"
 #include "kstviewwidget.h"
 
 KstTopLevelView::KstTopLevelView(QWidget *parent, const char *name, WFlags w)
@@ -73,11 +75,12 @@ void KstTopLevelView::save(QTextStream& ts) {
 
 
 void KstTopLevelView::paint(KstPaintType type, QPainter& p) {
-  updateAlignment();
+  if (type != P_PRINT && type != P_EXPORT) {
+    updateAlignment();
+  }
   QRegion clipRegion(geometry());
   p.setClipRegion(clipRegion);
   KstViewObject::paint(type, p);
-  // Anything to draw here?
 }
 
 
@@ -86,13 +89,15 @@ void KstTopLevelView::resized(const QSize& size) {
 }
 
 void KstTopLevelView::updateAlignment() {
-  double x_max=0,  x;
+  double x_max = 0,  x;
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     x = (*i)->xInternalAlignment();
-    if ( x>x_max ) x_max = x;
+    if (x > x_max) {
+      x_max = x;
+    }
   }
 
-  KST::alignment.setX( x_max );
+  KST::alignment.setX(x_max);
 }
 
 void KstTopLevelView::resize(const QSize& size) {
@@ -115,6 +120,7 @@ void KstTopLevelView::paint(KstPaintType type, bool erase) {
       r = r.subtract(QRegion((*i)->geometry()));
     }
     p.setClipRegion(r);
+    _lastClipRegion = r;
     QBrush brush(_backgroundColor);
     p.fillRect(geometry(), brush);
     p.setClipping(false);
@@ -431,7 +437,7 @@ static void slideInto(const QRect& region, QRect& obj) {
 
 
 // Optimize me: can store, for instance, the painter I think
-void KstTopLevelView::pressMove(const QPoint& pos) {
+void KstTopLevelView::pressMove(const QPoint& pos, bool shift) {
   if (_mode != LayoutMode) {
     _pressTarget = 0L;
     return;
@@ -441,6 +447,10 @@ void KstTopLevelView::pressMove(const QPoint& pos) {
     return;
   }
 
+  if (shift && _moveOffset == QPoint(-1, -1)) {
+    return;
+  }
+
   if (_pressTarget) {
     if (_pressDirection == 0) {
       QRect old = _prevBand;
@@ -507,6 +517,7 @@ void KstTopLevelView::releasePress(const QPoint& pos, bool shift) {
   }
 
   if (_pressTarget) {
+    KstApp::inst()->document()->setModified();
     if (_pressDirection == 0) {
       QRect obj = _pressTarget->geometry();
       QRect old = obj;
@@ -601,28 +612,30 @@ bool KstTopLevelView::popupMenu(KPopupMenu *menu, const QPoint& pos) {
 
   if (_selectionList.count() > 1) {
     if (_pressTarget && _mode == LayoutMode) {
-      KPopupMenu* pPopup;
+      KPopupMenu* subMenu;
 
       if (rc) {
         menu->insertSeparator();
       }
 
-      pPopup = new KPopupMenu(menu);
-      if (pPopup) {
-        pPopup->insertItem(i18n("Width"), this, SLOT(makeSameWidth()));
-        pPopup->insertItem(i18n("Height"), this, SLOT(makeSameHeight()));
-        pPopup->insertItem(i18n("Size"), this, SLOT(makeSameSize()));
-        menu->insertItem(i18n("Make Same"), pPopup);
-      }
+      subMenu = new KPopupMenu(menu);
+      subMenu->insertItem(i18n("Width"), this, SLOT(makeSameWidth()));
+      subMenu->insertItem(i18n("Height"), this, SLOT(makeSameHeight()));
+      subMenu->insertItem(i18n("Size"), this, SLOT(makeSameSize()));
+      menu->insertItem(i18n("Make Same"), subMenu);
+
+      subMenu = new KPopupMenu(menu);
+      subMenu->insertItem(i18n("Left"), this, SLOT(alignLeft()));
+      subMenu->insertItem(i18n("Right"), this, SLOT(alignRight()));
+      subMenu->insertItem(i18n("Top"), this, SLOT(alignTop()));
+      subMenu->insertItem(i18n("Bottom"), this, SLOT(alignBottom()));
+      menu->insertItem(i18n("Align"), subMenu);
+      
+      subMenu = new KPopupMenu(menu);
+      subMenu->insertItem(i18n("Horizontally"), this, SLOT(packHorizontally()));
+      subMenu->insertItem(i18n("Vertically"), this, SLOT(packVertically()));
+      menu->insertItem(i18n("Pack"), subMenu);
 
-      pPopup = new KPopupMenu(menu);
-      if (pPopup) {
-        pPopup->insertItem(i18n("Left"), this, SLOT(alignLeft()));
-        pPopup->insertItem(i18n("Right"), this, SLOT(alignRight()));
-        pPopup->insertItem(i18n("Top"), this, SLOT(alignTop()));
-        pPopup->insertItem(i18n("Bottom"), this, SLOT(alignBottom()));
-        menu->insertItem(i18n("Align"), pPopup);
-      }
       menu->insertSeparator();
     }
     menu->insertItem(i18n("Group Objects"), this, SLOT(groupSelection()));
@@ -637,12 +650,19 @@ bool KstTopLevelView::popupMenu(KPopupMenu *menu, const QPoint& pos) {
     rc = true;
   }
 
+  //
+  // don't release the _pressTarget reference here as we are not yet done with
+  //  it, since several of the menu handlers require it in order to function...
+  // _pressTarget = 0L; // release reference
+  //
+  
   return rc;
 }
 
 
 void KstTopLevelView::makeSameWidth() {
   if (_pressTarget) {
+    KstApp::inst()->document()->setModified();
     QRect gg = _pressTarget->geometry();
     QSize size;
     size.setWidth( gg.width() );
@@ -650,14 +670,14 @@ void KstTopLevelView::makeSameWidth() {
       size.setHeight((*i)->geometry().height());
       (*i)->resize(size);
     }
+    paint(P_PAINT);
   }
-
-  paint(P_PAINT);
 }
 
 
 void KstTopLevelView::makeSameHeight() {
   if (_pressTarget) {
+    KstApp::inst()->document()->setModified();
     QRect gg = _pressTarget->geometry();
     QSize size;
     size.setHeight( gg.height() );
@@ -665,13 +685,14 @@ void KstTopLevelView::makeSameHeight() {
       size.setWidth((*i)->geometry().width());
       (*i)->resize(size);
     }
+    paint(P_PAINT);
   }
-  paint(P_PAINT);
 }
 
 
 void KstTopLevelView::makeSameSize() {
   if (_pressTarget) {
+    KstApp::inst()->document()->setModified();
     QSize size;
     QRect gg = _pressTarget->geometry();
     size.setHeight(gg.height());
@@ -679,24 +700,25 @@ void KstTopLevelView::makeSameSize() {
     for (KstViewObjectList::Iterator i = _selectionList.begin(); i != _selectionList.end(); ++i) {
       (*i)->resize(size);
     }
+    paint(P_PAINT);
   }
-  paint(P_PAINT);
 }
 
 
-void KstTopLevelView::checkPosition( KstViewObjectPtr pObject, QPoint point ) {
-  QRect rect = pObject->geometry();
+void KstTopLevelView::checkPosition(KstViewObjectPtr obj, QPoint point) {
+  QRect rect = obj->geometry();
   rect.moveTopLeft(point);
   if (!_geom.contains(rect, true)) {
     slideInto(_geom, rect);
     point = rect.topLeft();
   }
-  pObject->move(point);
+  obj->move(point);
 }
 
 
 void KstTopLevelView::alignLeft() {
   if (_pressTarget) {
+    KstApp::inst()->document()->setModified();
     QRect gg = _pressTarget->geometry();
     QPoint point;
     point.setX(gg.x());
@@ -704,13 +726,14 @@ void KstTopLevelView::alignLeft() {
       point.setY((*i)->geometry().y());
       checkPosition(*i, point);
     }
+    paint(P_PAINT);
   }
-  paint(P_PAINT);
 }
 
 
 void KstTopLevelView::alignRight() {
   if (_pressTarget) {
+    KstApp::inst()->document()->setModified();
     QRect gg = _pressTarget->geometry();
     QPoint point;
     for (KstViewObjectList::Iterator i = _selectionList.begin(); i != _selectionList.end(); ++i) {
@@ -718,13 +741,14 @@ void KstTopLevelView::alignRight() {
       point.setY((*i)->geometry().y());
       checkPosition(*i, point);
     }
+    paint(P_PAINT);
   }
-  paint(P_PAINT);
 }
 
 
 void KstTopLevelView::alignTop() {
   if (_pressTarget) {
+    KstApp::inst()->document()->setModified();
     QRect gg = _pressTarget->geometry();
     QPoint point;
     point.setY(gg.y());
@@ -732,13 +756,14 @@ void KstTopLevelView::alignTop() {
       point.setX((*i)->geometry().x());
       checkPosition(*i, point);
     }
+    paint(P_PAINT);
   }
-  paint(P_PAINT);
 }
 
 
 void KstTopLevelView::alignBottom() {
   if (_pressTarget) {
+    KstApp::inst()->document()->setModified();
     QRect gg = _pressTarget->geometry();
     QPoint point;
     for (KstViewObjectList::Iterator i = _selectionList.begin(); i != _selectionList.end(); ++i) {
@@ -746,8 +771,118 @@ void KstTopLevelView::alignBottom() {
       point.setY(gg.y() + gg.height() - (*i)->geometry().height());
       checkPosition(*i, point);
     }
+    paint(P_PAINT);
+  }
+}
+
+
+void KstTopLevelView::packVertically() {
+  int iTop = 0;
+  int iBottom = 0;
+  int iCount = 0;
+
+  for (KstViewObjectList::Iterator i = _selectionList.begin(); i != _selectionList.end(); ++i) {
+    if (i == _selectionList.begin()) {
+      iTop = (*i)->geometry().y();
+      iBottom = (*i)->geometry().y() + (*i)->geometry().height();
+    } else {
+      if ((*i)->geometry().y() < iTop) {
+        iTop = (*i)->geometry().y();
+      }
+      if ((*i)->geometry().y() + (*i)->geometry().height() > iBottom) {
+        iBottom = (*i)->geometry().y() + (*i)->geometry().height();
+      }
+    }
+    iCount++;
+  }
+
+  if (iBottom > iTop && iCount > 0) {
+    KstViewObjectList selectionList = _selectionList;
+    KstViewObjectList::Iterator temp;
+    QPoint point;
+    QSize size;
+    int iPosMin = 0;
+    int iPos = iTop;
+    int iHeight = (iBottom - iTop) / iCount;
+
+    while (!selectionList.empty()) {
+      for (KstViewObjectList::Iterator i = selectionList.begin(); i != selectionList.end(); ++i) {
+        if (i == selectionList.begin()) {
+          iPosMin = (*i)->geometry().y();
+          temp = i;
+        } else if ((*i)->geometry().y() < iPosMin) {
+          iPosMin = (*i)->geometry().y();
+          temp = i;
+        }
+      }
+      size.setWidth((*temp)->geometry().width());
+      size.setHeight(iHeight);
+      (*temp)->resize(size);
+      point.setX((*temp)->geometry().x());
+      point.setY(iPos);
+      checkPosition(*temp, point);
+
+      selectionList.erase(temp);
+      iPos += iHeight;
+    }
+    KstApp::inst()->document()->setModified();
+    paint(P_PAINT);
+  }
+}
+
+
+void KstTopLevelView::packHorizontally() {
+  int iLeft = 0;
+  int iRight = 0;
+  int iCount = 0;
+  
+  for (KstViewObjectList::Iterator i = _selectionList.begin(); i != _selectionList.end(); ++i) {
+    if (i == _selectionList.begin()) {
+      iLeft = (*i)->geometry().x();
+      iRight = (*i)->geometry().x() + (*i)->geometry().width();
+    } else {
+      if ((*i)->geometry().x() < iLeft) {
+        iLeft = (*i)->geometry().x();
+      }
+      if ((*i)->geometry().x() + (*i)->geometry().width() > iRight) {
+        iRight = (*i)->geometry().x() + (*i)->geometry().width();
+      }
+    }
+    iCount++;
+  }
+
+  if (iRight > iLeft && iCount > 0) {
+    KstViewObjectList selectionList = _selectionList;
+    KstViewObjectList::Iterator temp;
+    QPoint point;
+    QSize size;
+    int iPosMin = 0;
+    int iPos = iLeft;
+    int iWidth = (iRight - iLeft) / iCount;
+    
+    while (!selectionList.empty()) {
+      for (KstViewObjectList::Iterator i = selectionList.begin(); i != selectionList.end(); ++i) {
+        if (i == selectionList.begin()) {
+          iPosMin = (*i)->geometry().x();
+          temp = i;
+        } else if ((*i)->geometry().x() < iPosMin) {
+          iPosMin = (*i)->geometry().x();
+          temp = i;
+        }
+      }
+      size.setWidth(iWidth);
+      size.setHeight((*temp)->geometry().height());
+      (*temp)->resize(size);
+      point.setX(iPos);
+      point.setY((*temp)->geometry().y());
+      checkPosition(*temp, point);
+
+      selectionList.erase(temp);
+      iPos += iWidth;
+    }
+    KstApp::inst()->document()->setModified();
+    paint(P_PAINT);
   }
-  paint(P_PAINT);
 }
 
 
@@ -779,6 +914,7 @@ void KstTopLevelView::groupSelection() {
       appendChild(pg.data());
     }
 
+    KstApp::inst()->document()->setModified();
     paint(P_PAINT);
   }
 }
@@ -821,10 +957,33 @@ KstViewWidget *KstTopLevelView::widget() const {
   return _w;
 }
 
+
 void KstTopLevelView::cleanupAction() {
   cleanup();
 }
 
 
+void KstTopLevelView::release() {
+  _hoverFocus = 0L;
+  _pressTarget = 0L;
+  _mouseGrabber = 0L;
+  _mouseGrabbed = false;
+  _selectionList.clear();
+  clearChildren(); // FIXME: remove this once we get rid of _parent
+  if (_w) {
+    _w->release();
+  }
+}
+
+
+QSize KstTopLevelView::averageChildSize() {
+  int widths = 0, heights = 0;
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    widths += (*i)->geometry().width();
+    heights += (*i)->geometry().height();
+  }
+  return QSize(widths / _children.count(), heights / _children.count());
+}
+
 #include "ksttoplevelview.moc"
 // vim: ts=2 sw=2 et
