@@ -138,7 +138,9 @@ bool KstVCurve::loadInputs() {
   if (EY) {
     EY->readLock();
   }
-  update(); // FIXME: this update happens in the wrong thread
+  update(); // This happens in the GUI thread.  Might be a good idea to provide
+            // a mechanism to ask the update thread to update an object instead.
+            // (which would save us all this ugly locking too :))
   VX->readUnlock();
   VY->readUnlock();
   if (EX) {
@@ -197,7 +199,7 @@ KstObject::UpdateType KstVCurve::update(int update_counter) {
   return UPDATE;
 }
 
-void KstVCurve::getPoint(int i, double &x, double &y) {
+void KstVCurve::point(int i, double &x, double &y) {
   x = VX->interpolate(i, NS);
   y = VY->interpolate(i, NS);
 }
@@ -232,7 +234,7 @@ QString KstVCurve::getYVTag() const{
 
 QString KstVCurve::getXETag() const{
   if (!EX.data()) {
-    return i18n("<None>");
+    return "<None>";
   } else {
     return EX->tagName();
   }
@@ -240,7 +242,7 @@ QString KstVCurve::getXETag() const{
 
 QString KstVCurve::getYETag() const {
   if (!EY.data()) {
-    return i18n("<None>");
+    return "<None>";
   } else {
     return EY->tagName();
   }
