@@ -19,10 +19,7 @@
 #define KSTVCURVE_H
 
 #include "kstrvector.h"
-#include "kstpoint.h"
 #include "kstbasecurve.h"
-#include <qstring.h>
-#include <qcolor.h>
 
 /**A class for handling curves for kst
 
@@ -43,7 +40,7 @@ public:
   virtual bool hasXError() const;
   virtual bool hasYError() const;
 
-  virtual void getPoint(int i, double &x1, double &y1);
+  virtual void point(int i, double &x1, double &y1);
   virtual void getEXPoint(int i, double &x1, double &y1, double &ex1);
   virtual void getEYPoint(int i, double &x1, double &y1, double &ey1);
 
