@@ -23,7 +23,7 @@
 #include <time.h>
 #include <assert.h>
 #include <kdebug.h>
-#include <klocale.h>  
+#include <klocale.h>
 #include "kstdoc.h"
 #include "kstdatacollection.h"
 #include "kstvector.h"
@@ -66,7 +66,7 @@ static inline double nan__()
 #define KST_ISNAN(a)	isnan(a)
 #endif
 
-static int iAnonymousVectorCounter = 0;
+static int anonymousVectorCounter = 0;
 
 // Use 1 for a simple for() loop
 #define ZERO_MEMORY 2
@@ -82,12 +82,13 @@ double KST::NOPOINT = 0.0/0.0; // NaN
 // FIXME: optimize scalar access
 
 /** Create a vector */
-KstVector::KstVector(const QString& name, int size)
-: KstObject(), _nsum(0), _scalars(11) {
+KstVector::KstVector(const QString& name, int size, bool isScalarList)
+: KstObject(), _nsum(0), _scalars(isScalarList ? 0 : 11) {
   //kdDebug() << "+++ CREATING VECTOR: " << (void*) this << endl;
   KstObject::setTagName(name);
   NumShifted = 0;
   NumNew = 0;
+  _isScalarList = isScalarList;
 
   if (size <= 0) {
     size = INITSIZE;
@@ -95,9 +96,9 @@ KstVector::KstVector(const QString& name, int size)
 
   if (name.isEmpty()) {
     QString nt = i18n("Anonymous Vector %1");
-    
+
     do {
-      KstObject::setTagName(nt.arg(iAnonymousVectorCounter++));
+      KstObject::setTagName(nt.arg(anonymousVectorCounter++));
     } while (KST::vectorTagNameNotUnique(tagName(), false));
   } else {
     while (KST::vectorTagNameNotUnique(tagName(), false)) {
@@ -105,7 +106,7 @@ KstVector::KstVector(const QString& name, int size)
     }
   }
 
-  _v = static_cast<double*>(malloc(size*sizeof(double)));
+  _v = static_cast<double*>(KST::malloc(size*sizeof(double)));
   _size = size;
   _is_rising = false;
 
@@ -121,7 +122,7 @@ KstVector::~KstVector() {
     it.current()->_KShared_unref();
   }
   KST::scalarList.lock().writeUnlock();
-  
+
   if (_v) {
     free(_v);
     _v = 0;
@@ -183,50 +184,56 @@ double KstVector::value(int i) {
 }
 
 void KstVector::CreateScalars() {
-  _scalars.insert("max", new KstScalar(tagName() + "-Max"));
-  _scalars["max"]->_KShared_ref();
-  _scalars.insert("min", new KstScalar(tagName() + "-Min"));
-  _scalars["min"]->_KShared_ref();
-  _scalars.insert("mean", new KstScalar(tagName() + "-Mean"));
-  _scalars["mean"]->_KShared_ref();
-  _scalars.insert("sigma", new KstScalar(tagName() + "-Sigma"));
-  _scalars["sigma"]->_KShared_ref();
-  _scalars.insert("rms", new KstScalar(tagName() + "-Rms"));
-  _scalars["rms"]->_KShared_ref();
-  _scalars.insert("ns", new KstScalar(tagName() + "-NS"));
-  _scalars["ns"]->_KShared_ref();
-  _scalars.insert("sum", new KstScalar(tagName() + "-Sum"));
-  _scalars["sum"]->_KShared_ref();
-  _scalars.insert("sumsquared", new KstScalar(tagName() + "-SumSquared"));
-  _scalars["sumsquared"]->_KShared_ref();
-  _scalars.insert("minpos", new KstScalar(tagName() + "-MinPos"));
-  _scalars["minpos"]->_KShared_ref();
+  if (!_isScalarList) {
+    _scalars.insert("max", new KstScalar(tagName() + "-Max"));
+    _scalars["max"]->_KShared_ref();
+    _scalars.insert("min", new KstScalar(tagName() + "-Min"));
+    _scalars["min"]->_KShared_ref();
+    _scalars.insert("mean", new KstScalar(tagName() + "-Mean"));
+    _scalars["mean"]->_KShared_ref();
+    _scalars.insert("sigma", new KstScalar(tagName() + "-Sigma"));
+    _scalars["sigma"]->_KShared_ref();
+    _scalars.insert("rms", new KstScalar(tagName() + "-Rms"));
+    _scalars["rms"]->_KShared_ref();
+    _scalars.insert("ns", new KstScalar(tagName() + "-NS"));
+    _scalars["ns"]->_KShared_ref();
+    _scalars.insert("sum", new KstScalar(tagName() + "-Sum"));
+    _scalars["sum"]->_KShared_ref();
+    _scalars.insert("sumsquared", new KstScalar(tagName() + "-SumSquared"));
+    _scalars["sumsquared"]->_KShared_ref();
+    _scalars.insert("minpos", new KstScalar(tagName() + "-MinPos"));
+    _scalars["minpos"]->_KShared_ref();
+  }
 }
 
 void KstVector::RenameScalars() {
-  _scalars["max"]->setTagName(tagName() + "-Max");
-  _scalars["min"]->setTagName(tagName() + "-Min");
-  _scalars["mean"]->setTagName(tagName() + "-Mean");
-  _scalars["sigma"]->setTagName(tagName() + "-Sigma");
-  _scalars["rms"]->setTagName(tagName() + "-Rms");
-  _scalars["ns"]->setTagName(tagName() + "-NS");
-  _scalars["sum"]->setTagName(tagName() + "-Sum");
-  _scalars["sumsquared"]->setTagName(tagName() + "-SumSquared");
-  _scalars["minpos"]->setTagName(tagName() + "-MinPos");
+  if (!_isScalarList) {
+    _scalars["max"]->setTagName(tagName() + "-Max");
+    _scalars["min"]->setTagName(tagName() + "-Min");
+    _scalars["mean"]->setTagName(tagName() + "-Mean");
+    _scalars["sigma"]->setTagName(tagName() + "-Sigma");
+    _scalars["rms"]->setTagName(tagName() + "-Rms");
+    _scalars["ns"]->setTagName(tagName() + "-NS");
+    _scalars["sum"]->setTagName(tagName() + "-Sum");
+    _scalars["sumsquared"]->setTagName(tagName() + "-SumSquared");
+    _scalars["minpos"]->setTagName(tagName() + "-MinPos");
+  }
 }
 
-void KstVector::UpdateScalars() {
-  _scalars["ns"]->setValue(_size);
+void KstVector::updateScalars() {
+  if (!_isScalarList) {
+    _scalars["ns"]->setValue(_size);
 
-  if (_nsum >= 2) {
-    _scalars["mean"]->setValue(_scalars["sum"]->value()/double(_nsum));
-    _scalars["sigma"]->setValue( sqrt(
-      (_scalars["sumsquared"]->value() - _scalars["sum"]->value()*_scalars["sum"]->value()/double(_nsum))/ double(_nsum-1) ) );
-    _scalars["rms"]->setValue(sqrt(_scalars["sumsquared"]->value()/double(_nsum)));
-  } else {
-    _scalars["sigma"]->setValue(_scalars["max"]->value() - _scalars["min"]->value());
-    _scalars["rms"]->setValue(sqrt(_scalars["sumsquared"]->value()));
-    _scalars["mean"]->setValue(0);
+    if (_nsum >= 2) {
+      _scalars["mean"]->setValue(_scalars["sum"]->value()/double(_nsum));
+      _scalars["sigma"]->setValue( sqrt(
+        (_scalars["sumsquared"]->value() - _scalars["sum"]->value()*_scalars["sum"]->value()/double(_nsum))/ double(_nsum-1) ) );
+      _scalars["rms"]->setValue(sqrt(_scalars["sumsquared"]->value()/double(_nsum)));
+    } else {
+      _scalars["sigma"]->setValue(_scalars["max"]->value() - _scalars["min"]->value());
+      _scalars["rms"]->setValue(sqrt(_scalars["sumsquared"]->value()));
+      _scalars["mean"]->setValue(0);
+    }
   }
 }
 
@@ -234,7 +241,7 @@ double* KstVector::realloced(double *memptr, int newSize) {
   double *old = _v;
   _v = memptr;
   _size = newSize;
-  UpdateScalars();
+  updateScalars();
   return old;
 }
 
@@ -243,14 +250,17 @@ void KstVector::zero() {
   for (int i = 0; i < _size; i++) {
     _v[i] = 0.0;
   }
-  UpdateScalars();
+  updateScalars();
 }
 
 
-void KstVector::resize(int sz, bool reinit) {
+bool KstVector::resize(int sz, bool reinit) {
   //kdDebug() << "resizing to: " << sz << endl;
   if (sz > 1) {
-    _v = static_cast<double*>(realloc(_v, sz*sizeof(double)));
+    _v = static_cast<double*>(KST::realloc(_v, sz*sizeof(double)));
+    if (!_v) {
+      return false;
+    }
 #ifdef ZERO_MEMORY
     if (reinit && _size < sz) {
 #if ZERO_MEMORY == 2
@@ -263,15 +273,14 @@ void KstVector::resize(int sz, bool reinit) {
     }
 #endif
     _size = sz;
-    UpdateScalars();
+    updateScalars();
   }
+
+  return true;
 }
 
 
 KstObject::UpdateType KstVector::update(int update_counter) {
-  QString str;
-  QString Equation;
-  KstVectorList VectorsUsed;
   int i, i0;
   double max, min, sum, sum2, minpos, v;
   double last_v;
@@ -286,18 +295,21 @@ KstObject::UpdateType KstVector::update(int update_counter) {
   if (_size > 0) {
     _is_rising = true;
 
+    // FIXME: expensive!!  Can we somehow cache this at least?
     for (i = 0; i < _size && !finite(_v[i]); i++)
       /* Do Nothing */ ;
 
     if (i == _size) {
-      _scalars["sum"]->setValue(sum);
-      _scalars["sumsquared"]->setValue(sum2);
-      _scalars["max"]->setValue(max);
-      _scalars["min"]->setValue(min);
-      _scalars["minpos"]->setValue(minpos);
+      if (!_isScalarList) {
+        _scalars["sum"]->setValue(sum);
+        _scalars["sumsquared"]->setValue(sum2);
+        _scalars["max"]->setValue(max);
+        _scalars["min"]->setValue(min);
+        _scalars["minpos"]->setValue(minpos);
+      }
       _ns_max = _ns_min = 0;
 
-      UpdateScalars();
+      updateScalars();
       return UPDATE;
     }
 
@@ -307,7 +319,9 @@ KstObject::UpdateType KstVector::update(int update_counter) {
       _is_rising = false;
     }
 
-    max = min = sum = sum2 = _v[i0];
+    max = min = _v[i0];
+    sum = sum2 = 0.0;
+
     if (_v[i0] > 0) {
       minpos = _v[i0];
     } else {
@@ -373,14 +387,16 @@ KstObject::UpdateType KstVector::update(int update_counter) {
       }
     }
 
-    _scalars["sum"]->setValue(sum);
-    _scalars["sumsquared"]->setValue(sum2);
-    _scalars["max"]->setValue(max);
-    _scalars["min"]->setValue(min);
-    _scalars["minpos"]->setValue(minpos);
+    if (!_isScalarList) {
+      _scalars["sum"]->setValue(sum);
+      _scalars["sumsquared"]->setValue(sum2);
+      _scalars["max"]->setValue(max);
+      _scalars["min"]->setValue(min);
+      _scalars["minpos"]->setValue(minpos);
+    }
+
+    updateScalars();
 
-    UpdateScalars();
-    
     return UPDATE;
   }
 
@@ -399,7 +415,7 @@ void KstVector::setTagName(const QString& newTag) {
   RenameScalars();
 }
 
-void KstVector::SetNewAndShift(int inNew, int inShift) {
+void KstVector::setNewAndShift(int inNew, int inShift) {
   NumNew = inNew;
   NumShifted = inShift;
 }
@@ -417,19 +433,35 @@ double *const KstVector::value() const {
 }
 
 double KstVector::min() const {
-  return _scalars["min"]->value();
+  if (!_isScalarList) {
+    return _scalars["min"]->value();
+  } else {
+    return 0.0;
+  }
 }
 
 double KstVector::max() const {
-  return _scalars["max"]->value();
+  if (!_isScalarList) {
+    return _scalars["max"]->value();
+  } else {
+    return 0.0;
+  }
 }
 
 double KstVector::mean() const {
-  return _scalars["mean"]->value();
+  if (!_isScalarList) {
+    return _scalars["mean"]->value();
+  } else {
+    return 0.0;
+  }
 }
 
 double KstVector::minPos() const {
-  return _scalars["minpos"]->value();
+  if (!_isScalarList) {
+    return _scalars["minpos"]->value();
+  } else {
+    return 0.0;
+  }
 }
 
 int KstVector::sampleCount() const {
@@ -488,7 +520,7 @@ KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const QStrin
 
   xv->_scalars["min"]->setValue(x0);
   xv->_scalars["max"]->setValue(x1);
-  xv->UpdateScalars();
+  xv->updateScalars();
 
   return xv;
 }
