@@ -19,10 +19,7 @@
 #define KSTVECTOR_H
 
 #include <math.h>
-#include <qstring.h>
-#include <qdom.h>
 #include <qdict.h>
-#include "kstobject.h"
 #include "kstscalar.h"
 #include "kstdebug.h"
 
@@ -52,7 +49,7 @@ class KstVector : public KstObject {
     /**
      * Vectors automatically add themselves to the global vector collection
      */
-    KstVector(const QString& name = QString::null, int size = 0);
+    KstVector(const QString& name = QString::null, int size = 0, bool bIsScalarList = false);
 
     virtual ~KstVector();
 
@@ -72,10 +69,10 @@ class KstVector : public KstObject {
     double max() const;
 
     /** Return SpikeInsensitive max value in vector **/
-    double ns_max() const {return _ns_max;}
+    double ns_max() const { return _ns_max; }
 
     /** Return SpikeInsensitive min value in vector **/
-    double ns_min() const {return _ns_min;}
+    double ns_min() const { return _ns_min; }
 
     /** Return Mean value in Vector */
     double mean() const;
@@ -92,7 +89,7 @@ class KstVector : public KstObject {
     /** Number of samples  shifted since last newSync */
     int numShift() const;
 
-    bool isRising() const {return _is_rising;}
+    bool isRising() const { return _is_rising; }
 
     /** reset New Samples and Shifted samples */
     void newSync();
@@ -103,15 +100,15 @@ class KstVector : public KstObject {
     /** return a sensible top label.... */
     virtual QString fileLabel() const;
 
-    virtual void resize(int sz, bool reinit = true);
+    virtual bool resize(int sz, bool reinit = true);
 
-    virtual void SetNewAndShift(int inNew, int inShift);
+    virtual void setNewAndShift(int inNew, int inShift);
 
     /** Clear out the vector */
     void zero();
 
     /* Generally you don't need to call this */
-    void UpdateScalars();
+    void updateScalars();
 
   public:
     /** Save vector information */
@@ -128,6 +125,9 @@ class KstVector : public KstObject {
     /** Return a pointer to the raw vector */
     double *const value() const;
 
+    /** access functions for _isScalarList */
+    bool isScalarList() const { return _isScalarList; }
+    
   protected: // Protected attributes
     /** current number of samples */
     int _size;
@@ -155,7 +155,11 @@ class KstVector : public KstObject {
     QDict<KstScalar> _scalars;
 
     /** is the vector monotonically rising */
-    bool _is_rising;
+    bool _is_rising : 1;
+
+    /** if true then we have a scalar list and do not want to be able to
+    use it in a curve, display statistics for it, etc. */
+    bool _isScalarList : 1;
 
     /** Scalar Maintenance methods */
     void CreateScalars();
