@@ -18,52 +18,54 @@
 #ifndef KSTVECTORDIALOGI_H
 #define KSTVECTORDIALOGI_H
 
-#include <kcompletion.h>
 #include "kstrvector.h"
 #include "vectordialog.h"
 
+class KCompletion;
+
 class KstVectorDialogI : public KstVectorDialog {
   Q_OBJECT
- public:
-  KstVectorDialogI(QWidget* parent = 0, const char* name = 0,
-                   bool modal = FALSE, WFlags fl = 0 );
-  virtual ~KstVectorDialogI();
+  public:
+    KstVectorDialogI(QWidget* parent = 0, const char* name = 0,
+        bool modal = false, WFlags fl = 0 );
+    virtual ~KstVectorDialogI();
 
- public slots:
-  /** update no longer does anything */
-  void update();
+  public slots:
+    /** update no longer does anything */
+    void update();
 
-  bool new_I();
-  bool edit_I();
+    bool new_I();
+    bool edit_I();
 
-  static KstVectorDialogI *globalInstance();
+    static KstVectorDialogI *globalInstance();
 
-  void updateCompletion();
- signals:
-  void modified();
-  void vectorCreated(KstVectorPtr v);
+    void updateCompletion();
 
- private:
-  static KstVectorDialogI *_inst;
-  KCompletion *_fieldCompletion;
-  KstRVectorPtr _getPtr(const QString &tagin);
+  signals:
+    void modified();
+    void vectorCreated(KstVectorPtr v);
 
-  KstRVectorPtr DP;
-  bool _newDialog;
+  private:
+    static KstVectorDialogI *_inst;
+    QGuardedPtr<KCompletion> _fieldCompletion;
+    KstRVectorPtr _getPtr(const QString &tagin);
 
- /***********************************/
- /** defined in dataobjectdialog.h **/
- public slots:
-  void show_Edit(const QString &field);
-  void show_New();
-  void OK();
-  void Init();
-  void close();
-  void reject();
- private:
-  void _fillFieldsForEdit();
-  void _fillFieldsForNew();
+    KstRVectorPtr DP;
+    bool _newDialog;
 
+    /***********************************/
+    /** defined in dataobjectdialog.h **/
+  public slots:
+    void show_Edit(const QString &field);
+    void show_New();
+    void OK();
+    void Init();
+    void close();
+    void reject();
+  private:
+    void _fillFieldsForEdit();
+    void _fillFieldsForNew();
 };
 
 #endif
+// vim: ts=2 sw=2 et
