@@ -2,7 +2,7 @@
                    kstviewobject.cpp: base class for view objects
                              -------------------
     begin                : Mar 11, 2004
-    copyright            : (C) 2003 The University of Toronto
+    copyright            : (C) 2004 The University of Toronto
     email                :
  ***************************************************************************/
 
@@ -37,6 +37,7 @@
 #include "kst.h"
 #include "kstdoc.h"
 #include "kstplotgroup.h"
+#include "kstsettings.h"
 #include "kstviewobject.h"
 #include "kstviewobjectfactory.h"
 #include "kstviewwindow.h"
@@ -44,16 +45,20 @@
 KstViewObject::KstViewObject(const QString& type) : KstObject(), _geom(0, 0, 1, 1), _type(type) {
   _standardActions = 0;
   _layoutActions = 0;
-  _bZoomed = false;
+  _maximized = false;
   updateAspect();
   _onGrid = true;
   _columns = 0;
   _focus = false;
   _selected = false;
+  _foregroundColor = KstSettings::globalSettings()->foregroundColor;
+  _backgroundColor = KstSettings::globalSettings()->backgroundColor;
 }
 
 
 KstViewObject::KstViewObject(QDomElement& e) : KstObject() {
+  _foregroundColor = KstSettings::globalSettings()->foregroundColor;
+  _backgroundColor = KstSettings::globalSettings()->backgroundColor;
   load(e);
 }
 
@@ -62,7 +67,7 @@ void KstViewObject::load(QDomElement& e) {
   _children.clear();
   _standardActions = 0;
   _layoutActions = 0;
-  _bZoomed = false;
+  _maximized = false;
   _onGrid = true;
   _columns = 0;
   _focus = false;
@@ -76,6 +81,9 @@ void KstViewObject::load(QDomElement& e) {
     if (!el.isNull()) { // the node was really an element.
       if (el.tagName() == "tag") {
         setTagName(el.text());
+      } else if (el.tagName() == "columns") {
+        _onGrid = true;
+        _columns = el.text().toInt();
       } else if (el.tagName() == "aspect") {
         _aspect.x = el.attribute("x", "0.0").toDouble();
         _aspect.y = el.attribute("y", "0.0").toDouble();
@@ -93,40 +101,16 @@ KstViewObject::~KstViewObject() {
 }
 
 
-#if 0
-KstViewWidget *KstViewObject::viewWidget() {
-  KstViewObjectPtr viewObject;
-  KstTopLevelViewPtr pTLV;
-  KstViewWidget *pWidget = 0L;
-
-  viewObject = this;
-  while (viewObject.data()) {
-    pTLV = dynamic_cast<KstTopLevelView*>(viewObject.data());
-    if (pTLV) {
-      pWidget = dynamic_cast<KstViewWidget*>(pTLV->widget());
-
-      break;
-    }
-    viewObject = viewObject->_parent;
-  }
-
-  return pWidget;
-}
-#endif
-
-
 void KstViewObject::loadChildren(QDomElement& e) {
-  QString in_tag;
   QDomNode n = e.firstChild();
-  Kst2DPlotPtr plot;
 
   while (!n.isNull()) {
     QDomElement el = n.toElement(); // try to convert the node to an element.
     if (!el.isNull()) { // the node was really an element.
       if (el.tagName() == "plot") {
-        in_tag = el.text();
+        QString in_tag = el.text();
         if (KstApp::inst()->plotHolderWhileOpeningDocument().count(in_tag) > 0) {
-          plot = KstApp::inst()->plotHolderWhileOpeningDocument()[in_tag];
+          Kst2DPlotPtr plot = KstApp::inst()->plotHolderWhileOpeningDocument()[in_tag];
           if (plot) {
             appendChild(plot.data(), true);
             plot->loadChildren(el);
@@ -134,11 +118,9 @@ void KstViewObject::loadChildren(QDomElement& e) {
           }
         }
       } else if (el.tagName() == "plotgroup") {
-        KstPlotGroupPtr pPlotGroup = new KstPlotGroup(el);
-        if (pPlotGroup) {
-          appendChild(pPlotGroup.data(), true);
-          pPlotGroup->loadChildren(el);
-        }
+        KstPlotGroupPtr plotGroup = new KstPlotGroup(el);
+        appendChild(plotGroup.data(), true);
+        plotGroup->loadChildren(el);
       }
     }
     n = n.nextSibling();
@@ -154,7 +136,7 @@ KstObject::UpdateType KstViewObject::update(int) {
 void KstViewObject::save(QTextStream& ts) {
   KstAspectRatio aspect;
 
-  if (_bZoomed) {
+  if (_maximized) {
     aspect = _aspectOldZoomedObject;
   } else {
     aspect = _aspect;
@@ -165,6 +147,9 @@ void KstViewObject::save(QTextStream& ts) {
                "\" y=\"" << aspect.y <<
                "\" w=\"" << aspect.w <<
                "\" h=\"" << aspect.h << "\" />" << endl;
+  if (_onGrid && _columns > 0) { // only if on a grid and we have contents
+    ts << "  <columns>" << _columns << "</columns>" << endl;
+  }
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     (*i)->save(ts);
   }
@@ -174,7 +159,7 @@ void KstViewObject::save(QTextStream& ts) {
 void KstViewObject::saveTag(QTextStream& ts) {
   KstAspectRatio aspect;
 
-  if (_bZoomed) {
+  if (_maximized) {
     aspect = _aspectOldZoomedObject;
   } else {
     aspect = _aspect;
@@ -191,35 +176,33 @@ void KstViewObject::saveTag(QTextStream& ts) {
 }
 
 void KstViewObject::readLock() const {
-  assert(0);
+  abort();
 }
 
 
 void KstViewObject::readUnlock() const {
-  assert(0);
+  abort();
 }
 
 
 void KstViewObject::writeLock() const {
-  assert(0);
+  abort();
 }
 
 
 void KstViewObject::writeUnlock() const {
-  assert(0);
+  abort();
 }
 
 
 void KstViewObject::paint(KstPaintType type, QPainter& p) {
-
   bool hadClipping = p.hasClipping();
   QRegion oldRegion = p.clipRegion();
-  QRegion clipRegion;
 
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     // Compute the clip reg - this can't be stored if we want to keep the code
     // generic, but it's slow, so it's a bit of a conundrum
-    clipRegion = oldRegion;
+    QRegion clipRegion = oldRegion;
     KstViewObjectList::Iterator j = i;
     for (++j; j != _children.end(); ++j) {
       clipRegion -= QRegion((*j)->geometry());
@@ -232,10 +215,12 @@ void KstViewObject::paint(KstPaintType type, QPainter& p) {
   p.setClipRegion(oldRegion);
 
   if (_focus) {
+    //kdDebug() << "Object " << tagName() << " has focus" << endl;
     drawFocusRect(p);
   }
 
   if (isSelected()) {
+    //kdDebug() << "Object " << tagName() << " is selected" << endl;
     drawSelectRect(p);
   }
 
@@ -244,7 +229,7 @@ void KstViewObject::paint(KstPaintType type, QPainter& p) {
 
 
 void KstViewObject::drawFocusRect(QPainter& p) {
-  QPen pen(_backgroundColor.dark(), 2); // FIXME: better color choice
+  QPen pen(Qt::black, 2);
   p.setPen(pen);
   QRect r;
   r.setX(_geom.left());
@@ -260,7 +245,7 @@ void KstViewObject::drawFocusRect(QPainter& p) {
 
 
 void KstViewObject::drawSelectRect(QPainter& p) {
-  QPen pen(_backgroundColor.dark(), 2); // FIXME: better color choice
+  QPen pen(Qt::black, 2);
   p.setPen(pen);
   QRect r;
   r.setX(_geom.left() + 2);
@@ -353,6 +338,21 @@ void KstViewObject::resize(const QSize& size) {
   }
 }
 
+void KstViewObject::resizeForPrint(const QSize& size) {
+  _geomOld = _geom;
+  _geom.setSize(size);
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    (*i)->parentResizedForPrint();
+  }
+}
+
+void KstViewObject::revertForPrint() {
+  _geom = _geomOld;
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    (*i)->parentRevertedForPrint();
+  }
+}
+
 void KstViewObject::resizeFromAspect(double x, double y, double w, double h) {
   _aspect.x = x;
   _aspect.y = y;
@@ -377,23 +377,33 @@ void KstViewObject::parentResized() {
   }
 }
 
+void KstViewObject::parentResizedForPrint() {
+  _geomOld = _geom;
+  updateFromAspect();
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    (*i)->parentResizedForPrint();
+  }
+}
+
+void KstViewObject::parentRevertedForPrint() {
+  _geom = _geomOld;
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    (*i)->parentRevertedForPrint();
+  }
+}
 
 void KstViewObject::setBackgroundColor(const QColor& color) {
   _backgroundColor = color;
 }
 
-
 QColor KstViewObject::backgroundColor() const {
   return _backgroundColor;
 }
 
-
-
 void KstViewObject::setForegroundColor(const QColor& color) {
   _foregroundColor = color;
 }
 
-
 QColor KstViewObject::foregroundColor() const {
   return _foregroundColor;
 }
@@ -580,7 +590,7 @@ bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObject
 
   if (_standardActions & Zoom) {
     id = menu->insertItem(i18n("Maximi&ze"), this, SLOT(zoomToggle()));
-    if (_bZoomed) {
+    if (_maximized) {
       menu->setItemChecked(id,true);
     }
     rc = true;
@@ -601,7 +611,6 @@ bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObject
 bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topParent) {
   Q_UNUSED(pos)
 
-  bool bHasEntry = false;
   bool rc = false;
   int id;
 
@@ -648,37 +657,33 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
 
   if (_layoutActions & MoveTo) {
     int i = 0;
+    bool hasEntry = false;
 
     KPopupMenu *submenu = new KPopupMenu(menu);
-    KstTopLevelView *pTLV;
 
     id = menu->insertItem(i18n("&Move To"), submenu);
 
     KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
     while (it->currentItem()) {
       KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
-      pTLV = dynamic_cast<KstTopLevelView*>(topParent.data());
+      KstTopLevelViewPtr tlv = dynamic_cast<KstTopLevelView*>(topParent.data());
 
-      if (c) {
-        if (!pTLV || pTLV != c->view()) {
-          bHasEntry = true;
-          submenu->insertItem(it->currentItem()->caption(), i);
-          submenu->connectItem(i, this, SLOT(moveTo(int)));
-          _moveToMap[i] = it->currentItem()->caption();
-          i++;
-        }
+      if (c && (!tlv || tlv != c->view())) {
+        hasEntry = true;
+        submenu->insertItem(it->currentItem()->caption(), i);
+        submenu->connectItem(i, this, SLOT(moveTo(int)));
+        _moveToMap[i] = it->currentItem()->caption();
+        i++;
       }
       it->next();
     }
     KstApp::inst()->deleteIterator(it);
 
-    menu->setItemEnabled(id, bHasEntry);
+    menu->setItemEnabled(id, hasEntry);
 
     rc = true;
   }
 
-  // FIXME: implement the rest of the actions
-
   return rc;
 }
 
@@ -689,6 +694,7 @@ void KstViewObject::deleteObject() {
   if (_parent) {
     _parent->removeChild(this);
   }
+  QTimer::singleShot(0, KstApp::inst(), SLOT(updateDialogs()));
 }
 
 
@@ -847,12 +853,12 @@ void KstViewObject::setSelected(bool selected) {
 
 
 void KstViewObject::zoom(bool zoom) {
-  if (_bZoomed && !zoom) {
-    _bZoomed = false;
+  if (_maximized && !zoom) {
+    _maximized = false;
     _aspect = _aspectOldZoomedObject;
     updateFromAspect();
-  } else if (!_bZoomed && zoom) {
-    _bZoomed = true;
+  } else if (!_maximized && zoom) {
+    _maximized = true;
     _aspectOldZoomedObject = _aspect;
     resizeFromAspect(0.0, 0.0, 1.0, 1.0);
     raiseToTop();
@@ -861,17 +867,17 @@ void KstViewObject::zoom(bool zoom) {
 
 
 void KstViewObject::zoomToggle() {
-  if (_bZoomed) {
-    _bZoomed = false;
+  if (_maximized) {
+    _maximized = false;
     _aspect = _aspectOldZoomedObject;
-    if (_parent && _parent->_bZoomed) {
+    if (_parent && _parent->_maximized) {
       _parent->zoomToggle();
     }
     updateFromAspect();
   } else {
-    _bZoomed = true;
+    _maximized = true;
     _aspectOldZoomedObject = _aspect;
-    if (_parent && !_parent->_bZoomed) {
+    if (_parent && !_parent->_maximized) {
       _parent->zoomToggle();
     }
     resizeFromAspect(0.0, 0.0, 1.0, 1.0);
