@@ -17,49 +17,48 @@
 
 #include <qlabel.h>
 #include <qlayout.h>
+#include <qmessagebox.h>
 #include <qpushbutton.h>
 #include <qstring.h>
 #include <qtable.h>
 #include <qwidget.h>
-#include <qmessagebox.h>
+
 #include <kdebug.h>
 #include <klocale.h>
 
-#include "vectorselector.h"
-#include "kstvectortable.h"
-#include "kstrvector.h"
 #include "kstdatacollection.h"
+#include "kstrvector.h"
+#include "kstvectortable.h"
 #include "kstviewvectorsdialog_i.h"
+#include "vectorselector.h"
 
 KstViewVectorsDialogI::KstViewVectorsDialogI(QWidget* parent,
                                              const char* name,
                                              bool modal,
-                                             WFlags fl) :
-KstViewVectorsDialog(parent, name, modal, fl) {
+                                             WFlags fl)
+: KstViewVectorsDialog(parent, name, modal, fl) {
   
-  tableVectors = new KstVectorTable( this, "tableVectors" );
-  tableVectors->setNumRows( 0 );
-  tableVectors->setNumCols( 1 );
-  tableVectors->setReadOnly( true );
-  tableVectors->setSorting( false );
-  tableVectors->setSelectionMode( QTable::Single );
-  layout2->insertWidget( 1, tableVectors );
+  tableVectors = new KstVectorTable(this, "tableVectors");
+  tableVectors->setNumRows(0);
+  tableVectors->setNumCols(1);
+  tableVectors->setReadOnly(true);
+  tableVectors->setSorting(false);
+  tableVectors->setSelectionMode(QTable::Single);
+  layout2->insertWidget(1, tableVectors);
 
-  connect( Cancel, SIGNAL( clicked() ), this, SLOT( close() ) );
-  connect( vectorSelector, SIGNAL( selectionChanged(const QString&) ), this, SLOT( vectorChanged(const QString&) ) );
-  connect( vectorSelector, SIGNAL( newVectorCreated(const QString&) ), this, SLOT( vectorChanged(const QString&) ) );
+  connect(Cancel, SIGNAL(clicked()), this, SLOT(close()));
+  connect(vectorSelector, SIGNAL(selectionChanged(const QString&)), this, SLOT( vectorChanged(const QString&)));
+  connect(vectorSelector, SIGNAL(newVectorCreated(const QString&)), this, SLOT(vectorChanged(const QString&)));
   
-  if( tableVectors->numCols( ) != 1 ) {  
-    int i;
-    
-    for( i=0; i<tableVectors->numCols( );  ) {
-      tableVectors->removeColumn( 0 );
+  if (tableVectors->numCols() != 1) {  
+    for (; 0 < tableVectors->numCols(); ) {
+      tableVectors->removeColumn(0);
     }
-    tableVectors->insertColumns( 0, 1 );
+    tableVectors->insertColumns(0, 1);
   }
   
-  tableVectors->setReadOnly( true );
-  languageChange( );
+  tableVectors->setReadOnly(true);
+  languageChange();
 }
 
 KstViewVectorsDialogI::~KstViewVectorsDialogI() {
@@ -68,22 +67,23 @@ KstViewVectorsDialogI::~KstViewVectorsDialogI() {
 void KstViewVectorsDialogI::updateViewVectorsDialog() {
   vectorSelector->update();
   
-  QString strVector = vectorSelector->selectedVector();
-  tableVectors->setVector(strVector);
-  
-  updateViewVectorsDialog(strVector);
+  QString vector = vectorSelector->selectedVector();
+  tableVectors->setVector(vector);
+  updateViewVectorsDialog(vector);
 }
 
-void KstViewVectorsDialogI::updateViewVectorsDialog(const QString& strVector) {
+void KstViewVectorsDialogI::updateViewVectorsDialog(const QString& vectorName) {
   int needed = 0;
-  KstVectorPtr vector = *KST::vectorList.findTag(strVector);
+  KstVectorPtr vector = *KST::vectorList.findTag(vectorName);
   if (vector) {
+    vector->readLock();
     needed = vector->length();
+    vector->readUnlock();
   }
   
   tableVectors->setNumRows(needed);
   QRect rect = tableVectors->horizontalHeader()->rect();
-  tableVectors->setColumnWidth( 0, rect.width() );
+  tableVectors->setColumnWidth(0, rect.width());
 }
 
 void KstViewVectorsDialogI::showViewVectorsDialog() {
@@ -94,9 +94,9 @@ void KstViewVectorsDialogI::showViewVectorsDialog() {
   raise();
 }
 
-void KstViewVectorsDialogI::vectorChanged(const QString& strVector) {
-  updateViewVectorsDialog(strVector);
-  tableVectors->setVector(strVector);
+void KstViewVectorsDialogI::vectorChanged(const QString& vector) {
+  updateViewVectorsDialog(vector);
+  tableVectors->setVector(vector);
   tableVectors->update();
 }
 
@@ -106,14 +106,14 @@ void KstViewVectorsDialogI::vectorChanged(const QString& strVector) {
  */
 void KstViewVectorsDialogI::languageChange()
 {
-  setCaption( i18n( "View Vector Values" ) );
-  tableVectors->horizontalHeader()->setLabel( 0, i18n("Values") );
+  setCaption(i18n("View Vector Values"));
+  tableVectors->horizontalHeader()->setLabel(0, i18n("Values"));
 
   KstViewVectorsDialog::languageChange();
 }
 
 void KstViewVectorsDialogI::updateDefaults(int index) {
-  Q_UNUSED( index )
+  Q_UNUSED(index)
 }
 
 #include "kstviewvectorsdialog_i.moc"
