@@ -30,13 +30,15 @@
 #include "kst2dplot.h"
 #include "kstcolorsequence.h"
 #include "kstdatacollection.h"
+#include "kstdefaultnames.h"
+#include "kstdoc.h"
 #include "kstequationcurve.h"
 #include "ksthistogram.h"
 #include "kstlegend.h"
 #include "kstpsdcurve.h"
+#include "ksttoplevelview.h"
 #include "kstvcurve.h"
 #include "kstversion.h"
-#include "ksttoplevelview.h"
 #include "plugincollection.h"
 
 
@@ -236,7 +238,7 @@ KstRVector *GetOrCreateVector(QString field, KstDataSourcePtr file, struct InTyp
 
   for (i_v=0; i_v<n_v; i_v++) {
     V = *vectorList.at(i_v);
-    if (V->getField() == field) {
+    if (V->field() == field) {
       if (V->filename() == file->fileName()) {
         return (V);
       }
@@ -248,7 +250,7 @@ KstRVector *GetOrCreateVector(QString field, KstDataSourcePtr file, struct InTyp
                           + "-" + field,
                           in.f, in.n, in.skip, in.doskip, in.doave);
   if (!V->isValid()) {
-    kdError() << I18N_NOOP("Error: Invalid field: ") << V->getField()
+    kdError() << I18N_NOOP("Error: Invalid field: ") << V->field()
               << endl << I18N_NOOP("File: ") << V->filename() << endl;
     exit(0); // fixme: this can cause crashes
   }
@@ -337,13 +339,13 @@ int main(int argc, char *argv[]) {
 
     /* temp variables: these all get stuck into list objects */
     KstDataSourcePtr file;
-    KstRVector *xvector=NULL;
-    KstRVector *yvector;
-    KstRVector *evector;
-    KstVCurve *curve;
-    KstPSDCurve *psdcurve;
-    KstEquationCurve *eqcurve;
-    KstHistogram *hscurve;
+    KstRVectorPtr xvector;
+    KstRVectorPtr yvector;
+    KstRVectorPtr evector;
+    KstVCurvePtr curve;
+    KstPSDCurvePtr psdcurve;
+    KstEquationCurvePtr eqcurve;
+    KstHistogramPtr hscurve;
     Kst2DPlotPtr plot;
 
     int n_y, n_eq=0;
@@ -407,12 +409,12 @@ int main(int argc, char *argv[]) {
                   new KstEquationCurve(QString("E")+QString::number(n_eq+1)+
                                        "-" + eqS,
                                        eqS,
-                                       min,max,n, KstColorSequence::next());
+                                       min, max, n, KstColorSequence::next());
 
                 KST::dataObjectList.lock().writeLock();
-                KST::dataObjectList.append(eqcurve);
+                KST::dataObjectList.append(eqcurve.data());
                 KST::dataObjectList.lock().writeUnlock();
-                plot->addCurve(eqcurve);
+                plot->addCurve(eqcurve.data());
 
                 if (in.sep_plots) {
                   i_plot++;
@@ -436,7 +438,7 @@ int main(int argc, char *argv[]) {
           exit(0);
         }
 
-        if (!file->isValid() || file->frameCount() < 1) {
+        if (!file->isValid() || file->isEmpty()) {
           std::cerr << I18N_NOOP("No data in file: ")
             << args->arg(i_file)
             << ".  Trying to continue...\n";
@@ -458,7 +460,7 @@ int main(int argc, char *argv[]) {
             /* make the curves */
             color = KstColorSequence::next();
             curve = new KstVCurve(QString("C") + QString::number(1+i_curve++)
-                                  + "-" + yvector->getField(),
+                                  + "-" + yvector->field(),
                                   KstVectorPtr(xvector), KstVectorPtr(yvector),
                                   0L, 0L, color);
             if (in.has_points) {
@@ -472,9 +474,9 @@ int main(int argc, char *argv[]) {
             }
 
             KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(curve);
+            KST::dataObjectList.append(curve.data());
             KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(curve);
+            plot->addCurve(curve.data());
 
             if (in.sep_plots) {
               i_plot++;
@@ -511,9 +513,9 @@ int main(int argc, char *argv[]) {
                     true, KstColorSequence::next());
             }
             KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(eqcurve);
+            KST::dataObjectList.append(eqcurve.data());
             KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(eqcurve);
+            plot->addCurve(eqcurve.data());
 
             if (in.sep_plots) {
               i_plot++;
@@ -532,7 +534,7 @@ int main(int argc, char *argv[]) {
             color = KstColorSequence::next();
 
             psdcurve = new KstPSDCurve(
-                QString("P") + QString::number(1+i_curve++) + "-" + yvector->getField(),
+                QString("P") + QString::number(1+i_curve++) + "-" + yvector->field(),
                 KstVectorPtr(yvector), in.rate, true, in.len,
                 true, true, in.VUnits, in.RUnits, color);
             if (in.has_points) {
@@ -540,9 +542,9 @@ int main(int argc, char *argv[]) {
               psdcurve->setHasLines(false);
             }
             KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(psdcurve);
+            KST::dataObjectList.append(psdcurve.data());
             KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(psdcurve);
+            plot->addCurve(psdcurve.data());
 
             if (in.sep_plots) {
               i_plot++;
@@ -566,14 +568,14 @@ int main(int argc, char *argv[]) {
 
             hscurve = new KstHistogram(QString("H") +
                 QString::number(1+i_curve++)
-                + "-" + yvector->getField(),
+                + "-" + yvector->field(),
                 KstVectorPtr(yvector), min, max, N,
                 KST_HS_NUMBER,
                 color);
             KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(KstDataObjectPtr(hscurve));
+            KST::dataObjectList.append(hscurve.data());
             KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(hscurve);
+            plot->addCurve(hscurve.data());
 
             if (in.sep_plots) {
               i_plot++;
@@ -609,20 +611,18 @@ int main(int argc, char *argv[]) {
       //kst->openDocumentFile();
     }
 
-    QString printfile;
-    printfile = args->getOption("print");
-    QString pngfile;
-    pngfile = args->getOption("png");
+    QString printfile = args->getOption("print");
+    QString pngfile = args->getOption("png");
     bool print_and_exit = false;
 
-    if (printfile!="<none>") {
+    if (printfile != "<none>") {
       args->clear();
       kst->forceUpdate();
       kst->immediatePrintToFile(printfile);
       print_and_exit = true;
     }
 
-    if (pngfile!="<none>") {
+    if (pngfile != "<none>") {
       args->clear();
       kst->forceUpdate();
       kst->immediatePrintToPng(pngfile);
@@ -637,6 +637,7 @@ int main(int argc, char *argv[]) {
       app.setMainWidget(kst);
       kst->show();
       kst->updateDialogs();
+      kst->document()->setModified(false);
     }
 
     // LEAVE THIS HERE - causes crashes otherwise!
