@@ -19,6 +19,7 @@
 #include "plugin.h"
 
 #include <klibloader.h>
+#include <klocale.h>
 #include <kdebug.h>
 
 
@@ -27,12 +28,14 @@ const int Plugin::CallError = -424242;
 Plugin::Plugin() : KstShared() {
   _lib = 0L;
   _symbol = 0L;
+  _parameterName = 0L;
   //kdDebug() << "Creating Plugin: " << long(this) << endl;
 }
 
 
 Plugin::~Plugin() {
   _symbol = 0L;
+  _parameterName = 0L;
 
   if (_lib) {
     _lib->unload();  // this deletes it too
@@ -56,7 +59,7 @@ const QString& Plugin::soFile() const {
 int Plugin::call(const double *const inArrays[], const int inArrayLens[],
                  const double inScalars[], double *outArrays[],
                  int outArrayLens[], double outScalars[], void** local) const {
-  if (!_symbol || _data._filter) {
+  if (!_symbol) {
     return CallError;
   }
 
@@ -70,7 +73,7 @@ int Plugin::call(const double *const inArrays[], const int inArrayLens[],
 int Plugin::call(const double *const inArrays[], const int inArrayLens[],
                  const double inScalars[], double *outArrays[],
                  int outArrayLens[], double outScalars[]) const {
-  if (!_symbol || _data._filter) {
+  if (!_symbol) {
     return CallError;
   }
 
@@ -81,25 +84,22 @@ int Plugin::call(const double *const inArrays[], const int inArrayLens[],
 }
 
 
-int Plugin::filter(const double *const inArray, int inArrayLen,
-                 const double inScalars[], double *outArray[],
-                 int *outArrayLen) const {
-  if (!_symbol || !_data._filter) {
-    return CallError;
+QString Plugin::parameterName(int iIndex) {
+  QString strParameter;
+  char* pName = 0L;
+  
+  if (_data._isFit && _parameterName) {
+    if (((int(*)(int, char**))_parameterName)(iIndex, &pName) && pName) {
+      strParameter = pName; // deep copy into QString
+      free(pName);
+    }
   }
 
-  return ((int(*)(const double *const, int, const double[], double*[], int*))_symbol) (inArray, inArrayLen, inScalars, outArray, outArrayLen);
-}
-
-
-int Plugin::filter(const double *const inArray, int inArrayLen,
-                 const double inScalars[], double *outArray[],
-                 int *outArrayLen, void **local) const {
-  if (!_symbol || !_data._filter) {
-    return CallError;
+  if (strParameter.isEmpty()) {
+    strParameter = i18n("Param%1").arg(iIndex);
   }
-
-  return ((int(*)(const double *const, int, const double[], double*[], int*, void**))_symbol) (inArray, inArrayLen, inScalars, outArray, outArrayLen, local);
+  
+  return strParameter;
 }
 
 
@@ -108,14 +108,16 @@ const Plugin::Data& Plugin::data() const {
 }
 
 
-
 void Plugin::Data::clear() {
-  _filter = false;
+  _isFilter = false;
   _localdata = false;
   _name = QString::null;
+  _readableName = QString::null;
   _author = QString::null;
   _description = QString::null;
   _version = QString::null;
+  _filterInputVector = QString::null;
+  _filterOutputVector = QString::null;
   _state = Unknown;
   _isFit = false;
   _isFitWeighted = false;
