@@ -47,19 +47,13 @@ public:
            const double inScalars[],
            double *outArrays[], int outArrayLens[],
            double outScalars[], void **local) const;
-  int filter(const double *const inArray, int inArrayLen,
-             const double inScalars[], double *outArray[],
-             int *outArrayLen) const;
-  int filter(const double *const inArray, int inArrayLen,
-             const double inScalars[], double *outArray[],
-             int *outArrayLen, void **local) const;
-
+  QString parameterName(int iIndex);
 
   static const int CallError;
 
   class Data {
     public:
-      Data() : _filter(false), _localdata(false) {}
+      Data() : _isFilter(false), _localdata(false), _isFit(false), _isFitWeighted(false) {}
 
       /**
        *   Clear all values.
@@ -79,13 +73,14 @@ public:
             StringType,
             MapType,
             IntegerType,
-            FloatType
+            FloatType,
           } ValueType;
           typedef enum { UnknownSubType,
             AnySubType,
             FloatSubType,
             StringSubType,
-            IntegerSubType
+            IntegerSubType,
+            FloatNonVectorSubType,
           } ValueSubType;
 
           QString _name;
@@ -95,15 +90,17 @@ public:
       };
 
       // Intro
-      bool _filter;
-      bool _localdata;
-      bool _isFit;
-      bool _isFitWeighted;
+      bool _isFilter : 1;
+      bool _localdata : 1;
+      bool _isFit : 1;
+      bool _isFitWeighted : 1;
       QString _name;
       QString _readableName;
       QString _author;
       QString _description;
       QString _version;
+      QString _filterInputVector;
+      QString _filterOutputVector;
       PluginState _state;
       // Unimplemented:  platforms, language, documentation
 
@@ -124,7 +121,8 @@ protected:
   KLibrary *_lib;
 
   void *_symbol;
-
+  void *_parameterName;
+  
   // Related files
   QString _xmlFile, _soFile;
 };
