@@ -26,12 +26,12 @@
 #include <kglobal.h>
 #include <klibloader.h>
 #include <klocale.h>
+#include <kstandarddirs.h>
 #include <kstaticdeleter.h>
-#include <kstddirs.h>
 
-#include <qregexp.h>
 #include <qdir.h>
 #include <qfileinfo.h>
+#include <qregexp.h>
 
 PluginCollection *PluginCollection::_self = 0L;
 static KStaticDeleter<PluginCollection> _pcSelf;
