@@ -37,7 +37,7 @@
 
 KstStdinSource::KstStdinSource()
 : KstDataSource("stdin", "stdin") {
-  _file = new KTempFile();
+  _file = new KTempFile;
   _filename = _file->name();
   update();
   _src = KstDataSource::loadSource(_filename, "ASCII");
@@ -117,7 +117,7 @@ bool KstStdinSource::isValidField(const QString& field) const {
 }
 
 
-int KstStdinSource::samplesPerFrame(const QString &field) {
+int KstStdinSource::samplesPerFrame(const QString& field) {
   if (isValid()) {
     return _src->samplesPerFrame(field);
   }
@@ -125,9 +125,9 @@ int KstStdinSource::samplesPerFrame(const QString &field) {
 }
 
 
-int KstStdinSource::frameCount() const {
+int KstStdinSource::frameCount(const QString& field) const {
   if (isValid()) {
-    return _src->frameCount();
+    return _src->frameCount(field);
   }
   return 0;
 }
@@ -150,8 +150,12 @@ void KstStdinSource::save(QTextStream &ts) {
 
 
 bool KstStdinSource::isValid() const {
-    return _valid && _src && _src->isValid();
+  return _valid && _src && _src->isValid();
 }
 
 
+bool KstStdinSource::isEmpty() const {
+  return !isValid() || _src->isEmpty();
+}
+
 // vim: ts=2 sw=2 et
