@@ -5,7 +5,11 @@
 VALGRIND=
 
 if [ "x$1" == "x-v" ]; then
-	VALGRIND="valgrind -v --num-callers=15";
+	VALGRIND="valgrind -v --num-callers=25";
+fi
+
+if [ "x$1" == "x-t" ]; then
+	VALGRIND="valgrind -v --skin=helgrind --num-callers=25";
 fi
 
 
@@ -26,7 +30,7 @@ doExit() {
 	$QUIT
 	rm $TMPFILE
 	rm $TESTFILE
-	rm $KSTFILE
+	rm -f $KSTFILE
 	exit $1;
 }
 
@@ -441,20 +445,23 @@ checkStringResponse 612 "true"
 # 613
 $DCOP newFile >$TMPFILE
 
+$DCOP curveList >$TMPFILE
+checkEmptyResponse 614
+
 # crashes Kst generally, bypassed with #599
 $DCOP open $KSTFILE >$TMPFILE
-checkStringResponse 614 "true"
+checkStringResponse 620 "true"
 
-rm $KSTFILE
+rm -f $KSTFILE
 
 $DCOP save >$TMPFILE
-checkStringResponse 615 "true"
+checkStringResponse 621 "true"
 
 $DCOP open $KSTFILE >$TMPFILE
-checkStringResponse 616 "true"
+checkStringResponse 622 "true"
 
 $DCOP save >$TMPFILE
-checkStringResponse 617 "true"
+checkStringResponse 623 "true"
 
 echo ""
 doExit 0
